@@ -117,13 +117,12 @@ static char **linephonec_readline_completion(const char *text,
 #endif
 
 /* These are callback for linphone core */
-static void linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm,
-	const char *username);
+static void linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm, const char *username, const char *domain);
 static void linphonec_display_refer (LinphoneCore * lc, const char *refer_to);
 static void linphonec_display_something (LinphoneCore * lc, const char *something);
 static void linphonec_display_url (LinphoneCore * lc, const char *something, const char *url);
 static void linphonec_display_warning (LinphoneCore * lc, const char *something);
-static void linphonec_notify_received(LinphoneCore *lc, LinphoneCall *call, const char *from,const char *event);
+static void linphonec_transfer_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState new_call_state);
 
 static void linphonec_notify_presence_received(LinphoneCore *lc,LinphoneFriend *fid);
 static void linphonec_new_unknown_subscriber(LinphoneCore *lc,
@@ -149,6 +148,7 @@ static char last_in_history[256];
 #endif
 //auto answer (-a) option
 static bool_t auto_answer=FALSE;
+static bool_t real_early_media_sending=FALSE;
 static bool_t answer_call=FALSE;
 static bool_t vcap_enabled=FALSE;
 static bool_t display_enabled=FALSE;
@@ -161,6 +161,7 @@ static int trace_level = 0;
 static char *logfile_name = NULL;
 static char configfile_name[PATH_MAX];
 static char zrtpsecrets[PATH_MAX];
+static char usr_certificates_path[PATH_MAX];
 static const char *factory_configfile_name=NULL;
 static char *sip_addr_to_call = NULL; /* for autocall */
 static int window_id = 0; /* 0=standalone window, or window id for embedding video */
@@ -256,7 +257,7 @@ linphonec_display_url (LinphoneCore * lc, const char *something, const char *url
  * Linphone core callback
  */
 static void
-linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm, const char *username)
+linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm, const char *username, const char *domain)
 {
 	/* no prompt possible when using pipes or tcp mode*/
 	if (unix_socket){
@@ -272,7 +273,7 @@ linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm, const char *usern
 			return;
 		}
 
-		pending_auth=linphone_auth_info_new(username,NULL,NULL,NULL,realm);
+		pending_auth=linphone_auth_info_new(username,NULL,NULL,NULL,realm,domain);
 		auth_stack.elem[auth_stack.nitems++]=pending_auth;
 	}
 }
@@ -281,13 +282,14 @@ linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm, const char *usern
  * Linphone core callback
  */
 static void
-linphonec_notify_received(LinphoneCore *lc, LinphoneCall *call, const char *from,const char *event)
+linphonec_transfer_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState new_call_state)
 {
-	if(!strcmp(event,"refer"))
-	{
-		linphonec_out("The distand endpoint %s of call %li has been transfered, you can safely close the call.\n",
-		              from,(long)linphone_call_get_user_pointer (call));
+	char *remote=linphone_call_get_remote_address_as_string(call);
+	if (new_call_state==LinphoneCallConnected){
+		linphonec_out("The distant endpoint %s of call %li has been transfered, you can safely close the call.\n",
+		              remote,(long)linphone_call_get_user_pointer (call));
 	}
+	ms_free(remote);
 }
 
 
@@ -362,10 +364,17 @@ static void linphonec_call_state_changed(LinphoneCore *lc, LinphoneCall *call, L
 			linphone_call_enable_camera (call,linphonec_camera_enabled);
 			id=(long)linphone_call_get_user_pointer (call);
 			linphonec_set_caller(from);
+			linphonec_out("Receiving new incoming call from %s, assigned id %i\n", from,id);
 			if ( auto_answer)  {
 				answer_call=TRUE;
+			} else if (real_early_media_sending) {
+				LinphoneCallParams* callparams = linphone_core_create_default_call_parameters(lc);
+				linphonec_out("Sending early media using real hardware\n");
+				linphone_call_params_enable_early_media_sending(callparams, TRUE);
+				if (vcap_enabled) linphone_call_params_enable_video(callparams, TRUE);
+				linphone_core_accept_early_media_with_params(lc, call, callparams);
+				linphone_call_params_destroy(callparams);
 			}
-			linphonec_out("Receiving new incoming call from %s, assigned id %i\n", from,id);
 		break;
 		case LinphoneCallOutgoingInit:
 			linphonec_call_identify(call);
@@ -492,7 +501,6 @@ static void *pipe_thread(void*p){
 }
 
 static void start_pipe_reader(void){
-	ms_mutex_init(&prompt_mutex,NULL);
 	pipe_reader_run=TRUE;
 	ortp_thread_create(&pipe_reader_th,NULL,pipe_thread,NULL);
 }
@@ -528,6 +536,7 @@ char *linphonec_readline(char *prompt){
 		fprintf(stdout,"%s",prompt);
 		fflush(stdout);
 		while(1){
+
 			ms_mutex_lock(&prompt_mutex);
 			if (have_prompt){
 				char *ret=strdup(received_prompt);
@@ -538,15 +547,17 @@ char *linphonec_readline(char *prompt){
 			ms_mutex_unlock(&prompt_mutex);
 			linphonec_idle_call();
 #ifdef WIN32
-			Sleep(20);
-			/* Following is to get the video window going as it
-				 should. Maybe should we only have this on when the option -V
-				 or -D is on? */
-			MSG msg;
-	
-			if (PeekMessage(&msg, NULL, 0, 0,1)) {
-				TranslateMessage(&msg);
-				DispatchMessage(&msg);
+			{
+				MSG msg;
+				Sleep(20);
+				/* Following is to get the video window going as it
+					should. Maybe should we only have this on when the option -V
+					or -D is on? */
+
+				if (PeekMessage(&msg, NULL, 0, 0,1)) {
+					TranslateMessage(&msg);
+					DispatchMessage(&msg);
+				}
 			}
 #else
 			usleep(20000);
@@ -628,8 +639,8 @@ int
 main (int argc, char *argv[]) {
 #endif
 	linphonec_vtable.call_state_changed=linphonec_call_state_changed;
-	linphonec_vtable.notify_presence_recv = linphonec_notify_presence_received;
-	linphonec_vtable.new_subscription_request = linphonec_new_unknown_subscriber;
+	linphonec_vtable.notify_presence_received = linphonec_notify_presence_received;
+	linphonec_vtable.new_subscription_requested = linphonec_new_unknown_subscriber;
 	linphonec_vtable.auth_info_requested = linphonec_prompt_for_auth;
 	linphonec_vtable.display_status = linphonec_display_status;
 	linphonec_vtable.display_message=linphonec_display_something;
@@ -638,9 +649,9 @@ main (int argc, char *argv[]) {
 	linphonec_vtable.text_received=linphonec_text_received;
 	linphonec_vtable.dtmf_received=linphonec_dtmf_received;
 	linphonec_vtable.refer_received=linphonec_display_refer;
-	linphonec_vtable.notify_recv=linphonec_notify_received;
+	linphonec_vtable.transfer_state_changed=linphonec_transfer_state_changed;
 	linphonec_vtable.call_encryption_changed=linphonec_call_encryption_changed;
-	
+
 	if (! linphonec_init(argc, argv) ) exit(EXIT_FAILURE);
 
 	linphonec_main_loop (linphonec);
@@ -663,13 +674,15 @@ linphonec_init(int argc, char **argv)
 	 * Set initial values for global variables
 	 */
 	mylogfile = NULL;
-	
-	
+
+
 #ifndef _WIN32
 	snprintf(configfile_name, PATH_MAX, "%s/.linphonerc",
 			getenv("HOME"));
 	snprintf(zrtpsecrets, PATH_MAX, "%s/.linphone-zidcache",
 			getenv("HOME"));
+	snprintf(usr_certificates_path, PATH_MAX, "%s/.linphone-usr-crt",
+			getenv("HOME"));
 #elif defined(_WIN32_WCE)
 	strncpy(configfile_name,PACKAGE_DIR "\\linphonerc",PATH_MAX);
 	mylogfile=fopen(PACKAGE_DIR "\\" "linphonec.log","w");
@@ -679,6 +692,8 @@ linphonec_init(int argc, char **argv)
 			getenv("APPDATA"));
 	snprintf(zrtpsecrets, PATH_MAX, "%s/Linphone/linphone-zidcache",
 			getenv("APPDATA"));
+	snprintf(usr_certificates_path, PATH_MAX, "%s/Linphone/linphone-usr-crt",
+			getenv("APPDATA"));
 #endif
 	/* Handle configuration filename changes */
 	switch (handle_configfile_migration())
@@ -693,7 +708,6 @@ linphonec_init(int argc, char **argv)
 		default:
 			break;
 	}
-
 #ifdef ENABLE_NLS
 	if (NULL == bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR))
 		perror ("bindtextdomain failed");
@@ -733,11 +747,15 @@ linphonec_init(int argc, char **argv)
 	 * Initialize linphone core
 	 */
 	linphonec=linphone_core_new (&linphonec_vtable, configfile_name, factory_configfile_name, NULL);
+
+	linphone_core_set_user_agent(linphonec,"Linphonec", LINPHONE_VERSION);
 	linphone_core_set_zrtp_secrets_file(linphonec,zrtpsecrets);
-	linphone_core_enable_video(linphonec,vcap_enabled,display_enabled);
-	if (display_enabled && window_id != 0) 
+	linphone_core_set_user_certificates_path(linphonec,usr_certificates_path);
+	linphone_core_enable_video_capture(linphonec, vcap_enabled);
+	linphone_core_enable_video_display(linphonec, display_enabled);
+	if (display_enabled && window_id != 0)
 	{
-		printf ("Setting window_id: 0x%x\n", window_id);
+		printf("Setting window_id: 0x%x\n", window_id);
 		linphone_core_set_native_video_window_id(linphonec,window_id);
 	}
 
@@ -773,7 +791,7 @@ linphonec_finish(int exit_status)
 {
 	// Do not allow concurrent destroying to prevent glibc errors
 	static bool_t terminating=FALSE;
-	if (terminating) return; 
+	if (terminating) return;
 	terminating=TRUE;
 	linphonec_out("Terminating...\n");
 
@@ -792,6 +810,7 @@ linphonec_finish(int exit_status)
 	if (mylogfile != NULL && mylogfile != stdout)
 	{
 		fclose (mylogfile);
+		mylogfile=stdout;
 	}
 	printf("\n");
 	exit(exit_status);
@@ -818,12 +837,13 @@ linphonec_prompt_for_auth_final(LinphoneCore *lc)
 #ifdef HAVE_READLINE
 	rl_hook_func_t *old_event_hook;
 #endif
+	LinphoneAuthInfo *pending_auth;
 
 	if (reentrancy!=0) return 0;
-	
+
 	reentrancy++;
-	
-	LinphoneAuthInfo *pending_auth=auth_stack.elem[auth_stack.nitems-1];
+
+	pending_auth=auth_stack.elem[auth_stack.nitems-1];
 
 	snprintf(auth_prompt, 256, "Password for %s on %s: ",
 		pending_auth->username, pending_auth->realm);
@@ -905,12 +925,14 @@ print_usage (int exit_status)
 "  -l  logfile          specify the log file for your SIP phone\n"
 "  -s  sipaddress       specify the sip call to do at startup\n"
 "  -a                   enable auto answering for incoming calls\n"
+"  --real-early-media   enable sending early media using real audio/video (beware of privacy issue)\n"
 "  -V                   enable video features globally (disabled by default)\n"
 "  -C                   enable video capture only (disabled by default)\n"
 "  -D                   enable video display only (disabled by default)\n"
 "  -S                   show general state messages (disabled by default)\n"
 "  --wid  windowid      force embedding of video window into provided windowid (disabled by default)\n"
-"  -v or --version      display version and exits.\n");
+"  -v or --version      display version and exits.\n"
+);
 
   	exit(exit_status);
 }
@@ -1074,7 +1096,7 @@ linphonec_initialize_readline()
 	rl_attempted_completion_function = linephonec_readline_completion;
 
 	/* printf("Readline initialized.\n"); */
-        setlinebuf(stdout);
+	setlinebuf(stdout);
 	return 0;
 }
 
@@ -1149,7 +1171,6 @@ linphonec_main_loop (LinphoneCore * opm)
 			add_history(iptr);
 		}
 #endif
-
 		linphonec_parse_command_line(linphonec, iptr);
 		linphonec_command_finished();
 		free(input);
@@ -1228,6 +1249,10 @@ linphonec_parse_cmdline(int argc, char **argv)
                 {
                         auto_answer = TRUE;
                 }
+                else if (strncmp ("--real-early-media", argv[arg_num], strlen("--real-early-media")) == 0)
+                {
+                        real_early_media_sending = TRUE;
+                }
 		else if (strncmp ("-C", argv[arg_num], 2) == 0)
                 {
                         vcap_enabled = TRUE;
