@@ -24,8 +24,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "belle-sip/belle-sip.h"
 #include "belle-sip/belle-sdp.h"
 
-
-
 struct Sal{
 	SalCallbacks callbacks;
 	MSList *pending_auths;/*MSList of SalOp */
@@ -33,12 +31,15 @@ struct Sal{
 	belle_sip_provider_t *prov;
 	belle_sip_header_user_agent_t* user_agent;
 	belle_sip_listener_t *listener;
+	void *tunnel_client;
 	void *up; /*user pointer*/
 	int session_expires;
 	unsigned int keep_alive;
 	char *root_ca;
 	char *uuid;
 	int refresher_retry_after; /*retry after value for refresher*/
+	MSList *supported_tags;/*list of char * */
+	belle_sip_header_t *supported;
 	bool_t one_matching_codec;
 	bool_t use_tcp_tls_keep_alive;
 	bool_t nat_helper_enabled;
@@ -47,6 +48,9 @@ struct Sal{
 	bool_t use_dates;
 	bool_t auto_contacts;
 	bool_t enable_test_features;
+	bool_t no_initial_route;
+	bool_t enable_sip_update; /*true by default*/
+	SalOpSDPHandling default_sdp_handling;
 };
 
 typedef enum SalOpState {
@@ -76,9 +80,11 @@ const char* sal_op_type_to_string(SalOpType type);
 
 struct SalOp{
 	SalOpBase base;
-	belle_sip_listener_callbacks_t callbacks;
+	const belle_sip_listener_callbacks_t *callbacks;
+	SalErrorInfo error_info;
 	belle_sip_client_transaction_t *pending_auth_transaction;
 	belle_sip_server_transaction_t* pending_server_trans;
+	belle_sip_server_transaction_t* pending_update_server_trans;
 	belle_sip_client_transaction_t* pending_client_trans;
 	SalAuthInfo* auth_info;
 	belle_sip_dialog_t* dialog;
@@ -98,6 +104,8 @@ struct SalOp{
 	bool_t sdp_offering;
 	bool_t call_released;
 	bool_t manual_refresher;
+	bool_t has_auth_pending;
+	SalOpSDPHandling sdp_handling;
 	int auth_requests; /*number of auth requested for this op*/
 };
 
@@ -131,13 +139,15 @@ bool_t sal_op_is_secure(const SalOp* op);
 void sal_process_authentication(SalOp *op);
 belle_sip_header_contact_t* sal_op_create_contact(SalOp *op) ;
 
-bool_t sal_compute_sal_errors(belle_sip_response_t* response,SalError* sal_err,SalReason* sal_reason,char* reason, size_t reason_size);
-void sal_compute_sal_errors_from_code(int code ,SalError* sal_err,SalReason* sal_reason) ;
+bool_t _sal_compute_sal_errors(belle_sip_response_t* response, SalReason* sal_reason, char* reason, size_t reason_size);
+SalReason _sal_reason_from_sip_code(int code);
+
+void sal_op_set_error_info_from_response(SalOp *op, belle_sip_response_t *response);
 /*presence*/
 void sal_op_presence_fill_cbs(SalOp*op);
 /*messaging*/
 void sal_op_message_fill_cbs(SalOp*op);
-
+void sal_process_incoming_message(SalOp *op,const belle_sip_request_event_t *event);
 void sal_op_subscribe_fill_cbs(SalOp*op);
 
 /*call transfer*/
@@ -158,8 +168,6 @@ bool_t sal_op_get_body(SalOp *op, belle_sip_message_t *msg, SalBody *salbody);
 
 SalReason sal_reason_to_sip_code(SalReason r);
 
-belle_sip_header_t * sal_make_supported_header(Sal *sal);
-
 void _sal_op_add_custom_headers(SalOp *op, belle_sip_message_t *msg);
 
 #endif /* SAL_IMPL_H_ */
