@@ -19,7 +19,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "sal_impl.h"
 #include "offeranswer.h"
 
-static int extract_sdp(belle_sip_message_t* message,belle_sdp_session_description_t** session_desc, SalReason *error);
+static int extract_sdp(SalOp* op,belle_sip_message_t* message,belle_sdp_session_description_t** session_desc, SalReason *error);
 
 /*used for calls terminated before creation of a dialog*/
 static void call_set_released(SalOp* op){
@@ -27,37 +27,26 @@ static void call_set_released(SalOp* op){
 		op->state=SalOpStateTerminated;
 		op->base.root->callbacks.call_released(op);
 		op->call_released=TRUE;
+		/*be aware that the following line may destroy the op*/
+		set_or_update_dialog(op,NULL);
 	}
 }
 
-/*used when the SalOp was ref'd by the dialog, in which case we rely only on the dialog terminated notification*/
-static void call_set_released_and_unref(SalOp* op) {
-	call_set_released(op);
-	sal_op_unref(op);
-}
-
-
 static void call_set_error(SalOp* op,belle_sip_response_t* response){
-	SalError error=SalErrorUnknown;
-	SalReason sr=SalReasonUnknown;
-	belle_sip_header_t* reason_header = belle_sip_message_get_header(BELLE_SIP_MESSAGE(response),"Reason");
-	char* reason=(char*)belle_sip_response_get_reason_phrase(response);
-	int code = belle_sip_response_get_status_code(response);
-	if (reason_header){
-		reason = ms_strdup_printf("%s %s",reason,belle_sip_header_get_unparsed_value(reason_header));
-	}
-	sal_compute_sal_errors_from_code(code,&error,&sr);
-	op->base.root->callbacks.call_failure(op,error,sr,reason,code);
-	if (reason_header != NULL){
-		ms_free(reason);
-	}
+	sal_op_set_error_info_from_response(op,response);
+	op->base.root->callbacks.call_failure(op);
 }
 
 static void sdp_process(SalOp *h){
 	ms_message("Doing SDP offer/answer process of type %s",h->sdp_offering ? "outgoing" : "incoming");
 	if (h->result){
 		sal_media_description_unref(h->result);
+		h->result = NULL;
 	}
+
+	/* if SDP was invalid */
+	if (h->base.remote_media == NULL) return;
+
 	h->result=sal_media_description_new();
 	if (h->sdp_offering){
 		offer_answer_initiate_outgoing(h->base.local_media,h->base.remote_media,h->result);
@@ -73,16 +62,19 @@ static void sdp_process(SalOp *h){
 		strcpy(h->result->addr,h->base.remote_media->addr);
 		h->result->bandwidth=h->base.remote_media->bandwidth;
 
-		for(i=0;i<h->result->n_active_streams;++i){
-			strcpy(h->result->streams[i].rtp_addr,h->base.remote_media->streams[i].rtp_addr);
-			h->result->streams[i].ptime=h->base.remote_media->streams[i].ptime;
-			h->result->streams[i].bandwidth=h->base.remote_media->streams[i].bandwidth;
-			h->result->streams[i].rtp_port=h->base.remote_media->streams[i].rtp_port;
-			strcpy(h->result->streams[i].rtcp_addr,h->base.remote_media->streams[i].rtcp_addr);
-			h->result->streams[i].rtcp_port=h->base.remote_media->streams[i].rtcp_port;
+		for(i=0;i<h->result->nb_streams;++i){
+			/*copy back parameters from remote description that we need in our result description*/
+			if (h->result->streams[i].rtp_port!=0){ /*if stream was accepted*/
+				strcpy(h->result->streams[i].rtp_addr,h->base.remote_media->streams[i].rtp_addr);
+				h->result->streams[i].ptime=h->base.remote_media->streams[i].ptime;
+				h->result->streams[i].bandwidth=h->base.remote_media->streams[i].bandwidth;
+				h->result->streams[i].rtp_port=h->base.remote_media->streams[i].rtp_port;
+				strcpy(h->result->streams[i].rtcp_addr,h->base.remote_media->streams[i].rtcp_addr);
+				h->result->streams[i].rtcp_port=h->base.remote_media->streams[i].rtcp_port;
 
-			if (h->result->streams[i].proto == SalProtoRtpSavp) {
-				h->result->streams[i].crypto[0] = h->base.remote_media->streams[i].crypto[0];
+				if ((h->result->streams[i].proto == SalProtoRtpSavpf) || (h->result->streams[i].proto == SalProtoRtpSavp)) {
+					h->result->streams[i].crypto[0] = h->base.remote_media->streams[i].crypto[0];
+				}
 			}
 		}
 	}
@@ -91,22 +83,34 @@ static void sdp_process(SalOp *h){
 static int set_sdp(belle_sip_message_t *msg,belle_sdp_session_description_t* session_desc) {
 	belle_sip_header_content_type_t* content_type ;
 	belle_sip_header_content_length_t* content_length;
-	belle_sip_error_code error = BELLE_SIP_OK;
+	belle_sip_error_code error = BELLE_SIP_BUFFER_OVERFLOW;
 	size_t length = 0;
-	char buff[2048];
 
 	if (session_desc) {
+		size_t bufLen = 2048;
+		size_t hardlimit = 16*1024; /* 16k SDP limit seems reasonable */
+		char* buff = belle_sip_malloc(bufLen);
 		content_type = belle_sip_header_content_type_create("application","sdp");
-		error = belle_sip_object_marshal(BELLE_SIP_OBJECT(session_desc),buff,sizeof(buff),&length);
-		if (error != BELLE_SIP_OK) {
-			ms_error("Buffer too small or sdp too big");
+
+		/* try to marshal the description. This could go higher than 2k so we iterate */
+		while( error != BELLE_SIP_OK && bufLen <= hardlimit && buff != NULL){
+			error = belle_sip_object_marshal(BELLE_SIP_OBJECT(session_desc),buff,bufLen,&length);
+			if( error != BELLE_SIP_OK ){
+				bufLen *= 2;
+				length  = 0;
+				buff = belle_sip_realloc(buff,bufLen);
+			}
+		}
+		/* give up if hard limit reached */
+		if (error != BELLE_SIP_OK || buff == NULL) {
+			ms_error("Buffer too small (%d) or not enough memory, giving up SDP", (int)bufLen);
 			return -1;
 		}
 
-		content_length= belle_sip_header_content_length_create(length);
+		content_length = belle_sip_header_content_length_create(length);
 		belle_sip_message_add_header(msg,BELLE_SIP_HEADER(content_type));
 		belle_sip_message_add_header(msg,BELLE_SIP_HEADER(content_length));
-		belle_sip_message_set_body(msg,buff,length);
+		belle_sip_message_assign_body(msg,buff,length);
 		return 0;
 	} else {
 		return -1;
@@ -122,12 +126,13 @@ static int set_sdp_from_desc(belle_sip_message_t *msg, const SalMediaDescription
 }
 static void call_process_io_error(void *user_ctx, const belle_sip_io_error_event_t *event){
 	SalOp* op=(SalOp*)user_ctx;
-	
+
 	if (op->state==SalOpStateTerminated) return;
-	
+
 	if (!op->dialog)  {
 		/*call terminated very early*/
-		op->base.root->callbacks.call_failure(op,SalErrorNoResponse,SalReasonUnknown,"Service Unavailable",503);
+		sal_error_info_set(&op->error_info,SalReasonIOError,503,"IO error",NULL);
+		op->base.root->callbacks.call_failure(op);
 		call_set_released(op);
 	} else {
 		/*dialog will terminated shortly, nothing to do*/
@@ -139,7 +144,7 @@ static void process_dialog_terminated(void *ctx, const belle_sip_dialog_terminat
 	if (op->dialog && op->dialog==belle_sip_dialog_terminated_event_get_dialog(event))  {
 		/*belle_sip_transaction_t* trans=belle_sip_dialog_get_last_transaction(op->dialog);*/
 		ms_message("Dialog [%p] terminated for op [%p]",belle_sip_dialog_terminated_event_get_dialog(event),op);
-		
+
 		switch(belle_sip_dialog_get_previous_state(op->dialog)) {
 			case BELLE_SIP_DIALOG_CONFIRMED:
 				if (op->state!=SalOpStateTerminated && op->state!=SalOpStateTerminating) {
@@ -152,7 +157,7 @@ static void process_dialog_terminated(void *ctx, const belle_sip_dialog_terminat
 			break;
 		}
 		belle_sip_main_loop_do_later(belle_sip_stack_get_main_loop(op->base.root->stack)
-							,(belle_sip_callback_t) call_set_released_and_unref
+							,(belle_sip_callback_t) call_set_released
 							, op);
 	} else {
 		ms_error("dialog unknown for op ");
@@ -162,13 +167,18 @@ static void process_dialog_terminated(void *ctx, const belle_sip_dialog_terminat
 static void handle_sdp_from_response(SalOp* op,belle_sip_response_t* response) {
 	belle_sdp_session_description_t* sdp;
 	SalReason reason;
-	if (extract_sdp(BELLE_SIP_MESSAGE(response),&sdp,&reason)==0) {
+	if (op->base.remote_media){
+		sal_media_description_unref(op->base.remote_media);
+		op->base.remote_media=NULL;
+	}
+	if (extract_sdp(op,BELLE_SIP_MESSAGE(response),&sdp,&reason)==0) {
 		if (sdp){
 			op->base.remote_media=sal_media_description_new();
 			sdp_to_media_description(sdp,op->base.remote_media);
-			if (op->base.local_media) sdp_process(op);
 		}/*if no sdp in response, what can we do ?*/
 	}
+	/* process sdp in any case to reset result media description*/
+	if (op->base.local_media) sdp_process(op);
 }
 
 static void cancelling_invite(SalOp* op ){
@@ -179,6 +189,13 @@ static void cancelling_invite(SalOp* op ){
 	op->state=SalOpStateTerminating;
 }
 
+static int vfu_retry (void *user_data, unsigned int events) {
+	SalOp *op=(SalOp *)user_data;
+	sal_call_send_vfu_request(op);
+	sal_op_unref(op);
+	return BELLE_SIP_STOP;
+}
+
 static void call_process_response(void *op_base, const belle_sip_response_event_t *event){
 	SalOp* op = (SalOp*)op_base;
 	belle_sip_request_t* ack;
@@ -187,22 +204,25 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 	belle_sip_request_t* req;
 	belle_sip_response_t* response=belle_sip_response_event_get_response(event);
 	int code = belle_sip_response_get_status_code(response);
-
+	belle_sip_header_content_type_t *header_content_type=NULL;
+	belle_sip_dialog_t *dialog=belle_sip_response_event_get_dialog(event);
+	const char *method;
 
 	if (!client_transaction) {
 		ms_warning("Discarding stateless response [%i] on op [%p]",code,op);
 		return;
 	}
 	req=belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(client_transaction));
-	set_or_update_dialog(op,belle_sip_response_event_get_dialog(event));
-	dialog_state=op->dialog?belle_sip_dialog_get_state(op->dialog):BELLE_SIP_DIALOG_NULL;
-	
-	ms_message("Op [%p] receiving call response [%i], dialog is [%p] in state [%s]",op,code,op->dialog,belle_sip_dialog_state_to_string(dialog_state));
-
+	set_or_update_dialog(op,dialog);
+	dialog_state=dialog ? belle_sip_dialog_get_state(dialog) : BELLE_SIP_DIALOG_NULL;
+	method=belle_sip_request_get_method(req);
+	ms_message("Op [%p] receiving call response [%i], dialog is [%p] in state [%s]",op,code,dialog,belle_sip_dialog_state_to_string(dialog_state));
+	/*to make sure no cb will destroy op*/
+	sal_op_ref(op);
 	switch(dialog_state) {
 		case BELLE_SIP_DIALOG_NULL:
 		case BELLE_SIP_DIALOG_EARLY: {
-			if (strcmp("INVITE",belle_sip_request_get_method(req))==0 ) {
+			if (strcmp("INVITE",method)==0 ) {
 				if (op->state == SalOpStateTerminating) {
 					/*check if CANCEL was sent before*/
 					if (strcmp("CANCEL",belle_sip_request_get_method(belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(op->pending_client_trans))))!=0) {
@@ -219,41 +239,62 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 							if (op->dialog==NULL) call_set_released(op);
 						}
 					}
-				} else if (code >= 180 && code<300) {
-					handle_sdp_from_response(op,response);
-					op->base.root->callbacks.call_ringing(op);
+				} else if (code >= 180 && code<200) {
+					belle_sip_response_t *prev_response=belle_sip_object_data_get(BELLE_SIP_OBJECT(dialog),"early_response");
+					if (!prev_response || code>belle_sip_response_get_status_code(prev_response)){
+						handle_sdp_from_response(op,response);
+						op->base.root->callbacks.call_ringing(op);
+					}
+					belle_sip_object_data_set(BELLE_SIP_OBJECT(dialog),"early_response",belle_sip_object_ref(response),belle_sip_object_unref);
 				} else if (code>=300){
 					call_set_error(op,response);
 					if (op->dialog==NULL) call_set_released(op);
 				}
+			} else if (code >=200
+						&& code<300
+						&& strcmp("UPDATE",belle_sip_request_get_method(req))==0) {
+					handle_sdp_from_response(op,response);
+					op->base.root->callbacks.call_accepted(op);
 			}
 		}
 		break;
 		case BELLE_SIP_DIALOG_CONFIRMED: {
 			switch (op->state) {
 				case SalOpStateEarly:/*invite case*/
-				case SalOpStateActive: /*re-invite case*/
-					if (code >=200
-						&& code<300
-						&& strcmp("INVITE",belle_sip_request_get_method(req))==0) {
-						handle_sdp_from_response(op,response);
-						ack=belle_sip_dialog_create_ack(op->dialog,belle_sip_dialog_get_local_seq_number(op->dialog));
-						if (ack==NULL) {
-							ms_error("This call has been already terminated.");
-							return ;
+				case SalOpStateActive: /*re-invite, INFO, UPDATE case*/
+					if (strcmp("INVITE",method)==0){
+						if (code >=200 && code<300) {
+							handle_sdp_from_response(op,response);
+							ack=belle_sip_dialog_create_ack(op->dialog,belle_sip_dialog_get_local_seq_number(op->dialog));
+							if (ack==NULL) {
+								ms_error("This call has been already terminated.");
+								return ;
+							}
+							if (op->sdp_answer){
+								set_sdp(BELLE_SIP_MESSAGE(ack),op->sdp_answer);
+								belle_sip_object_unref(op->sdp_answer);
+								op->sdp_answer=NULL;
+							}
+							belle_sip_dialog_send_ack(op->dialog,ack);
+							op->base.root->callbacks.call_accepted(op); /*INVITE*/
+							op->state=SalOpStateActive;
+						}else if (code >= 300){
+							call_set_error(op,response);
 						}
-						if (op->sdp_answer){
-							set_sdp(BELLE_SIP_MESSAGE(ack),op->sdp_answer);
-							belle_sip_object_unref(op->sdp_answer);
-							op->sdp_answer=NULL;
+					}else if (strcmp("INFO",method)==0){
+						if (code == 491
+							&& (header_content_type = belle_sip_message_get_header_by_type(req,belle_sip_header_content_type_t))
+							&& strcmp("application",belle_sip_header_content_type_get_type(header_content_type))==0
+							&& strcmp("media_control+xml",belle_sip_header_content_type_get_subtype(header_content_type))==0) {
+						unsigned int retry_in =1000*((float)rand()/RAND_MAX);
+						belle_sip_source_t *s=sal_create_timer(op->base.root,vfu_retry,sal_op_ref(op), retry_in, "vfu request retry");
+						ms_message("Rejected vfu request on op [%p], just retry in [%ui] ms",op,retry_in);
+						belle_sip_object_unref(s);
+						}else {
+								/*ignoring*/
 						}
-						belle_sip_dialog_send_ack(op->dialog,ack);
+					}else if (strcmp("UPDATE",method)==0){
 						op->base.root->callbacks.call_accepted(op); /*INVITE*/
-						op->state=SalOpStateActive;
-					}  else if (code >= 300 && strcmp("INVITE",belle_sip_request_get_method(req))==0){
-						call_set_error(op,response);
-					} else {
-							/*ignoring*/
 					}
 				break;
 				case SalOpStateTerminating:
@@ -276,16 +317,18 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 		}
 		break;
 	}
+	sal_op_unref(op);
 }
 
 static void call_process_timeout(void *user_ctx, const belle_sip_timeout_event_t *event) {
 	SalOp* op=(SalOp*)user_ctx;
-	
+
 	if (op->state==SalOpStateTerminated) return;
-	
+
 	if (!op->dialog)  {
 		/*call terminated very early*/
-		op->base.root->callbacks.call_failure(op,SalErrorNoResponse,SalReasonUnknown,"Request Timeout",408);
+		sal_error_info_set(&op->error_info,SalReasonRequestTimeout,408,"Request timeout",NULL);
+		op->base.root->callbacks.call_failure(op);
 		call_set_released(op);
 	} else {
 		/*dialog will terminated shortly, nothing to do*/
@@ -298,6 +341,8 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 	belle_sip_server_transaction_t *server_transaction=belle_sip_transaction_terminated_event_get_server_transaction(event);
 	belle_sip_request_t* req;
 	belle_sip_response_t* resp;
+	bool_t release_call=FALSE;
+
 	if (client_transaction) {
 		req=belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(client_transaction));
 		resp=belle_sip_transaction_get_response(BELLE_SIP_TRANSACTION(client_transaction));
@@ -308,9 +353,21 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 	if (op->state ==SalOpStateTerminating
 			&& strcmp("BYE",belle_sip_request_get_method(req))==0
 			&& (!resp || (belle_sip_response_get_status_code(resp) !=401
-			&& belle_sip_response_get_status_code(resp) !=407))) {
-		if (op->dialog==NULL) call_set_released(op);
+			&& belle_sip_response_get_status_code(resp) !=407))
+			&& op->dialog==NULL) {
+		release_call=TRUE;
 	}
+	if (server_transaction){
+		if (op->pending_server_trans==server_transaction){
+			belle_sip_object_unref(op->pending_server_trans);
+			op->pending_server_trans=NULL;
+		}
+		if (op->pending_update_server_trans==server_transaction){
+			belle_sip_object_unref(op->pending_update_server_trans);
+			op->pending_update_server_trans=NULL;
+		}
+	}
+	if (release_call) call_set_released(op);
 }
 
 static void call_terminated(SalOp* op,belle_sip_server_transaction_t* server_transaction, belle_sip_request_t* request,int status_code) {
@@ -322,7 +379,7 @@ static void call_terminated(SalOp* op,belle_sip_server_transaction_t* server_tra
 
 static void unsupported_method(belle_sip_server_transaction_t* server_transaction,belle_sip_request_t* request) {
 	belle_sip_response_t* resp;
-	resp=belle_sip_response_create_from_request(request,500);
+	resp=belle_sip_response_create_from_request(request,501);
 	belle_sip_server_transaction_send_response(server_transaction,resp);
 	return;
 }
@@ -331,14 +388,34 @@ static void unsupported_method(belle_sip_server_transaction_t* server_transactio
  * Extract the sdp from a sip message.
  * If there is no body in the message, the session_desc is set to null, 0 is returned.
  * If body was present is not a SDP or parsing of SDP failed, -1 is returned and SalReason is set appropriately.
- * 
+ *
 **/
-static int extract_sdp(belle_sip_message_t* message,belle_sdp_session_description_t** session_desc, SalReason *error) {
-	belle_sip_header_content_type_t* content_type=belle_sip_message_get_header_by_type(message,belle_sip_header_content_type_t);
+static int extract_sdp(SalOp *op, belle_sip_message_t* message,belle_sdp_session_description_t** session_desc, SalReason *error) {
+	const char *body;
+	belle_sip_header_content_type_t* content_type;
+
+	if (op&&op->sdp_handling == SalOpSDPSimulateError){
+		ms_error("Simulating SDP parsing error for op %p", op);
+		*session_desc=NULL;
+		*error=SalReasonNotAcceptable;
+		return -1;
+	} else if( op && op->sdp_handling == SalOpSDPSimulateRemove){
+		ms_error("Simulating no SDP for op %p", op);
+		*session_desc = NULL;
+		return 0;
+	}
+
+	body = belle_sip_message_get_body(message);
+	if(body == NULL) {
+		*session_desc = NULL;
+		return 0;
+	}
+
+	content_type = belle_sip_message_get_header_by_type(message,belle_sip_header_content_type_t);
 	if (content_type){
 		if (strcmp("application",belle_sip_header_content_type_get_type(content_type))==0
 			&& strcmp("sdp",belle_sip_header_content_type_get_subtype(content_type))==0) {
-			*session_desc=belle_sdp_session_description_parse(belle_sip_message_get_body(message));
+			*session_desc=belle_sdp_session_description_parse(body);
 			if (*session_desc==NULL) {
 				ms_error("Failed to parse SDP message.");
 				*error=SalReasonNotAcceptable;
@@ -353,7 +430,7 @@ static int extract_sdp(belle_sip_message_t* message,belle_sdp_session_descriptio
 }
 
 static int is_media_description_acceptable(SalMediaDescription *md){
-	if (md->n_total_streams==0){
+	if (md->nb_streams==0){
 		ms_warning("Media description does not define any stream.");
 		return FALSE;
 	}
@@ -364,7 +441,7 @@ static int process_sdp_for_invite(SalOp* op,belle_sip_request_t* invite) {
 	belle_sdp_session_description_t* sdp;
 	int err=0;
 	SalReason reason;
-	if (extract_sdp(BELLE_SIP_MESSAGE(invite),&sdp,&reason)==0) {
+	if (extract_sdp(op,BELLE_SIP_MESSAGE(invite),&sdp,&reason)==0) {
 		if (sdp){
 			op->sdp_offering=FALSE;
 			op->base.remote_media=sal_media_description_new();
@@ -377,13 +454,23 @@ static int process_sdp_for_invite(SalOp* op,belle_sip_request_t* invite) {
 			belle_sip_object_unref(sdp);
 		}else op->sdp_offering=TRUE; /*INVITE without SDP*/
 	}else err=-1;
-	
+
 	if (err==-1){
 		sal_call_decline(op,reason,NULL);
 	}
 	return err;
 }
 
+static void sal_op_reset_descriptions(SalOp *op) {
+	if (op->base.remote_media){
+		sal_media_description_unref(op->base.remote_media);
+		op->base.remote_media=NULL;
+	}
+	if (op->result){
+		sal_media_description_unref(op->result);
+		op->result=NULL;
+	}
+}
 static void process_request_event(void *op_base, const belle_sip_request_event_t *event) {
 	SalOp* op = (SalOp*)op_base;
 	belle_sip_server_transaction_t* server_transaction=NULL;
@@ -392,21 +479,29 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 	belle_sip_dialog_state_t dialog_state;
 	belle_sip_response_t* resp;
 	belle_sip_header_t* call_info;
+	const char *method=belle_sip_request_get_method(req);
+	bool_t is_update=FALSE;
 
-	if (strcmp("ACK",belle_sip_request_get_method(req))!=0){  /*ACK does'nt create srv transaction*/
+	if (strcmp("ACK",method)!=0){  /*ACK does'nt create srv transaction*/
 		server_transaction = belle_sip_provider_create_server_transaction(op->base.root->prov,belle_sip_request_event_get_request(event));
 		belle_sip_object_ref(server_transaction);
-		belle_sip_transaction_set_application_data(BELLE_SIP_TRANSACTION(server_transaction),op);
-		sal_op_ref(op);
+		belle_sip_transaction_set_application_data(BELLE_SIP_TRANSACTION(server_transaction),sal_op_ref(op));
 	}
 
-	if (strcmp("INVITE",belle_sip_request_get_method(req))==0) {
+	if (strcmp("INVITE",method)==0) {
 		if (op->pending_server_trans) belle_sip_object_unref(op->pending_server_trans);
 		/*updating pending invite transaction*/
 		op->pending_server_trans=server_transaction;
 		belle_sip_object_ref(op->pending_server_trans);
 	}
 
+	if (strcmp("UPDATE",method)==0) {
+		if (op->pending_update_server_trans) belle_sip_object_unref(op->pending_update_server_trans);
+		/*updating pending update transaction*/
+		op->pending_update_server_trans=server_transaction;
+		belle_sip_object_ref(op->pending_update_server_trans);
+	}
+
 	if (!op->dialog) {
 		set_or_update_dialog(op,belle_sip_provider_create_dialog(op->base.root->prov,BELLE_SIP_TRANSACTION(op->pending_server_trans)));
 		ms_message("new incoming call from [%s] to [%s]",sal_op_get_from(op),sal_op_get_to(op));
@@ -414,30 +509,28 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 	dialog_state=belle_sip_dialog_get_state(op->dialog);
 	switch(dialog_state) {
 	case BELLE_SIP_DIALOG_NULL: {
-		if (strcmp("INVITE",belle_sip_request_get_method(req))==0) {
+		if (strcmp("INVITE",method)==0) {
 			if (!op->replaces && (op->replaces=belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(req),belle_sip_header_replaces_t))) {
 				belle_sip_object_ref(op->replaces);
 			} else if(op->replaces) {
 				ms_warning("replace header already set");
 			}
 
-			process_sdp_for_invite(op,req);
-
-			if ((call_info=belle_sip_message_get_header(BELLE_SIP_MESSAGE(req),"Call-Info"))) {
-				if( strstr(belle_sip_header_get_unparsed_value(call_info),"answer-after=") != NULL) {
-					op->auto_answer_asked=TRUE;
-					ms_message("The caller asked to automatically answer the call(Emergency?)\n");
+			if (process_sdp_for_invite(op,req) == 0) {
+				if ((call_info=belle_sip_message_get_header(BELLE_SIP_MESSAGE(req),"Call-Info"))) {
+					if( strstr(belle_sip_header_get_unparsed_value(call_info),"answer-after=") != NULL) {
+						op->auto_answer_asked=TRUE;
+						ms_message("The caller asked to automatically answer the call(Emergency?)\n");
+					}
 				}
+				op->base.root->callbacks.call_received(op);
 			}
-
-			op->base.root->callbacks.call_received(op);
-
 			break;
 		} /* else same behavior as for EARLY state*/
 	}
 	case BELLE_SIP_DIALOG_EARLY: {
 		//hmm probably a cancel
-		if (strcmp("CANCEL",belle_sip_request_get_method(req))==0) {
+		if (strcmp("CANCEL",method)==0) {
 			if(belle_sip_request_event_get_server_transaction(event)) {
 				/*first answer 200 ok to cancel*/
 				belle_sip_server_transaction_send_response(server_transaction
@@ -453,9 +546,13 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 				belle_sip_server_transaction_send_response(server_transaction
 							,sal_op_create_response_from_request(op,req,481));
 			}
-		} else if (strcmp("PRACK",belle_sip_request_get_method(req))==0) {
+		} else if (strcmp("PRACK",method)==0) {
 			resp=sal_op_create_response_from_request(op,req,200);
 			belle_sip_server_transaction_send_response(server_transaction,resp);
+		} else if (strcmp("UPDATE",method)==0) {
+			sal_op_reset_descriptions(op);
+			if (process_sdp_for_invite(op,req)==0)
+				op->base.root->callbacks.call_updating(op,TRUE);
 		} else {
 			belle_sip_error("Unexpected method [%s] for dialog state BELLE_SIP_DIALOG_EARLY",belle_sip_request_get_method(req));
 			unsupported_method(server_transaction,req);
@@ -464,10 +561,10 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 	}
 	case BELLE_SIP_DIALOG_CONFIRMED:
 		/*great ACK received*/
-		if (strcmp("ACK",belle_sip_request_get_method(req))==0) {
+		if (strcmp("ACK",method)==0) {
 			if (op->sdp_offering){
 				SalReason reason;
-				if (extract_sdp(BELLE_SIP_MESSAGE(req),&sdp,&reason)==0){
+				if (extract_sdp(op,BELLE_SIP_MESSAGE(req),&sdp,&reason)==0){
 					if (sdp){
 						if (op->base.remote_media)
 							sal_media_description_unref(op->base.remote_media);
@@ -480,30 +577,28 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 					}
 				}
 			}
-			/*FIXME
-		if (op->reinvite){
-			op->reinvite=FALSE;
-		}*/
 			op->base.root->callbacks.call_ack(op);
-		} else if(strcmp("BYE",belle_sip_request_get_method(req))==0) {
+		} else if(strcmp("BYE",method)==0) {
 			resp=sal_op_create_response_from_request(op,req,200);
 			belle_sip_server_transaction_send_response(server_transaction,resp);
 			op->base.root->callbacks.call_terminated(op,op->dir==SalOpDirIncoming?sal_op_get_from(op):sal_op_get_to(op));
 			op->state=SalOpStateTerminating;
 			/*call end not notified by dialog deletion because transaction can end before dialog*/
-		} else if(strcmp("INVITE",belle_sip_request_get_method(req))==0) {
-			/*re-invite*/
-			if (op->base.remote_media){
-				sal_media_description_unref(op->base.remote_media);
-				op->base.remote_media=NULL;
+		} else if(strcmp("INVITE",method)==0 || (is_update=(strcmp("UPDATE",method)==0)) ) {
+			if (is_update && !belle_sip_message_get_body(BELLE_SIP_MESSAGE(req))) {
+				/*session timer case*/
+				/*session expire should be handled. to be done when real session timer (rfc4028) will be implemented*/
+				resp=sal_op_create_response_from_request(op,req,200);
+				belle_sip_server_transaction_send_response(server_transaction,resp);
+				belle_sip_object_unref(op->pending_update_server_trans);
+				op->pending_update_server_trans=NULL;
+			} else {
+				/*re-invite*/
+				sal_op_reset_descriptions(op);
+				if (process_sdp_for_invite(op,req)==0)
+					op->base.root->callbacks.call_updating(op,is_update);
 			}
-			if (op->result){
-				sal_media_description_unref(op->result);
-				op->result=NULL;
-			}
-			if (process_sdp_for_invite(op,req)==0)
-				op->base.root->callbacks.call_updating(op);
-		} else if (strcmp("INFO",belle_sip_request_get_method(req))==0){
+		} else if (strcmp("INFO",method)==0){
 			if (belle_sip_message_get_body(BELLE_SIP_MESSAGE(req))
 				&&	strstr(belle_sip_message_get_body(BELLE_SIP_MESSAGE(req)),"picture_fast_update")) {
 				/*vfu request*/
@@ -528,19 +623,19 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 			}
 			resp=sal_op_create_response_from_request(op,req,200);
 			belle_sip_server_transaction_send_response(server_transaction,resp);
-		}else if (strcmp("REFER",belle_sip_request_get_method(req))==0) {
+		}else if (strcmp("REFER",method)==0) {
 			sal_op_process_refer(op,event,server_transaction);
-		} else if (strcmp("NOTIFY",belle_sip_request_get_method(req))==0) {
+		} else if (strcmp("NOTIFY",method)==0) {
 			sal_op_call_process_notify(op,event,server_transaction);
-		} else if (strcmp("OPTIONS",belle_sip_request_get_method(req))==0) {
+		} else if (strcmp("OPTIONS",method)==0) {
 			resp=sal_op_create_response_from_request(op,req,200);
 			belle_sip_server_transaction_send_response(server_transaction,resp);
-		} else if (strcmp("CANCEL",belle_sip_request_get_method(req))==0) {
+		} else if (strcmp("CANCEL",method)==0) {
 			/*call leg does not exist because 200ok already sent*/
-			belle_sip_server_transaction_send_response(	server_transaction
-														,sal_op_create_response_from_request(op,req,481));
-
-		} else{
+			belle_sip_server_transaction_send_response(server_transaction,sal_op_create_response_from_request(op,req,481));
+		} else if (strcmp("MESSAGE",method)==0){
+			sal_process_incoming_message(op,event);
+		}else{
 			ms_error("unexpected method [%s] for dialog [%p]",belle_sip_request_get_method(req),op->dialog);
 			unsupported_method(server_transaction,req);
 		}
@@ -562,7 +657,7 @@ int sal_call_set_local_media_description(SalOp *op, SalMediaDescription *desc){
 	if (op->base.local_media)
 		sal_media_description_unref(op->base.local_media);
 	op->base.local_media=desc;
-	
+
 	if (op->base.remote_media){
 		/*case of an incoming call where we modify the local capabilities between the time
 		 * the call is ringing and it is accepted (for example if you want to accept without video*/
@@ -575,22 +670,24 @@ int sal_call_set_local_media_description(SalOp *op, SalMediaDescription *desc){
 	return 0;
 }
 
-static belle_sip_header_allow_t *create_allow(){
+static belle_sip_header_allow_t *create_allow(bool_t enable_update){
 	belle_sip_header_allow_t* header_allow;
-        header_allow = belle_sip_header_allow_create("INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO");
+	char allow [256];
+	snprintf(allow,sizeof(allow),"INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO%s",(enable_update?", UPDATE":""));
+	header_allow = belle_sip_header_allow_create(allow);
 	return header_allow;
 }
 
 static void sal_op_fill_invite(SalOp *op, belle_sip_request_t* invite) {
-	belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite),BELLE_SIP_HEADER(create_allow()));
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite),BELLE_SIP_HEADER(create_allow(op->base.root->enable_sip_update)));
 
 	if (op->base.root->session_expires!=0){
-		belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite),belle_sip_header_create( "Session-expires", "200"));
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite),belle_sip_header_create( "Session-expires", "600;refresher=uas"));
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite),belle_sip_header_create( "Supported", "timer"));
 	}
 	if (op->base.local_media){
-			op->sdp_offering=TRUE;
-			set_sdp_from_desc(BELLE_SIP_MESSAGE(invite),op->base.local_media);
+		op->sdp_offering=TRUE;
+		set_sdp_from_desc(BELLE_SIP_MESSAGE(invite),op->base.local_media);
 	}else op->sdp_offering=FALSE;
 	return;
 }
@@ -605,6 +702,11 @@ int sal_call(SalOp *op, const char *from, const char *to){
 	ms_message("[%s] calling [%s] on op [%p]", from, to, op);
 	invite=sal_op_build_request(op,"INVITE");
 
+	if( invite == NULL ){
+		/* can happen if the op has an invalid address */
+		return -1;
+	}
+
 	sal_op_fill_invite(op,invite);
 
 	sal_op_call_fill_cbs(op);
@@ -613,17 +715,22 @@ int sal_call(SalOp *op, const char *from, const char *to){
 	}
 	if (op->referred_by)
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite),BELLE_SIP_HEADER(op->referred_by));
-	
+
 	return sal_op_send_request(op,invite);
 }
 
+static belle_sip_listener_callbacks_t call_op_callbacks={0};
+
 void sal_op_call_fill_cbs(SalOp*op) {
-	op->callbacks.process_io_error=call_process_io_error;
-	op->callbacks.process_response_event=call_process_response;
-	op->callbacks.process_timeout=call_process_timeout;
-	op->callbacks.process_transaction_terminated=call_process_transaction_terminated;
-	op->callbacks.process_request_event=process_request_event;
-	op->callbacks.process_dialog_terminated=process_dialog_terminated;
+	if (call_op_callbacks.process_response_event==NULL){
+		call_op_callbacks.process_io_error=call_process_io_error;
+		call_op_callbacks.process_response_event=call_process_response;
+		call_op_callbacks.process_timeout=call_process_timeout;
+		call_op_callbacks.process_transaction_terminated=call_process_transaction_terminated;
+		call_op_callbacks.process_request_event=process_request_event;
+		call_op_callbacks.process_dialog_terminated=process_dialog_terminated;
+	}
+	op->callbacks=&call_op_callbacks;
 	op->type=SalOpCall;
 }
 
@@ -634,8 +741,14 @@ static void handle_offer_answer_response(SalOp* op, belle_sip_response_t* respon
 			set_sdp_from_desc(BELLE_SIP_MESSAGE(response),op->base.local_media);
 		}else{
 
-			if (op->sdp_answer==NULL)
-				sdp_process(op);
+			if ( op->sdp_answer==NULL )
+			{
+				if( op->sdp_handling == SalOpSDPSimulateRemove ){
+					ms_warning("Simulating SDP removal in answer for op %p", op);
+				} else {
+					sdp_process(op);
+				}
+			}
 
 			if (op->sdp_answer){
 				set_sdp(BELLE_SIP_MESSAGE(response),op->sdp_answer);
@@ -654,7 +767,7 @@ int sal_call_notify_ringing(SalOp *op, bool_t early_media){
 	belle_sip_response_t* ringing_response = sal_op_create_response_from_request(op,req,status_code);
 	belle_sip_header_t *require;
 	const char *tags=NULL;
-	
+
 	if (early_media){
 		handle_offer_answer_response(op,ringing_response);
 	}
@@ -667,7 +780,7 @@ int sal_call_notify_ringing(SalOp *op, bool_t early_media){
 	}
 
 #ifndef SAL_OP_CALL_FORCE_CONTACT_IN_RINGING
-	if (tags && strstr(tags,"100rel")!=0) 
+	if (tags && strstr(tags,"100rel")!=0)
 #endif
 	{
 		belle_sip_header_address_t* contact= (belle_sip_header_address_t*)sal_op_get_contact_address(op);
@@ -685,35 +798,47 @@ int sal_call_notify_ringing(SalOp *op, bool_t early_media){
 int sal_call_accept(SalOp*h){
 	belle_sip_response_t *response;
 	belle_sip_header_contact_t* contact_header;
+	belle_sip_server_transaction_t* transaction;
 
-	if (!h->pending_server_trans) {
+	/*first check if an UPDATE transaction need to be accepted*/
+	if (h->pending_update_server_trans) {
+		transaction=h->pending_update_server_trans;
+	} else if (h->pending_server_trans) {
+		/*so it must be an invite/re-invite*/
+		transaction=h->pending_server_trans;
+	} else {
 		ms_error("No transaction to accept for op [%p]",h);
 		return -1;
 	}
-
+	ms_message("Accepting server transaction [%p] on op [%p]", transaction, h);
 	/* sends a 200 OK */
-	response = sal_op_create_response_from_request(h,belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(h->pending_server_trans)),200);
+	response = sal_op_create_response_from_request(h,belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(transaction)),200);
 
 	if (response==NULL){
 		ms_error("Fail to build answer for call");
 		return -1;
 	}
-	belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),BELLE_SIP_HEADER(create_allow()));
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),BELLE_SIP_HEADER(create_allow(h->base.root->enable_sip_update)));
 	if (h->base.root->session_expires!=0){
-		if (h->supports_session_timers) {
+/*		if (h->supports_session_timers) {*/
 			belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),belle_sip_header_create("Supported", "timer"));
-		}
+			belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),belle_sip_header_create( "Session-expires", "600;refresher=uac"));
+		/*}*/
 	}
 
 	if ((contact_header=sal_op_create_contact(h))) {
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),BELLE_SIP_HEADER(contact_header));
 	}
-	
+
 	_sal_op_add_custom_headers(h, BELLE_SIP_MESSAGE(response));
 
 	handle_offer_answer_response(h,response);
 
-	belle_sip_server_transaction_send_response(h->pending_server_trans,response);
+	belle_sip_server_transaction_send_response(transaction,response);
+	if (h->pending_update_server_trans) {
+		belle_sip_object_unref(h->pending_update_server_trans);
+		h->pending_update_server_trans=NULL;
+	}
 	return 0;
 }
 
@@ -721,7 +846,8 @@ int sal_call_decline(SalOp *op, SalReason reason, const char *redirection /*opti
 	belle_sip_response_t* response;
 	belle_sip_header_contact_t* contact=NULL;
 	int status=sal_reason_to_sip_code(reason);
-	
+	belle_sip_transaction_t *trans;
+
 	if (reason==SalReasonRedirect){
 		if (redirection!=NULL) {
 			if (strstr(redirection,"sip:")!=0) status=302;
@@ -732,18 +858,37 @@ int sal_call_decline(SalOp *op, SalReason reason, const char *redirection /*opti
 			ms_error("Cannot redirect to null");
 		}
 	}
-	response = sal_op_create_response_from_request(op,belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(op->pending_server_trans)),status);
+	trans=(belle_sip_transaction_t*)op->pending_server_trans;
+	if (!trans) trans=(belle_sip_transaction_t*)op->pending_update_server_trans;
+	if (!trans){
+		ms_error("sal_call_decline(): no pending transaction to decline.");
+		return -1;
+	}
+	response = sal_op_create_response_from_request(op,belle_sip_transaction_get_request(trans),status);
 	if (contact) belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),BELLE_SIP_HEADER(contact));
-	belle_sip_server_transaction_send_response(op->pending_server_trans,response);
+	belle_sip_server_transaction_send_response(BELLE_SIP_SERVER_TRANSACTION(trans),response);
 	return 0;
 }
 
-int sal_call_update(SalOp *op, const char *subject){
-	belle_sip_request_t *reinvite=belle_sip_dialog_create_request(op->dialog,"INVITE");
-	if (reinvite){
-		belle_sip_message_add_header(BELLE_SIP_MESSAGE(reinvite),belle_sip_header_create( "Subject", subject));
-		sal_op_fill_invite(op, reinvite);
-		return sal_op_send_request(op,reinvite);
+int sal_call_update(SalOp *op, const char *subject, bool_t no_user_consent){
+	belle_sip_request_t *update;
+	belle_sip_dialog_state_t state=belle_sip_dialog_get_state(op->dialog);
+	/*check for dialog state*/
+	if ( state == BELLE_SIP_DIALOG_CONFIRMED) {
+		if (no_user_consent)
+			update=belle_sip_dialog_create_request(op->dialog,"UPDATE");
+		else
+			update=belle_sip_dialog_create_request(op->dialog,"INVITE");
+	} else if (state == BELLE_SIP_DIALOG_EARLY)  {
+		update=belle_sip_dialog_create_request(op->dialog,"UPDATE");
+	} else {
+		ms_error("Cannot update op [%p] with dialog [%p] in state [%s]",op, op->dialog,belle_sip_dialog_state_to_string(state));
+		return  -1;
+	}
+	if (update){
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(update),belle_sip_header_create( "Subject", subject));
+		sal_op_fill_invite(op, update);
+		return sal_op_send_request(op,update);
 	}
 	return -1;
 }
@@ -765,7 +910,7 @@ int sal_call_send_dtmf(SalOp *h, char dtmf){
 		if (req){
 			int bodylen;
 			char dtmf_body[128]={0};
-			
+
 			snprintf(dtmf_body, sizeof(dtmf_body)-1, "Signal=%c\r\nDuration=250\r\n", dtmf);
 			bodylen=strlen(dtmf_body);
 			belle_sip_message_set_body((belle_sip_message_t*)req,dtmf_body,bodylen);
