@@ -18,7 +18,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 #include "sal_impl.h"
 
-
 /*create an operation */
 SalOp * sal_op_new(Sal *sal){
 	SalOp *op=ms_new0(SalOp,1);
@@ -26,6 +25,7 @@ SalOp * sal_op_new(Sal *sal){
 	op->type=SalOpUnknown;
 	op->privacy=SalPrivacyNone;
 	op->manual_refresher=FALSE;/*tells that requests with expiry (SUBSCRIBE, PUBLISH) will be automatically refreshed*/
+	op->sdp_handling=sal->default_sdp_handling;
 	sal_op_ref(op);
 	return op;
 }
@@ -44,8 +44,8 @@ void sal_op_release(SalOp *op){
 void sal_op_release_impl(SalOp *op){
 	ms_message("Destroying op [%p] of type [%s]",op,sal_op_type_to_string(op->type));
 	if (op->pending_auth_transaction) belle_sip_object_unref(op->pending_auth_transaction);
+	sal_remove_pending_auth(op->base.root,op);
 	if (op->auth_info) {
-		sal_remove_pending_auth(op->base.root,op);
 		sal_auth_info_delete(op->auth_info);
 	}
 	if (op->sdp_answer) belle_sip_object_unref(op->sdp_answer);
@@ -60,7 +60,9 @@ void sal_op_release_impl(SalOp *op){
 
 	if (op->pending_client_trans) belle_sip_object_unref(op->pending_client_trans);
 	if (op->pending_server_trans) belle_sip_object_unref(op->pending_server_trans);
+	if (op->pending_update_server_trans) belle_sip_object_unref(op->pending_update_server_trans);
 	if (op->event) belle_sip_object_unref(op->event);
+	sal_error_info_reset(&op->error_info);
 	__sal_op_free(op);
 	return ;
 }
@@ -88,18 +90,24 @@ SalAuthInfo * sal_op_get_auth_requested(SalOp *op){
 belle_sip_header_contact_t* sal_op_create_contact(SalOp *op){
 	belle_sip_header_contact_t* contact_header;
 	belle_sip_uri_t* contact_uri;
+
 	if (sal_op_get_contact_address(op)) {
 		contact_header = belle_sip_header_contact_create(BELLE_SIP_HEADER_ADDRESS(sal_op_get_contact_address(op)));
 	} else {
 		contact_header= belle_sip_header_contact_new();
 	}
+
 	if (!(contact_uri=belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(contact_header)))) {
 		/*no uri, just creating a new one*/
 		contact_uri=belle_sip_uri_new();
 		belle_sip_header_address_set_uri(BELLE_SIP_HEADER_ADDRESS(contact_header),contact_uri);
 	}
-	belle_sip_uri_set_secure(contact_uri,sal_op_is_secure(op));
 
+	belle_sip_uri_set_user_password(contact_uri,NULL);
+	belle_sip_uri_set_secure(contact_uri,sal_op_is_secure(op));
+	if (op->privacy!=SalPrivacyNone){
+		belle_sip_uri_set_user(contact_uri,NULL);
+	}
 	belle_sip_header_contact_set_automatic(contact_header,op->base.root->auto_contacts);
 	if (op->base.root->uuid){
 		if (belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(contact_header),"+sip.instance")==0){
@@ -111,8 +119,29 @@ belle_sip_header_contact_t* sal_op_create_contact(SalOp *op){
 	return contact_header;
 }
 
-belle_sip_header_t * sal_make_supported_header(Sal *sal){
-	return belle_sip_header_create("Supported","replaces, outbound");
+
+
+static void add_initial_route_set(belle_sip_request_t *request, const MSList *list){
+	const MSList *elem;
+	for (elem=list;elem!=NULL;elem=elem->next){
+		SalAddress *addr=(SalAddress*)elem->data;
+		belle_sip_header_route_t *route;
+		belle_sip_uri_t *uri;
+		/*Optimization: if the initial route set only contains one URI which is the same as the request URI, ommit it*/
+		if (elem==list && list->next==NULL){
+			belle_sip_uri_t *requri=belle_sip_request_get_uri(request);
+			/*skip the first route it is the same as the request uri*/
+			if (strcmp(sal_address_get_domain(addr),belle_sip_uri_get_host(requri))==0 ){
+				ms_message("Skipping top route of initial route-set because same as request-uri.");
+				continue;
+			}
+		}
+
+		route=belle_sip_header_route_create((belle_sip_header_address_t*)addr);
+		uri=belle_sip_header_address_get_uri((belle_sip_header_address_t*)route);
+		belle_sip_uri_set_lr_param(uri,1);
+		belle_sip_message_add_header((belle_sip_message_t*)request,(belle_sip_header_t*)route);
+	}
 }
 
 belle_sip_request_t* sal_op_build_request(SalOp *op,const char* method) {
@@ -121,20 +150,37 @@ belle_sip_request_t* sal_op_build_request(SalOp *op,const char* method) {
 	belle_sip_provider_t* prov=op->base.root->prov;
 	belle_sip_request_t *req;
 	belle_sip_uri_t* req_uri;
+	belle_sip_uri_t* to_uri;
+
+	const SalAddress* to_address;
+	const MSList *elem=sal_op_get_route_addresses(op);
 	char token[10];
 
+	/* check that the op has a correct to address */
+	to_address = sal_op_get_to_address(op);
+	if( to_address == NULL ){
+		ms_error("No To: address, cannot build request");
+		return NULL;
+	}
+	
+	to_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(to_address));
+	if( to_uri == NULL ){
+		ms_error("To: address is invalid, cannot build request");
+		return NULL;
+	}
 
 	if (strcmp("REGISTER",method)==0 || op->privacy==SalPrivacyNone) {
 		from_header = belle_sip_header_from_create(BELLE_SIP_HEADER_ADDRESS(sal_op_get_from_address(op))
-												,belle_sip_random_token(token,sizeof(token)));
+						,belle_sip_random_token(token,sizeof(token)));
 	} else {
 		from_header=belle_sip_header_from_create2("Anonymous <sip:anonymous@anonymous.invalid>",belle_sip_random_token(token,sizeof(token)));
 	}
 	/*make sure to preserve components like headers or port*/
-	req_uri = (belle_sip_uri_t*)belle_sip_object_clone((belle_sip_object_t*)belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(sal_op_get_to_address(op))));
+
+	req_uri = (belle_sip_uri_t*)belle_sip_object_clone((belle_sip_object_t*)to_uri);
 	belle_sip_uri_set_secure(req_uri,sal_op_is_secure(op));
 
-	to_header = belle_sip_header_to_create(BELLE_SIP_HEADER_ADDRESS(sal_op_get_to_address(op)),NULL);
+	to_header = belle_sip_header_to_create(BELLE_SIP_HEADER_ADDRESS(to_address),NULL);
 
 	req=belle_sip_request_create(
 					req_uri,
@@ -150,6 +196,11 @@ belle_sip_request_t* sal_op_build_request(SalOp *op,const char* method) {
 		belle_sip_header_p_preferred_identity_t* p_preferred_identity=belle_sip_header_p_preferred_identity_create(BELLE_SIP_HEADER_ADDRESS(sal_op_get_from_address(op)));
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(p_preferred_identity));
 	}
+
+	if (elem && strcmp(method,"REGISTER")!=0 && !op->base.root->no_initial_route){
+		add_initial_route_set(req,elem);
+	}
+
 	if (strcmp("REGISTER",method)!=0 && op->privacy!=SalPrivacyNone ){
 		belle_sip_header_privacy_t* privacy_header=belle_sip_header_privacy_new();
 		if (op->privacy&SalPrivacyCritical)
@@ -166,7 +217,7 @@ belle_sip_request_t* sal_op_build_request(SalOp *op,const char* method) {
 			belle_sip_header_privacy_add_privacy(privacy_header,sal_privacy_to_string(SalPrivacyUser));
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(privacy_header));
 	}
-	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),sal_make_supported_header(op->base.root));
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),op->base.root->supported);
 	return req;
 }
 
@@ -184,7 +235,10 @@ int sal_ping(SalOp *op, const char *from, const char *to){
 void sal_op_set_remote_ua(SalOp*op,belle_sip_message_t* message) {
 	belle_sip_header_user_agent_t* user_agent=belle_sip_message_get_header_by_type(message,belle_sip_header_user_agent_t);
 	char user_agent_string[256];
-	if(user_agent && belle_sip_header_user_agent_get_products_as_string(user_agent,user_agent_string,sizeof(user_agent_string))>0) {
+	if (user_agent && belle_sip_header_user_agent_get_products_as_string(user_agent,user_agent_string,sizeof(user_agent_string))>0) {
+		if (op->base.remote_ua!=NULL){
+			ms_free(op->base.remote_ua);
+		}
 		op->base.remote_ua=ms_strdup(user_agent_string);
 	}
 }
@@ -206,7 +260,7 @@ void sal_op_resend_request(SalOp* op, belle_sip_request_t* request) {
 }
 
 static void add_headers(SalOp *op, belle_sip_header_t *h, belle_sip_message_t *msg){
-	
+
 	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(h,belle_sip_header_contact_t)){
 		belle_sip_header_contact_t* newct;
 		/*special case for contact, we want to keep everything from the custom contact but set automatic mode and add our own parameters as well*/
@@ -217,7 +271,7 @@ static void add_headers(SalOp *op, belle_sip_header_t *h, belle_sip_message_t *m
 	}
 	/*if a header already exists in the message, replace it*/
 	belle_sip_message_set_header(msg,h);
-	
+
 }
 
 void _sal_op_add_custom_headers(SalOp *op, belle_sip_message_t *msg){
@@ -239,20 +293,21 @@ static int _sal_op_send_request_with_contact(SalOp* op, belle_sip_request_t* req
 	belle_sip_header_contact_t* contact;
 	int result =-1;
 	belle_sip_uri_t *next_hop_uri=NULL;
-	
+
 	if (add_contact) {
 		contact = sal_op_create_contact(op);
 		belle_sip_message_set_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_HEADER(contact));
 	}
-	
+
 	_sal_op_add_custom_headers(op, (belle_sip_message_t*)request);
-	
+
 	if (!op->dialog || belle_sip_dialog_get_state(op->dialog) == BELLE_SIP_DIALOG_NULL) {
 		/*don't put route header if  dialog is in confirmed state*/
 		const MSList *elem=sal_op_get_route_addresses(op);
 		const char *transport;
 		const char *method=belle_sip_request_get_method(request);
-		
+		belle_sip_listening_point_t *udplp=belle_sip_provider_get_listening_point(prov,"UDP");
+
 		if (elem) {
 			outbound_proxy=belle_sip_header_address_get_uri((belle_sip_header_address_t*)elem->data);
 			next_hop_uri=outbound_proxy;
@@ -264,7 +319,7 @@ static int _sal_op_send_request_with_contact(SalOp* op, belle_sip_request_t* req
 			/*compatibility mode: by default it should be udp as not explicitely set and if no udp listening point is available, then use
 			 * the first available transport*/
 			if (!belle_sip_uri_is_secure(next_hop_uri)){
-				if (belle_sip_provider_get_listening_point(prov,"UDP")==0){
+				if (udplp==NULL){
 					if (belle_sip_provider_get_listening_point(prov,"TCP")!=NULL){
 						transport="tcp";
 					}else if (belle_sip_provider_get_listening_point(prov,"TLS")!=NULL ){
@@ -276,8 +331,15 @@ static int _sal_op_send_request_with_contact(SalOp* op, belle_sip_request_t* req
 					belle_sip_uri_set_transport_param(next_hop_uri,transport);
 				}
 			}
+		}else{
+#ifdef TUNNEL_ENABLED
+			if (udplp && BELLE_SIP_OBJECT_IS_INSTANCE_OF(udplp,belle_sip_tunnel_listening_point_t)){
+				/* our tunnel mode only supports UDP. Force transport to be set to UDP */
+				belle_sip_uri_set_transport_param(next_hop_uri,"udp");
+			}
+#endif
 		}
-		if ((strcmp(method,"REGISTER")==0 || strcmp(method,"SUBSCRIBE")==0) && transport && 
+		if ((strcmp(method,"REGISTER")==0 || strcmp(method,"SUBSCRIBE")==0) && transport &&
 			(strcasecmp(transport,"TCP")==0 || strcasecmp(transport,"TLS")==0)){
 			/*RFC 5923: add 'alias' parameter to tell the server that we want it to keep the connection for future requests*/
 			belle_sip_header_via_t *via=belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(request),belle_sip_header_via_t);
@@ -297,15 +359,15 @@ static int _sal_op_send_request_with_contact(SalOp* op, belle_sip_request_t* req
 	if (!belle_sip_message_get_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_AUTHORIZATION)
 		&& !belle_sip_message_get_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_PROXY_AUTHORIZATION)) {
 		/*hmm just in case we already have authentication param in cache*/
-		belle_sip_provider_add_authorization(op->base.root->prov,request,NULL,NULL);
+		belle_sip_provider_add_authorization(op->base.root->prov,request,NULL,NULL,NULL,op->base.realm);
 	}
 	result = belle_sip_client_transaction_send_request_to(client_transaction,next_hop_uri/*might be null*/);
-	
+
 	/*update call id if not set yet for this OP*/
 	if (result == 0 && !op->base.call_id) {
 		op->base.call_id=ms_strdup(belle_sip_header_call_id_get_call_id(BELLE_SIP_HEADER_CALL_ID(belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(request), belle_sip_header_call_id_t))));
 	}
-	
+
 	return result;
 
 }
@@ -316,9 +378,9 @@ int sal_op_send_request(SalOp* op, belle_sip_request_t* request)  {
 		return -1; /*sanity check*/
 	}
 	/*
-  	  Header field          where   proxy ACK BYE CAN INV OPT REG
-      ___________________________________________________________
-      Contact                 R            o   -   -   m   o   o
+	  Header field          where   proxy ACK BYE CAN INV OPT REG
+	  ___________________________________________________________
+	  Contact                 R            o   -   -   m   o   o
 	 */
 	if (strcmp(belle_sip_request_get_method(request),"INVITE")==0
 			||strcmp(belle_sip_request_get_method(request),"REGISTER")==0
@@ -333,6 +395,12 @@ int sal_op_send_request(SalOp* op, belle_sip_request_t* request)  {
 SalReason sal_reason_to_sip_code(SalReason r){
 	int ret=500;
 	switch(r){
+		case SalReasonNone:
+			ret=200;
+			break;
+		case SalReasonIOError:
+			ret=503;
+			break;
 		case SalReasonUnknown:
 			ret=400;
 			break;
@@ -375,124 +443,174 @@ SalReason sal_reason_to_sip_code(SalReason r){
 		case SalReasonNoMatch:
 			ret=481;
 			break;
+		case SalReasonRequestTimeout:
+			ret=408;
+			break;
+		case SalReasonMovedPermanently:
+			ret=301;
+			break;
+		case SalReasonGone:
+			ret=410;
+			break;
+		case SalReasonAddressIncomplete:
+			ret=484;
+			break;
+		case SalReasonNotImplemented:
+			ret=501;
+			break;
+		case SalReasonServerTimeout:
+			ret=504;
+			break;
+		case SalReasonBadGateway:
+			ret=502;
+			break;
 	}
 	return ret;
 }
 
-void sal_compute_sal_errors_from_code(int code ,SalError* sal_err,SalReason* sal_reason) {
+SalReason _sal_reason_from_sip_code(int code) {
+	if (code>=100 && code<300) return SalReasonNone;
+
 	switch(code) {
+	case 0:
+		return SalReasonIOError;
+	case 301:
+		return SalReasonMovedPermanently;
 	case 302:
-		*sal_reason=SalReasonRedirect;
-		*sal_err=SalErrorFailure;
-		break;
-	case 400:
-		*sal_err=SalErrorUnknown;
-		break;
+		return SalReasonRedirect;
 	case 401:
 	case 407:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonUnauthorized;
-		break;
+		return SalReasonUnauthorized;
 	case 403:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonForbidden;
-		break;
+		return SalReasonForbidden;
 	case 404:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonNotFound;
-		break;
+		return SalReasonNotFound;
+	case 408:
+		return SalReasonRequestTimeout;
+	case 410:
+		return SalReasonGone;
 	case 415:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonUnsupportedContent;
-		break;
+		return SalReasonUnsupportedContent;
 	case 422:
 		ms_error ("422 not implemented yet");;
 		break;
 	case 480:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonTemporarilyUnavailable;
-		break;
+		return SalReasonTemporarilyUnavailable;
 	case 481:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonNoMatch;
+		return SalReasonNoMatch;
+	case 484:
+		return SalReasonAddressIncomplete;
 	case 486:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonBusy;
-		break;
+		return SalReasonBusy;
 	case 487:
-		break;
+		return SalReasonNone;
 	case 488:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonNotAcceptable;
-		break;
+		return SalReasonNotAcceptable;
 	case 491:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonRequestPending;
-		break;
+		return SalReasonRequestPending;
+	case 501:
+		return SalReasonNotImplemented;
+	case 502:
+		return SalReasonBadGateway;
+	case 504:
+		return SalReasonServerTimeout;
 	case 600:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonDoNotDisturb;
-		break;
+		return SalReasonDoNotDisturb;
 	case 603:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonDeclined;
-		break;
+		return SalReasonDeclined;
 	case 503:
-		*sal_err=SalErrorFailure;
-		*sal_reason=SalReasonServiceUnavailable;
-		break;
+		return SalReasonServiceUnavailable;
 	default:
-		if (code>=300){
-			*sal_err=SalErrorFailure;
-			*sal_reason=SalReasonUnknown;
-		}else if (code>=100){
-			*sal_err=SalErrorNone;
-			*sal_reason=SalReasonUnknown;
-		}else if (code==0){
-			*sal_err=SalErrorNoResponse;
-		}
-		/* no break */
+		return SalReasonUnknown;
+	}
+	return SalReasonUnknown;
+}
+
+const SalErrorInfo *sal_error_info_none(void){
+	static SalErrorInfo none={
+		SalReasonNone,
+		"Ok",
+		200,
+		NULL,
+		NULL
+	};
+	return &none;
+}
+
+void sal_error_info_reset(SalErrorInfo *ei){
+	if (ei->status_string){
+		ms_free(ei->status_string);
+		ei->status_string=NULL;
+	}
+	if (ei->warnings){
+		ms_free(ei->warnings);
+		ei->warnings=NULL;
+
+	}
+	if (ei->full_string){
+		ms_free(ei->full_string);
+		ei->full_string=NULL;
+	}
+	ei->protocol_code=0;
+	ei->reason=SalReasonNone;
+}
+
+void sal_error_info_set(SalErrorInfo *ei, SalReason reason, int code, const char *status_string, const char *warning){
+	sal_error_info_reset(ei);
+	if (reason==SalReasonUnknown) ei->reason=_sal_reason_from_sip_code(code);
+	else ei->reason=reason;
+	ei->protocol_code=code;
+	ei->status_string=status_string ? ms_strdup(status_string) : NULL;
+	ei->warnings=warning ? ms_strdup(warning) : NULL;
+	if (ei->status_string){
+		if (ei->warnings)
+			ei->full_string=ms_strdup_printf("%s %s",ei->status_string,ei->warnings);
+		else ei->full_string=ms_strdup(ei->status_string);
 	}
 }
-/*return TRUE if error code*/
-bool_t sal_compute_sal_errors(belle_sip_response_t* response,SalError* sal_err,SalReason* sal_reason,char* reason, size_t reason_size) {
-	int code = belle_sip_response_get_status_code(response);
-	belle_sip_header_t* reason_header = belle_sip_message_get_header(BELLE_SIP_MESSAGE(response),"Reason");
-	*sal_err=SalErrorUnknown;
-	*sal_reason = SalReasonUnknown;
 
-	if (reason_header){
-		snprintf(reason
-				,reason_size
-				,"%s %s"
-				,belle_sip_response_get_reason_phrase(response)
-				,belle_sip_header_get_unparsed_value(reason_header));
-	} else {
-		strncpy(reason,belle_sip_response_get_reason_phrase(response),reason_size);
-	}
-	if (code>=400) {
-		sal_compute_sal_errors_from_code(code,sal_err,sal_reason);
-		return TRUE;
-	} else {
-		return FALSE;
-	}
+void sal_op_set_error_info_from_response(SalOp *op, belle_sip_response_t *response){
+	int code = belle_sip_response_get_status_code(response);
+	const char *reason_phrase=belle_sip_response_get_reason_phrase(response);
+	/*Remark: the reason header is to be used mainly in SIP requests, thus the use and prototype of this function should be changed.*/
+	belle_sip_header_t* reason_header = belle_sip_message_get_header(BELLE_SIP_MESSAGE(response),"Reason");
+	belle_sip_header_t *warning=belle_sip_message_get_header(BELLE_SIP_MESSAGE(response),"Warning");
+	SalErrorInfo *ei=&op->error_info;
+	const char *warnings;
+
+	warnings=warning ? belle_sip_header_get_unparsed_value(warning) : NULL;
+	if (warnings==NULL) warnings=reason_header ? belle_sip_header_get_unparsed_value(reason_header) : NULL;
+	sal_error_info_set(ei,SalReasonUnknown,code,reason_phrase,warnings);
+}
+
+const SalErrorInfo *sal_op_get_error_info(const SalOp *op){
+	return &op->error_info;
+}
+
+static void unlink_op_with_dialog(SalOp *op, belle_sip_dialog_t* dialog){
+	belle_sip_dialog_set_application_data(dialog,NULL);
+	sal_op_unref(op);
+	belle_sip_object_unref(dialog);
+}
+
+static belle_sip_dialog_t *link_op_with_dialog(SalOp *op, belle_sip_dialog_t* dialog){
+	belle_sip_dialog_set_application_data(dialog,sal_op_ref(op));
+	belle_sip_object_ref(dialog);
+	return dialog;
 }
 
 void set_or_update_dialog(SalOp* op, belle_sip_dialog_t* dialog) {
-	/*check if dialog has changed*/
-	if (dialog && dialog != op->dialog) {
-		ms_message("Dialog set from [%p] to [%p] for op [%p]",op->dialog,dialog,op);
-		/*fixme, shouldn't we cancel previous dialog*/
-		if (op->dialog) {
-			belle_sip_dialog_set_application_data(op->dialog,NULL);
-			belle_sip_object_unref(op->dialog);
-			sal_op_unref(op);
+	ms_message("op [%p] : set_or_update_dialog() current=[%p] new=[%p]",op,op->dialog,dialog);
+	sal_op_ref(op);
+	if (op->dialog!=dialog){
+		if (op->dialog){
+			/*FIXME: shouldn't we delete unconfirmed dialogs ?*/
+			unlink_op_with_dialog(op,op->dialog);
+			op->dialog=NULL;
 		}
-		op->dialog=dialog;
-		belle_sip_dialog_set_application_data(op->dialog,op);
-		sal_op_ref(op);
-		belle_sip_object_ref(op->dialog);
+		if (dialog) op->dialog=link_op_with_dialog(op,dialog);
 	}
+	sal_op_unref(op);
 }
 /*return reffed op*/
 SalOp* sal_op_ref(SalOp* op) {
@@ -517,8 +635,16 @@ int sal_op_send_and_create_refresher(SalOp* op,belle_sip_request_t* req, int exp
 			belle_sip_object_unref(op->refresher);
 		}
 		if ((op->refresher = belle_sip_client_transaction_create_refresher(op->pending_client_trans))) {
+			/*since refresher acquires the transaction, we should remove our context from the transaction, because we won't be notified
+			 * that it is terminated anymore.*/
+			sal_op_unref(op);/*loose the reference that was given to the transaction when creating it*/
+			/* Note that the refresher will replace our data with belle_sip_transaction_set_application_data().
+			 Something in the design is not very good here, it makes things complicated to the belle-sip user.
+			 Possible ideas to improve things: refresher shall not use belle_sip_transaction_set_application_data() internally, refresher should let the first transaction
+			 notify the user as a normal transaction*/
 			belle_sip_refresher_set_listener(op->refresher,listener,op);
 			belle_sip_refresher_set_retry_after(op->refresher,op->base.root->refresher_retry_after);
+			belle_sip_refresher_set_realm(op->refresher,op->base.realm);
 			belle_sip_refresher_enable_manual_mode(op->refresher,op->manual_refresher);
 			return 0;
 		} else {
@@ -564,7 +690,7 @@ void sal_op_assign_recv_headers(SalOp *op, belle_sip_message_t *incoming){
 }
 
 const char *sal_op_get_remote_contact(const SalOp *op){
-	/* 
+	/*
 	 * remote contact is filled in process_response
 	 * return sal_custom_header_find(op->base.recv_custom_headers,"Contact");
 	 */
@@ -595,16 +721,16 @@ bool_t sal_op_get_body(SalOp *op, belle_sip_message_t *msg, SalBody *salbody){
 	belle_sip_header_content_type_t *content_type;
 	belle_sip_header_content_length_t *clen=NULL;
 	belle_sip_header_t *content_encoding;
-	
+
 	content_type=belle_sip_message_get_header_by_type(msg,belle_sip_header_content_type_t);
 	if (content_type){
 		body=belle_sip_message_get_body(msg);
 		clen=belle_sip_message_get_header_by_type(msg,belle_sip_header_content_length_t);
 	}
 	content_encoding=belle_sip_message_get_header(msg,"Content-encoding");
-	
+
 	memset(salbody,0,sizeof(SalBody));
-	
+
 	if (content_type && body && clen) {
 		salbody->type=belle_sip_header_content_type_get_type(content_type);
 		salbody->subtype=belle_sip_header_content_type_get_subtype(content_type);
@@ -639,7 +765,7 @@ bool_t sal_op_is_ipv6(SalOp *op){
 	belle_sip_transaction_t *tr=NULL;
 	belle_sip_header_address_t *contact;
 	belle_sip_request_t *req;
-	
+
 	if (op->refresher)
 		tr=(belle_sip_transaction_t *)belle_sip_refresher_get_transaction(op->refresher);
 
@@ -647,7 +773,7 @@ bool_t sal_op_is_ipv6(SalOp *op){
 		tr=(belle_sip_transaction_t *)op->pending_client_trans;
 	if (tr==NULL)
 		tr=(belle_sip_transaction_t *)op->pending_server_trans;
-	
+
 	if (tr==NULL){
 		ms_error("Unable to determine IP version from signaling operation.");
 		return FALSE;
@@ -667,4 +793,13 @@ bool_t sal_op_is_idle(SalOp *op){
 	return TRUE;
 }
 
+void sal_op_stop_refreshing(SalOp *op){
+	if (op->refresher){
+		belle_sip_refresher_stop(op->refresher);
+	}
+}
 
+void sal_call_set_sdp_handling(SalOp *h, SalOpSDPHandling handling)  {
+	if (handling != SalOpSDPNormal) ms_message("Enabling special SDP handling for SalOp[%p]!", h);
+	h->sdp_handling = handling;
+}
