@@ -36,29 +36,49 @@ static void xml2lpc_callback(void *ctx, xml2lpc_log_level level, const char *fmt
 static void linphone_remote_provisioning_apply(LinphoneCore *lc, const char *xml) {
 	xml2lpc_context *context = xml2lpc_context_new(xml2lpc_callback, lc);
 	int result = xml2lpc_set_xml_string(context, xml);
+	char * error_msg = NULL;
 	if (result == 0) {
-		result = xml2lpc_convert(context, linphone_core_get_config(lc));
+		LpConfig * lpc = linphone_core_get_config(lc);
+		result = xml2lpc_convert(context, lpc);
 		if (result == 0) {
-			lp_config_sync(linphone_core_get_config(lc));
-			xml2lpc_context_destroy(context);
-			linphone_configuring_terminated(lc, LinphoneConfiguringSuccessful, NULL);
+			// if the remote provisioning added a proxy config and none was set before, set it
+			if (lp_config_has_section(lpc, "proxy_0") && lp_config_get_int(lpc, "sip", "default_proxy", -1) == -1){
+				lp_config_set_int(lpc, "sip", "default_proxy", 0);
+			}
+			lp_config_sync(lpc);
+
 		} else {
-			xml2lpc_context_destroy(context);
-			linphone_configuring_terminated(lc, LinphoneConfiguringFailed, "xml to lpc failed");
+			error_msg = "xml to lpc failed";
 		}
 	} else {
-		xml2lpc_context_destroy(context);
-		linphone_configuring_terminated(lc, LinphoneConfiguringFailed, "invalid xml");
+		error_msg = "invalid xml";
 	}
+
+	xml2lpc_context_destroy(context);
+	linphone_configuring_terminated(lc
+									,error_msg ? LinphoneConfiguringFailed : LinphoneConfiguringSuccessful
+									, error_msg);
+}
+
+int linphone_remote_provisioning_load_file( LinphoneCore* lc, const char* file_path){
+	int status = -1;
+	char* provisioning=ms_load_path_content(file_path, NULL);
+
+	if (provisioning){
+		linphone_remote_provisioning_apply(lc, provisioning);
+		status = 0;
+		ms_free(provisioning);
+	}
+	return status;
 }
 
 static void belle_request_process_response_event(void *ctx, const belle_http_response_event_t *event) {
 	LinphoneCore *lc = (LinphoneCore *)ctx;
-	belle_sip_message_t *body = BELLE_SIP_MESSAGE(event->response);
-	const char *message = belle_sip_message_get_body(body);
-	
+	belle_sip_message_t *message = BELLE_SIP_MESSAGE(event->response);
+	const char *body = belle_sip_message_get_body(message);
+
 	if (belle_http_response_get_status_code(event->response) == 200) {
-		linphone_remote_provisioning_apply(lc, message);
+		linphone_remote_provisioning_apply(lc, body);
 	} else {
 		linphone_configuring_terminated(lc, LinphoneConfiguringFailed, "http error");
 	}
@@ -80,31 +100,43 @@ static void belle_request_process_auth_requested(void *ctx, belle_sip_auth_event
 }
 
 int linphone_remote_provisioning_download_and_apply(LinphoneCore *lc, const char *remote_provisioning_uri) {
+
 	belle_generic_uri_t *uri=belle_generic_uri_parse(remote_provisioning_uri);
-	belle_http_request_listener_callbacks_t belle_request_listener = {
-		belle_request_process_response_event,
-		belle_request_process_io_error,
-		belle_request_process_timeout,
-		belle_request_process_auth_requested
-	};
-	belle_http_request_listener_t *listener = belle_http_request_listener_create_from_callbacks(&belle_request_listener, lc);
-	belle_http_request_t *request;
-	
-	if (uri==NULL) {
-		belle_sip_error("Invalid provisioning URI [%s]",remote_provisioning_uri);
+	const char* scheme = uri ? belle_generic_uri_get_scheme(uri) : NULL;
+
+	if( scheme && (strcmp(scheme,"file") == 0) ){
+		// We allow for 'local remote-provisioning' in case the file is to be opened from the hard drive.
+		const char* file_path = remote_provisioning_uri + strlen("file://"); // skip scheme
+		return linphone_remote_provisioning_load_file(lc, file_path);
+
+	} else if( scheme && strncmp(scheme, "http", 4) == 0 ) {
+		belle_http_request_listener_callbacks_t belle_request_listener={0};
+		belle_http_request_listener_t *listener;
+		belle_http_request_t *request;
+
+		belle_request_listener.process_response=belle_request_process_response_event;
+		belle_request_listener.process_auth_requested=belle_request_process_auth_requested;
+		belle_request_listener.process_io_error=belle_request_process_io_error;
+		belle_request_listener.process_timeout=belle_request_process_timeout;
+
+		listener = belle_http_request_listener_create_from_callbacks(&belle_request_listener, lc);
+
+		request=belle_http_request_create("GET",uri, NULL);
+		return belle_http_provider_send_request(lc->http_provider, request, listener);
+	} else {
+		ms_error("Invalid provisioning URI [%s] (missing scheme?)",remote_provisioning_uri);
 		return -1;
 	}
-	request=belle_http_request_create("GET",uri, NULL);
-	belle_http_provider_send_request(lc->http_provider, request, listener);
-	return 0;
 }
 
-void linphone_core_set_provisioning_uri(LinphoneCore *lc, const char*uri){
-	if (linphone_core_ready(lc)){
-		lp_config_set_string(lc->config,"misc","config-uri",uri);
-	}
+void linphone_core_set_provisioning_uri(LinphoneCore *lc, const char *uri) {
+	lp_config_set_string(lc->config,"misc","config-uri",uri);
 }
 
 const char*linphone_core_get_provisioning_uri(const LinphoneCore *lc){
 	return lp_config_get_string(lc->config,"misc","config-uri",NULL);
 }
+
+bool_t linphone_core_is_provisioning_transient(LinphoneCore *lc) {
+	return lp_config_get_int(lc->config, "misc", "transient_provisioning", 0) == 1;
+}
