@@ -23,6 +23,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <gtkosxapplication.h>
 #endif
 
+#if defined(WIN32) && !defined(F_OK)
+#define F_OK 00 /*visual studio does not define F_OK*/
+#endif
+
 #define NB_MSG_HIST 250
 
 #define CONFIG_FILE ".linphone-history.db"
@@ -30,9 +34,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 const char *linphone_gtk_message_storage_get_db_file(const char *filename){
 	const int path_max=1024;
 	static char *db_file=NULL;
-	
+
 	if (db_file) return db_file;
-	
+
 	db_file=(char *)malloc(path_max*sizeof(char));
 	if (filename==NULL) filename=CONFIG_FILE;
 	/*try accessing a local file first if exists*/
@@ -63,7 +67,7 @@ void linphone_gtk_quit_chatroom(LinphoneChatRoom *cr) {
 	GtkWidget *w=g_object_get_data(G_OBJECT(friendlist),"chatview");
 	gchar *from;
 	GHashTable *table=g_object_get_data(G_OBJECT(w),"table");
-	
+
 	g_return_if_fail(w!=NULL);
 	gtk_notebook_remove_page(GTK_NOTEBOOK(nb),gtk_notebook_page_num(GTK_NOTEBOOK(nb),w));
 	linphone_chat_room_mark_as_read(cr);
@@ -76,7 +80,7 @@ void linphone_gtk_quit_chatroom(LinphoneChatRoom *cr) {
 	}
 	g_hash_table_destroy(table);
 	g_object_set_data(G_OBJECT(w),"cr",NULL);
-	g_object_set_data(G_OBJECT(friendlist),"from",NULL);
+	linphone_gtk_friend_list_set_active_address(NULL);
 	gtk_widget_destroy(w);
 }
 
@@ -95,7 +99,7 @@ GtkWidget *create_tab_chat_header(LinphoneChatRoom *cr,const LinphoneAddress *ur
 	GtkWidget *l;
 	GtkWidget *image=gtk_image_new_from_stock(GTK_STOCK_CLOSE,GTK_ICON_SIZE_MENU);
 	GtkWidget *b=gtk_button_new();
-	
+
 	gtk_button_set_image(GTK_BUTTON(b),image);
 	gtk_button_set_relief(GTK_BUTTON(b),GTK_RELIEF_NONE);
 	gtk_widget_set_size_request(b,25,20);
@@ -131,14 +135,15 @@ void udpate_tab_chat_header(GtkWidget *chat_view,const LinphoneAddress *uri,Linp
 
 static gboolean scroll_to_end(GtkTextView *w){
 	GtkTextBuffer *buffer=gtk_text_view_get_buffer(w);
+	GtkTextMark *mark;
 	GtkTextIter iter;
 	gtk_text_buffer_get_end_iter(buffer,&iter);
-	GtkTextMark *mark=gtk_text_buffer_create_mark(buffer,NULL,&iter,FALSE);
-	gtk_text_view_scroll_mark_onscreen(w,mark); 
+	mark=gtk_text_buffer_create_mark(buffer,NULL,&iter,FALSE);
+	gtk_text_view_scroll_mark_onscreen(w,mark);
 	return FALSE;
 }
 
-void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from, 
+void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
                  gboolean me,LinphoneChatRoom *cr,LinphoneChatMessage *msg, gboolean hist){
 	GtkTextView *text=GTK_TEXT_VIEW(linphone_gtk_get_widget(w,"textview"));
 	GtkTextBuffer *buffer=gtk_text_view_get_buffer(text);
@@ -153,23 +158,26 @@ void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
 	struct tm *tm;
 	int tnow_day;
 	int tnow_year;
-	
+
 	gtk_text_buffer_get_start_iter(buffer,&begin);
 	gtk_text_buffer_get_end_iter(buffer,&iter);
 	off=gtk_text_iter_get_offset(&iter);
 	if(g_strcmp0(from_message,from_str)!=0){
 		gtk_text_buffer_get_iter_at_offset(buffer,&iter,off);
 		gtk_text_buffer_get_end_iter(buffer,&iter);
-		gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,get_display_name(from),-1,"bold",me ? "bg":NULL,NULL);
+		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, get_display_name(from), -1,
+		                                         "bold", me ? "bg" : NULL, me ? "font_black" : NULL, NULL);
 		gtk_text_buffer_get_end_iter(buffer,&iter);
-		gtk_text_buffer_insert_with_tags_by_name(buffer,&iter," : ",-1,"bold",me ? "bg":NULL,NULL);
+		gtk_text_buffer_insert_with_tags_by_name(buffer,&iter, " : ", -1,
+		                                         "bold", me ? "bg" : NULL, me ? "font_black" : NULL, NULL);
 		gtk_text_buffer_get_end_iter(buffer,&iter);
 		gtk_text_buffer_insert(buffer,&iter,"\n",-1);
 		g_free(from_message);
 		g_object_set_data(G_OBJECT(w),"from_message",g_strdup(from_str));
 	}
 	gtk_text_buffer_get_end_iter(buffer,&iter);
-	gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,linphone_chat_message_get_text(msg),-1,"margin",me ? "bg":NULL,NULL);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, linphone_chat_message_get_text(msg), -1,
+	                                         "margin", me ? "bg" : NULL, me ? "font_black" : NULL, NULL);
 	gtk_text_buffer_get_end_iter(buffer,&iter);
 	gtk_text_buffer_insert(buffer,&iter,"\n",-1);
 	gtk_text_buffer_get_end_iter(buffer,&iter);
@@ -178,7 +186,7 @@ void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
 		case LinphoneChatMessageStateInProgress:
 		{
 			g_hash_table_insert(table,(gpointer)msg,GINT_TO_POINTER(gtk_text_iter_get_line(&iter)));
-			gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,"Sending .. ",-1,									
+			gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,"Sending ..",-1,
 		                                "right","small","italic","font_grey","bg",NULL);
 			g_object_set_data(G_OBJECT(w),"table",table);
 			break;
@@ -195,15 +203,15 @@ void linphone_gtk_push_text(GtkWidget *w, const LinphoneAddress *from,
 			} else {
 				strftime(buf,80,"%H:%M",tm);
 			}
-			gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,buf,-1,									
+			gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,buf,-1,
 	                      "right","small","italic","font_grey",me ? "bg":NULL,NULL);
 			break;
 		}
 		case  LinphoneChatMessageStateNotDelivered:
-				gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,"Message not sent",-1,									
+				gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,"Message not sent",-1,
 	                       "right","small","italic","font_grey",me ? "bg":NULL,NULL);
 				break;
-		default : gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,"Sending ..",-1,									
+		default : gtk_text_buffer_insert_with_tags_by_name(buffer,&iter,"Sending ..",-1,
 	                       "right","small","italic","font_grey",me ? "bg":NULL,NULL);
 	}
 	gtk_text_buffer_get_end_iter(buffer,&iter);
@@ -225,7 +233,7 @@ void update_chat_state_message(LinphoneChatMessageState state,LinphoneChatMessag
 	GtkWidget *friendlist=linphone_gtk_get_widget(main_window,"contact_list");
 	GtkWidget *page=(GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
 	GHashTable *table=(GHashTable*)g_object_get_data(G_OBJECT(page),"table");
-	
+
 	if(page!=NULL){
 		GtkTextView *text=GTK_TEXT_VIEW(linphone_gtk_get_widget(page,"textview"));
 		GtkTextBuffer *b=gtk_text_view_get_buffer(text);
@@ -249,7 +257,7 @@ void update_chat_state_message(LinphoneChatMessageState state,LinphoneChatMessag
 
 		switch (state) {
 			case LinphoneChatMessageStateInProgress:
-				result="Sending ";
+				result="Sending ..";
 				break;
 			case LinphoneChatMessageStateDelivered:
 			{
@@ -258,24 +266,37 @@ void update_chat_state_message(LinphoneChatMessageState state,LinphoneChatMessag
 				char buf[80];
 				strftime(buf,80,"%H:%M",tm);
 				result=buf;
+				g_hash_table_remove(table,msg);
 				break;
 			}
 			case  LinphoneChatMessageStateNotDelivered:
+			{
 				result="Message not sent";
+				g_hash_table_remove(table,msg);
 				break;
+			}
 			default : result="Sending ..";
 		}
 		gtk_text_buffer_insert_with_tags_by_name(b,&iter,result,-1,
 												"right","small","italic","font_grey","bg",NULL);
-		g_hash_table_remove(table,msg);
 		g_object_set_data(G_OBJECT(page),"table",table);
-	} 
+	}
 }
 
-static void on_chat_state_changed(LinphoneChatMessage *msg, LinphoneChatMessageState state, void *user_pointer){
+static void on_chat_state_changed(LinphoneChatMessage *msg, LinphoneChatMessageState state){
 	update_chat_state_message(state,msg);
 }
 
+void linphone_gtk_compose_text(void) {
+	GtkWidget *main_window=linphone_gtk_get_main_window();
+	GtkWidget *friendlist=linphone_gtk_get_widget(main_window,"contact_list");
+	GtkWidget *w=(GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
+	LinphoneChatRoom *cr=g_object_get_data(G_OBJECT(w),"cr");
+	if (cr) {
+		linphone_chat_room_compose(cr);
+	}
+}
+
 void linphone_gtk_send_text(){
 	GtkWidget *main_window=linphone_gtk_get_main_window();
 	GtkWidget *friendlist=linphone_gtk_get_widget(main_window,"contact_list");
@@ -286,11 +307,18 @@ void linphone_gtk_send_text(){
 	entered=gtk_entry_get_text(GTK_ENTRY(entry));
 	if (strlen(entered)>0) {
 		LinphoneChatMessage *msg;
+		LinphoneChatMessageCbs *cbs;
 		msg=linphone_chat_room_create_message(cr,entered);
-		linphone_chat_room_send_message2(cr,msg,on_chat_state_changed,NULL);
+		cbs=linphone_chat_message_get_callbacks(msg);
+		linphone_chat_message_cbs_set_msg_state_changed(cbs,on_chat_state_changed);
+		linphone_chat_room_send_chat_message(cr,msg);
 		linphone_gtk_push_text(w,linphone_chat_message_get_from(msg),
 				TRUE,cr,msg,FALSE);
+
+		// Disconnect and reconnect the "changed" signal to prevent triggering it when clearing the text entry.
+		g_signal_handlers_disconnect_by_func(G_OBJECT(entry),(GCallback)linphone_gtk_compose_text,NULL);
 		gtk_entry_set_text(GTK_ENTRY(entry),"");
+		g_signal_connect_swapped(G_OBJECT(entry),"changed",(GCallback)linphone_gtk_compose_text,NULL);
 	}
 }
 
@@ -313,8 +341,8 @@ void display_history_message(GtkWidget *chat_view,MSList *messages,const Linphon
 			LinphoneChatMessage *msg=(LinphoneChatMessage *)it->data;
 			from_str=linphone_address_as_string_uri_only(linphone_chat_message_get_from(msg));
 			with_str=linphone_address_as_string_uri_only(with);
-			linphone_gtk_push_text(chat_view,strcmp(from_str,with_str)==0? with : 
-				                       linphone_chat_message_get_from(msg), 
+			linphone_gtk_push_text(chat_view,strcmp(from_str,with_str)==0? with :
+				                       linphone_chat_message_get_from(msg),
             						strcmp(from_str,with_str)==0? FALSE : TRUE,
 			                        linphone_chat_message_get_chat_room(msg),msg,TRUE);
 		}
@@ -326,52 +354,47 @@ void display_history_message(GtkWidget *chat_view,MSList *messages,const Linphon
 		ms_free(from_str);
 		ms_free(with_str);
 		linphone_gtk_free_list(messages);
-	} 
+	}
 }
 
 void linphone_gtk_chat_add_contact(const LinphoneAddress *addr){
 	LinphoneFriend *lf=NULL;
-	char *uri=linphone_address_as_string(addr);
-	lf=linphone_friend_new_with_addr(uri);
-	ms_free(uri);
-	char *fixed_uri=NULL;
+	LinphoneAddress *fixed_uri=NULL;
 	gboolean show_presence=FALSE;
+	char *uri=linphone_address_as_string(addr);
+
+	lf=linphone_friend_new_with_address(uri);
+	ms_free(uri);
 
 	linphone_friend_set_inc_subscribe_policy(lf,LinphoneSPDeny);
 	linphone_friend_send_subscribe(lf,show_presence);
 
-	linphone_core_interpret_friend_uri(linphone_gtk_get_core(),uri,&fixed_uri);
+	fixed_uri = linphone_core_interpret_url(linphone_gtk_get_core(),uri);
 	if (fixed_uri==NULL){
 		linphone_gtk_display_something(GTK_MESSAGE_WARNING,_("Invalid sip contact !"));
 		return ;
 	}
-	linphone_friend_set_addr(lf,addr);
+	linphone_friend_set_address(lf,addr);
 	linphone_core_add_friend(linphone_gtk_get_core(),lf);
-	ms_free(fixed_uri);
+	linphone_address_destroy(fixed_uri);
 	linphone_gtk_show_friends();
 }
 
 GtkWidget* linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddress *with){
 	GtkWidget *chat_view=linphone_gtk_create_widget("main","chatroom_frame");
-	GtkWidget *main_window=linphone_gtk_get_main_window ();
+	GtkWidget *main_window=linphone_gtk_get_main_window();
 	GtkNotebook *notebook=(GtkNotebook *)linphone_gtk_get_widget(main_window,"viewswitch");
 	GtkWidget *text=linphone_gtk_get_widget(chat_view,"textview");
-	GdkColor color;
-	GdkColor colorb;
+	GdkColor color_grey = {0, 32512, 32512, 32512};
+	GdkColor color_light_grey = {0, 56832, 60928, 61952};
+	GdkColor color_black = {0};
 	int idx;
 	GtkWidget *button;
-	GtkWidget *entry;
+	GtkWidget *entry = linphone_gtk_get_widget(chat_view,"text_entry");
 	MSList *messages;
 	GHashTable *table;
 	char *with_str;
 
-	color.red = 32512;
-  	color.green = 32512;
-  	color.blue = 32512;
-	colorb.red = 56832;
-  	colorb.green = 60928;
-  	colorb.blue = 61952;
-	
 	with_str=linphone_address_as_string_uri_only(with);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text),GTK_WRAP_WORD_CHAR);
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(text),FALSE);
@@ -395,17 +418,20 @@ GtkWidget* linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddres
 	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
 	                           	"small","size",9*PANGO_SCALE,NULL);
 	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
-	                           	"font_grey","foreground-gdk",&color,NULL);
+	                           	"font_grey","foreground-gdk",&color_grey,NULL);
+	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
+	                           	"font_black","foreground-gdk",&color_black,NULL);
 	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
 	                           	"margin","indent",10,NULL);
 	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
-	                           	"bg","paragraph-background-gdk",&colorb,NULL);
+	                           	"bg","paragraph-background-gdk",&color_light_grey,NULL);
 	messages = linphone_chat_room_get_history(cr,NB_MSG_HIST);
 	display_history_message(chat_view,messages,with);
 	button = linphone_gtk_get_widget(chat_view,"send");
 	g_signal_connect_swapped(G_OBJECT(button),"clicked",(GCallback)linphone_gtk_send_text,NULL);
-	entry = linphone_gtk_get_widget(chat_view,"text_entry");
+	
 	g_signal_connect_swapped(G_OBJECT(entry),"activate",(GCallback)linphone_gtk_send_text,NULL);
+	g_signal_connect_swapped(G_OBJECT(entry),"changed",(GCallback)linphone_gtk_compose_text,NULL);
 	g_signal_connect(G_OBJECT(notebook),"switch_page",(GCallback)linphone_gtk_notebook_tab_select,NULL);
 	ms_free(with_str);
 	return chat_view;
@@ -413,7 +439,7 @@ GtkWidget* linphone_gtk_init_chatroom(LinphoneChatRoom *cr, const LinphoneAddres
 
 LinphoneChatRoom * linphone_gtk_create_chatroom(const LinphoneAddress *with){
 	char *tmp=linphone_address_as_string(with);
-	LinphoneChatRoom *cr=linphone_core_create_chat_room(linphone_gtk_get_core(),tmp);
+	LinphoneChatRoom *cr=linphone_core_get_or_create_chat_room(linphone_gtk_get_core(),tmp);
 	ms_free(tmp);
 	return cr;
 }
@@ -426,7 +452,7 @@ void linphone_gtk_load_chatroom(LinphoneChatRoom *cr,const LinphoneAddress *uri,
 	char *uri_str=linphone_address_as_string(uri);
 	char *uri_only=linphone_address_as_string_uri_only(uri);
 	MSList *messages=NULL;
-	
+
 	if(g_strcmp0(from_str,uri_only)!=0){
 		GtkTextView *text_view=GTK_TEXT_VIEW(linphone_gtk_get_widget(chat_view,"textview"));
 		GtkTextIter start;
@@ -451,8 +477,9 @@ void linphone_gtk_load_chatroom(LinphoneChatRoom *cr,const LinphoneAddress *uri,
 }
 
 void linphone_gtk_chat_destroyed(GtkWidget *w){
+	/*
 	LinphoneChatRoom *cr=(LinphoneChatRoom*)g_object_get_data(G_OBJECT(w),"cr");
-	linphone_chat_room_destroy(cr);
+	*/
 }
 
 
@@ -463,12 +490,13 @@ void linphone_gtk_text_received ( LinphoneCore *lc, LinphoneChatRoom *room,
 	GtkWidget *w;
 	gboolean send=TRUE;
 	/*GtkNotebook *notebook= ( GtkNotebook * ) linphone_gtk_get_widget ( main_window,"viewswitch" );*/
-	char *from=linphone_address_as_string ( linphone_chat_message_get_from ( msg ) );
+	const LinphoneAddress *from= linphone_chat_message_get_from ( msg );
 
 	w= ( GtkWidget* ) g_object_get_data ( G_OBJECT ( friendlist ),"chatview" );
 	if ( w!=NULL ) {
-		char *from_chatview= ( char * ) g_object_get_data ( G_OBJECT ( friendlist ),"from" );
-		if ( g_strcmp0 ( from,from_chatview ) ==0 ) {
+	/* Chat window opened */
+		const LinphoneAddress *from_chatview=linphone_gtk_friend_list_get_active_address();
+		if (linphone_address_weak_equal(from,from_chatview)) {
 			send=TRUE;
 		} else {
 			if ( !linphone_gtk_friend_list_is_contact ( linphone_chat_message_get_from ( msg ) ) ) {
@@ -477,13 +505,18 @@ void linphone_gtk_text_received ( LinphoneCore *lc, LinphoneChatRoom *room,
 			send=FALSE;
 		}
 	} else {
+	/* Chat window closed */
+#ifdef MSG_STORAGE_ENABLED
 		send=FALSE;
+#else
+		send=TRUE;
+#endif
 		if ( !linphone_gtk_friend_list_is_contact ( linphone_chat_message_get_from ( msg ) ) ) {
 			linphone_gtk_chat_add_contact ( linphone_chat_message_get_from ( msg ) );
 		}
 		w=linphone_gtk_init_chatroom ( room,linphone_chat_message_get_from ( msg ) );
 		g_object_set_data ( G_OBJECT ( friendlist ),"chatview", ( gpointer ) w );
-		g_object_set_data ( G_OBJECT ( friendlist ),"from",from );
+		linphone_gtk_friend_list_set_active_address(from);
 	}
 
 #ifdef HAVE_GTK_OSX
@@ -503,6 +536,11 @@ void linphone_gtk_text_received ( LinphoneCore *lc, LinphoneChatRoom *room,
 		linphone_gtk_push_text ( w,linphone_chat_message_get_from ( msg ),
 								 FALSE,room,msg,FALSE );
 	}
+	linphone_core_play_local(lc,linphone_gtk_get_sound_path("incoming_chat.wav"));
 	linphone_gtk_show_friends();
-	
+
+}
+
+void linphone_gtk_is_composing_received(LinphoneCore *lc, LinphoneChatRoom *room) {
+	linphone_gtk_friend_list_update_chat_picture();
 }
