@@ -19,7 +19,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "linphone.h"
 
-void linphone_gtk_login_frame_connect_clicked(GtkWidget *button);
 void test_button_clicked_cb(GtkWidget *button);
 void linphone_gtk_exit_login_frame(void);
 
@@ -122,7 +121,7 @@ void linphone_gtk_exit_login_frame(void){
 	gtk_widget_show(linphone_gtk_get_widget(mw,"disconnect_item"));
 }
 
-void linphone_gtk_logout_clicked(){
+void linphone_gtk_logout_clicked(void){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	LinphoneProxyConfig *cfg=NULL;
 	linphone_core_get_default_proxy(lc,&cfg);
