@@ -22,11 +22,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "linphone.h"
 #include "lpconfig.h"
+#include "liblinphone_gitversion.h"
 
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 
 #ifdef HAVE_GTK_OSX
 #include <gtkosxapplication.h>
@@ -35,6 +38,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifdef WIN32
 #define chdir _chdir
 #include "direct.h"
+#ifndef F_OK
+#define F_OK 00 /*visual studio does not define F_OK*/
+#endif
 #endif
 
 #if defined(HAVE_NOTIFY1) || defined(HAVE_NOTIFY4)
@@ -45,7 +51,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <libnotify/notify.h>
 #endif
 
-#define LINPHONE_ICON "linphone.png"
+#ifdef ENABLE_NLS
+#include <locale.h>
+#endif
+
+
 
 const char *this_program_ident_string="linphone_ident_string=" LINPHONE_VERSION;
 
@@ -74,6 +84,7 @@ void linphone_gtk_status_icon_set_blinking(gboolean val);
 void _linphone_gtk_enable_video(gboolean val);
 void linphone_gtk_on_uribar_changed(GtkEditable *uribar, gpointer user_data);
 static void linphone_gtk_init_ui(void);
+static void linphone_gtk_quit(void);
 
 #ifndef HAVE_GTK_OSX
 static gint main_window_x=0;
@@ -83,8 +94,11 @@ static gboolean verbose=0;
 static gboolean quit_done=FALSE;
 static gboolean auto_answer = 0;
 static gchar * addr_to_call = NULL;
+static int start_option = START_LINPHONE;
 static gboolean no_video=FALSE;
 static gboolean iconified=FALSE;
+static gboolean run_audio_assistant=FALSE;
+static gboolean selftest=FALSE;
 static gchar *workingdir=NULL;
 static char *progpath=NULL;
 gchar *linphone_logfile=NULL;
@@ -93,63 +107,43 @@ static gchar *custom_config_file=NULL;
 static gboolean restart=FALSE;
 static GtkWidget *config_fetching_dialog=NULL;
 
+#if _MSC_VER
+
+#define LINPHONE_OPTION(optlname, optsname, optarg, optargdata, optdesc) \
+{ \
+	optlname, \
+	optsname, \
+	0, \
+	optarg, \
+	optargdata, \
+	optdesc, \
+	NULL \
+}
+
+#else
+
+#define LINPHONE_OPTION(optlname, optsname, optarg, optargdata, optdesc) \
+{ \
+	.long_name = optlname, \
+	.short_name = optsname, \
+	.arg = optarg, \
+	.arg_data = optargdata, \
+	.description = optdesc, \
+}
+
+#endif
+
 static GOptionEntry linphone_options[]={
-	{
-		.long_name="verbose",
-		.short_name= '\0',
-		.arg=G_OPTION_ARG_NONE,
-		.arg_data= (gpointer)&verbose,
-		.description=N_("log to stdout some debug information while running.")
-	},
-	{
-	    .long_name = "logfile",
-	    .short_name = 'l',
-	    .arg = G_OPTION_ARG_STRING,
-	    .arg_data = &linphone_logfile,
-	    .description = N_("path to a file to write logs into.")
-	},
-	{
-	    .long_name = "no-video",
-	    .short_name = '\0',
-	    .arg = G_OPTION_ARG_NONE,
-	    .arg_data = (gpointer)&no_video,
-	    .description = N_("Start linphone with video disabled.")
-	},
-	{
-		.long_name="iconified",
-		.short_name= '\0',
-		.arg=G_OPTION_ARG_NONE,
-		.arg_data= (gpointer)&iconified,
-		.description=N_("Start only in the system tray, do not show the main interface.")
-	},
-	{
-	    .long_name = "call",
-	    .short_name = 'c',
-	    .arg = G_OPTION_ARG_STRING,
-	    .arg_data = &addr_to_call,
-	    .description = N_("address to call right now")
-	},
-	{
-	    .long_name = "auto-answer",
-	    .short_name = 'a',
-	    .arg = G_OPTION_ARG_NONE,
-	    .arg_data = (gpointer) & auto_answer,
-	    .description = N_("if set automatically answer incoming calls")
-	},
-	{
-	    .long_name = "workdir",
-	    .short_name = '\0',
-	    .arg = G_OPTION_ARG_STRING,
-	    .arg_data = (gpointer) & workingdir,
-	    .description = N_("Specifiy a working directory (should be the base of the installation, eg: c:\\Program Files\\Linphone)")
-	},
-	{
-		.long_name = "config",
-		.short_name = '\0',
-		.arg = G_OPTION_ARG_FILENAME,
-		.arg_data = (gpointer) &custom_config_file,
-		.description = N_("Configuration file")
-	},
+	LINPHONE_OPTION("verbose",             '\0', G_OPTION_ARG_NONE,     (gpointer)&verbose,              N_("log to stdout some debug information while running.")),
+	LINPHONE_OPTION("logfile",             'l',  G_OPTION_ARG_STRING,   &linphone_logfile,               N_("path to a file to write logs into.")),
+	LINPHONE_OPTION("no-video",            '\0', G_OPTION_ARG_NONE,     (gpointer)&no_video,             N_("Start linphone with video disabled.")),
+	LINPHONE_OPTION("iconified",           '\0', G_OPTION_ARG_NONE,     (gpointer)&iconified,            N_("Start only in the system tray, do not show the main interface.")),
+	LINPHONE_OPTION("call",                'c',  G_OPTION_ARG_STRING,   &addr_to_call,                   N_("address to call right now")),
+	LINPHONE_OPTION("auto-answer",         'a',  G_OPTION_ARG_NONE,     (gpointer) & auto_answer,        N_("if set automatically answer incoming calls")),
+	LINPHONE_OPTION("workdir",             '\0', G_OPTION_ARG_STRING,   (gpointer) & workingdir,         N_("Specifiy a working directory (should be the base of the installation, eg: c:\\Program Files\\Linphone)")),
+	LINPHONE_OPTION("config",              '\0', G_OPTION_ARG_FILENAME, (gpointer) &custom_config_file,  N_("Configuration file")),
+	LINPHONE_OPTION("run-audio-assistant", '\0', G_OPTION_ARG_NONE,     (gpointer) &run_audio_assistant, N_("Run the audio assistant")),
+	LINPHONE_OPTION("selftest",            '\0', G_OPTION_ARG_NONE,     (gpointer) &selftest,            N_("Run self test and exit 0 if succeed")),
 	{0}
 };
 
@@ -160,9 +154,11 @@ static GOptionEntry linphone_options[]={
 #ifndef WIN32
 #define CONFIG_FILE ".linphonerc"
 #define SECRETS_FILE ".linphone-zidcache"
+#define CERTIFICATES_PATH ".linphone-usr-crt"
 #else
 #define CONFIG_FILE "linphonerc"
 #define SECRETS_FILE "linphone-zidcache"
+#define CERTIFICATES_PATH "linphone-usr-crt"
 #endif
 
 char *linphone_gtk_get_config_file(const char *filename){
@@ -259,10 +255,15 @@ void linphone_gtk_schedule_restart(void){
 	restart=TRUE;
 }
 
+gboolean linphone_gtk_get_audio_assistant_option(void){
+	return run_audio_assistant;
+}
+
 static void linphone_gtk_init_liblinphone(const char *config_file,
 		const char *factory_config_file, const char *db_file) {
 	LinphoneCoreVTable vtable={0};
 	gchar *secrets_file=linphone_gtk_get_config_file(SECRETS_FILE);
+	gchar *user_certificates_dir=linphone_gtk_get_config_file(CERTIFICATES_PATH);
 
 	vtable.global_state_changed=linphone_gtk_global_state_changed;
 	vtable.call_state_changed=linphone_gtk_call_state_changed;
@@ -300,8 +301,11 @@ static void linphone_gtk_init_liblinphone(const char *config_file,
 	linphone_core_set_waiting_callback(the_core,linphone_gtk_wait,NULL);
 	linphone_core_set_zrtp_secrets_file(the_core,secrets_file);
 	g_free(secrets_file);
+	linphone_core_set_user_certificates_path(the_core,user_certificates_dir);
+	g_free(user_certificates_dir);
 	linphone_core_enable_video_capture(the_core, TRUE);
 	linphone_core_enable_video_display(the_core, TRUE);
+	linphone_core_set_native_video_window_id(the_core,-1);/*don't create the window*/
 	if (no_video) {
 		_linphone_gtk_enable_video(FALSE);
 		linphone_gtk_set_ui_config_int("videoselfview",0);
@@ -402,9 +406,9 @@ GtkWidget *linphone_gtk_create_widget(const char *filename, const char *widget_n
 	object_ids[1]=NULL;
 
 	if (get_ui_file(filename,path,sizeof(path))==-1) return NULL;
-	
+
 	gtk_builder_set_translation_domain(builder,GETTEXT_PACKAGE);
-	
+
 	if (!gtk_builder_add_objects_from_file(builder,path,object_ids,&error)){
 		g_error("Couldn't load %s from builder file %s: %s", widget_name,path,error->message);
 		g_error_free (error);
@@ -423,9 +427,9 @@ GtkWidget *linphone_gtk_create_widget(const char *filename, const char *widget_n
 	return w;
 }
 
-static void entry_unmapped(GtkWidget *entry){
-	g_message("Entry is unmapped, calling unrealize to workaround chinese bug.");
-	gtk_widget_unrealize(entry);
+static void entry_unmapped(GtkWidget *widget){
+	ms_message("%s is unmapped, calling unrealize to workaround chinese bug.",G_OBJECT_TYPE_NAME(widget));
+	gtk_widget_unrealize(widget);
 }
 
 GtkWidget *linphone_gtk_get_widget(GtkWidget *window, const char *name){
@@ -442,10 +446,10 @@ GtkWidget *linphone_gtk_get_widget(GtkWidget *window, const char *name){
 		g_error("No widget named %s found in xml interface.",name);
 	}
 	if (workaround_gtk_entry_chinese_bug){
-		if (strcmp(G_OBJECT_TYPE_NAME(w),"GtkEntry")==0){
+		if (strcmp(G_OBJECT_TYPE_NAME(w),"GtkEntry")==0 || strcmp(G_OBJECT_TYPE_NAME(w),"GtkTextView")==0){
 			if (g_object_get_data(G_OBJECT(w),"entry_bug_workaround")==NULL){
 				g_object_set_data(G_OBJECT(w),"entry_bug_workaround",GINT_TO_POINTER(1));
-				g_message("%s is a GtkEntry",name);
+				ms_message("%s is a %s",name,G_OBJECT_TYPE_NAME(w));
 				g_signal_connect(G_OBJECT(w),"unmap",(GCallback)entry_unmapped,NULL);
 			}
 		}
@@ -505,7 +509,7 @@ static void about_url_clicked(GtkAboutDialog *dialog, const char *url, gpointer
 	linphone_gtk_open_browser(url);
 }
 
-void linphone_gtk_show_about(){
+void linphone_gtk_show_about(void){
 	struct stat filestat;
 	const char *license_file=PACKAGE_DATA_DIR "/linphone/COPYING";
 	GtkWidget *about;
@@ -524,13 +528,13 @@ void linphone_gtk_show_about(){
 	if (filestat.st_size>0){
 		char *license=g_malloc(filestat.st_size+1);
 		FILE *f=fopen(license_file,"r");
-		if (f && fread(license,filestat.st_size,1,f)==1){
+		if (f && fread(license,1,filestat.st_size,f)>0){
 			license[filestat.st_size]='\0';
 			gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about),license);
 		}
 		g_free(license);
 	}
-	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about),LINPHONE_VERSION);
+	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about),LIBLINPHONE_GIT_VERSION);
 	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(about),linphone_gtk_get_ui_config("title","Linphone"));
 	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about),linphone_gtk_get_ui_config("home","http://www.linphone.org"));
 	if (logo)	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about),logo);
@@ -550,64 +554,9 @@ void linphone_gtk_show_about(){
 	gtk_widget_show(about);
 }
 
-static void set_video_window_decorations(GdkWindow *w){
-	const char *title=linphone_gtk_get_ui_config("title","Linphone");
-	const char *icon_path=linphone_gtk_get_ui_config("icon",LINPHONE_ICON);
-	char video_title[256];
-	GdkPixbuf *pbuf=create_pixbuf(icon_path);
-
-	if (!linphone_core_in_call(linphone_gtk_get_core())){
-		snprintf(video_title,sizeof(video_title),"%s video",title);
-		/* When not in call, treat the video as a normal window */
-		gdk_window_set_keep_above(w, FALSE);
-	}else{
-		LinphoneAddress *uri =
-			linphone_address_clone(linphone_core_get_current_call_remote_address(linphone_gtk_get_core()));
-		char *display_name;
-
-		linphone_address_clean(uri);
-		if (linphone_address_get_display_name(uri)!=NULL){
-			display_name=ms_strdup(linphone_address_get_display_name(uri));
-		}else{
-			display_name=linphone_address_as_string(uri);
-		}
-		snprintf(video_title,sizeof(video_title),_("Call with %s"),display_name);
-		linphone_address_destroy(uri);
-		ms_free(display_name);
-
-		/* During calls, bring up the video window, arrange so that
-		it is above all the other windows */
-		gdk_window_deiconify(w);
-		gdk_window_set_keep_above(w,TRUE);
-		/* Maybe we should have the following, but then we want to
-		have a timer that turns it off after a little while. */
-		/* gdk_window_set_urgency_hint(w,TRUE); */
-	}
-	gdk_window_set_title(w,video_title);
-	/* Refrain the video window to be closed at all times. */
-	gdk_window_set_functions(w,
-				 GDK_FUNC_RESIZE|GDK_FUNC_MOVE|
-				 GDK_FUNC_MINIMIZE|GDK_FUNC_MAXIMIZE);
-	if (pbuf){
-		GList *l=NULL;
-		l=g_list_append(l,pbuf);
-		gdk_window_set_icon_list(w,l);
-		g_list_free(l);
-		g_object_unref(G_OBJECT(pbuf));
-	}
-}
-
-static gboolean video_needs_update=FALSE;
-
-static void update_video_title(){
-	video_needs_update=TRUE;
-}
 
 static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 	static gboolean first_time=TRUE;
-	unsigned long id;
-	static unsigned long previd=0;
-	static unsigned long preview_previd=0;
 	static gboolean in_iterate=FALSE;
 
 	/*avoid reentrancy*/
@@ -620,44 +569,6 @@ static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 		first_time=FALSE;
 	}
 
-	id=linphone_core_get_native_video_window_id(lc);
-	if (id!=previd || video_needs_update){
-		GdkWindow *w;
-		previd=id;
-		if (id!=0){
-			ms_message("Updating window decorations");
-#ifndef WIN32
-			w=gdk_window_foreign_new((GdkNativeWindow)id);
-#else
-			w=gdk_window_foreign_new((HANDLE)id);
-#endif
-			if (w) {
-				set_video_window_decorations(w);
-				g_object_unref(G_OBJECT(w));
-			}
-			else ms_error("gdk_window_foreign_new() failed");
-			if (video_needs_update) video_needs_update=FALSE;
-		}
-	}
-	id=linphone_core_get_native_preview_window_id (lc);
-	if (id!=preview_previd ){
-		GdkWindow *w;
-		preview_previd=id;
-		if (id!=0){
-			ms_message("Updating window decorations for preview");
-#ifndef WIN32
-			w=gdk_window_foreign_new((GdkNativeWindow)id);
-#else
-			w=gdk_window_foreign_new((HANDLE)id);
-#endif
-			if (w) {
-				set_video_window_decorations(w);
-				g_object_unref(G_OBJECT(w));
-			}
-			else ms_error("gdk_window_foreign_new() failed");
-			if (video_needs_update) video_needs_update=FALSE;
-		}
-	}
 	if (addr_to_call!=NULL){
 		/*make sure we are not showing the login screen*/
 		GtkWidget *mw=linphone_gtk_get_main_window();
@@ -844,7 +755,7 @@ static gboolean launch_contact_provider_search(void *userdata)
 
 	if( ldap && strlen(predicate) >= 3 ){ // don't search too small predicates
 		unsigned int max_res_count = linphone_ldap_contact_provider_get_max_result(ldap);
-
+		LinphoneContactSearch* search;
 		if( previous_search  &&
 			(strstr(predicate, previous_search) == predicate) && // last search contained results from this one
 			(prev_res_count != max_res_count) ){ // and we didn't reach the max result limit
@@ -859,7 +770,7 @@ static gboolean launch_contact_provider_search(void *userdata)
 		gtk_object_set_data(GTK_OBJECT(uribar), "previous_search", ms_strdup(predicate));
 
 		ms_message("launch_contact_provider_search");
-		LinphoneContactSearch* search =linphone_contact_provider_begin_search(
+		search =linphone_contact_provider_begin_search(
 					linphone_contact_provider_cast(ldap_provider),
 					predicate, on_contact_provider_search_results, uribar
 					);
@@ -894,9 +805,6 @@ bool_t linphone_gtk_video_enabled(void){
 
 void linphone_gtk_show_main_window(){
 	GtkWidget *w=linphone_gtk_get_main_window();
-	LinphoneCore *lc=linphone_gtk_get_core();
-	linphone_core_enable_video_preview(lc,linphone_gtk_get_ui_config_int("videoselfview",
-	    	VIDEOSELFVIEW_DEFAULT));
 	gtk_widget_show(w);
 	gtk_window_present(GTK_WINDOW(w));
 }
@@ -908,7 +816,6 @@ void linphone_gtk_call_terminated(LinphoneCall *call, const char *error){
 	}
 	if (linphone_gtk_use_in_call_view() && call)
 		linphone_gtk_in_call_view_terminate(call,error);
-	update_video_title();
 }
 
 static void linphone_gtk_update_call_buttons(LinphoneCall *call){
@@ -920,6 +827,7 @@ static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 	//bool_t stop_active=FALSE;
 	bool_t add_call=FALSE;
 	int call_list_size=ms_list_size(calls);
+	GtkWidget *conf_frame;
 
 	if (calls==NULL){
 		start_active=TRUE;
@@ -942,7 +850,7 @@ static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 	gtk_widget_set_visible(button,add_call);
 
 	//gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),stop_active);
-	GtkWidget *conf_frame=(GtkWidget *)g_object_get_data(G_OBJECT(mw),"conf_frame");
+	conf_frame=(GtkWidget *)g_object_get_data(G_OBJECT(mw),"conf_frame");
 	if(conf_frame==NULL){
 		linphone_gtk_enable_transfer_button(lc,call_list_size>1);
 		linphone_gtk_enable_conference_button(lc,call_list_size>1);
@@ -950,7 +858,6 @@ static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 		linphone_gtk_enable_transfer_button(lc,FALSE);
 		linphone_gtk_enable_conference_button(lc,FALSE);
 	}
-	update_video_title();
 	if (call) {
 		linphone_gtk_update_video_button(call);
 	}
@@ -963,27 +870,37 @@ gchar *linphone_gtk_get_record_path(const LinphoneAddress *address, gboolean is_
 	char date[64]={0};
 	time_t curtime=time(NULL);
 	struct tm loctime;
-	
+	const char **fmts=linphone_core_get_supported_file_formats(linphone_gtk_get_core());
+	int i;
+	const char *ext="wav";
+
 #ifdef WIN32
 	loctime=*localtime(&curtime);
 #else
 	localtime_r(&curtime,&loctime);
 #endif
 	snprintf(date,sizeof(date)-1,"%i%02i%02i-%02i%02i",loctime.tm_year+1900,loctime.tm_mon+1,loctime.tm_mday, loctime.tm_hour, loctime.tm_min);
-	
+
+	for (i=0;fmts[i]!=NULL;++i){
+		if (strcmp(fmts[i],"mkv")==0){
+			ext="mkv";
+			break;
+		}
+	}
+
 	if (address){
 		id=linphone_address_get_username(address);
 		if (id==NULL) id=linphone_address_get_domain(address);
 	}
 	if (is_conference){
-		snprintf(filename,sizeof(filename)-1,"%s-conference-%s.wav",
+		snprintf(filename,sizeof(filename)-1,"%s-conference-%s.%s",
 			linphone_gtk_get_ui_config("title","Linphone"),
-			date);
+			date,ext);
 	}else{
-		snprintf(filename,sizeof(filename)-1,"%s-call-%s-%s.wav",
+		snprintf(filename,sizeof(filename)-1,"%s-call-%s-%s.%s",
 			linphone_gtk_get_ui_config("title","Linphone"),
 			date,
-			id);
+			id,ext);
 	}
 	if (!dir) {
 		ms_message ("No directory for music, using [%s] instead",dir=getenv("HOME"));
@@ -995,7 +912,7 @@ static gboolean linphone_gtk_start_call_do(GtkWidget *uri_bar){
 	const char *entered=gtk_entry_get_text(GTK_ENTRY(uri_bar));
 	LinphoneCore *lc=linphone_gtk_get_core();
 	LinphoneAddress *addr=linphone_core_interpret_url(lc,entered);
-	
+
 	if (addr!=NULL){
 		LinphoneCallParams *params=linphone_core_create_default_call_parameters(lc);
 		gchar *record_file=linphone_gtk_get_record_path(addr,FALSE);
@@ -1081,13 +998,6 @@ void _linphone_gtk_enable_video(gboolean val){
 	linphone_core_enable_video_capture(linphone_gtk_get_core(), TRUE);
 	linphone_core_enable_video_display(linphone_gtk_get_core(), TRUE);
 	linphone_core_set_video_policy(linphone_gtk_get_core(),&policy);
-
-	if (val){
-		linphone_core_enable_video_preview(linphone_gtk_get_core(),
-		linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT));
-	}else{
-		linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
-	}
 }
 
 void linphone_gtk_enable_video(GtkWidget *w){
@@ -1099,7 +1009,6 @@ void linphone_gtk_enable_video(GtkWidget *w){
 void linphone_gtk_enable_self_view(GtkWidget *w){
 	gboolean val=gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w));
 	LinphoneCore *lc=linphone_gtk_get_core();
-	linphone_core_enable_video_preview(lc,val);
 	linphone_core_enable_self_view(lc,val);
 	linphone_gtk_set_ui_config_int("videoselfview",val);
 }
@@ -1142,13 +1051,14 @@ static void linphone_gtk_new_subscriber_response(GtkWidget *dialog, guint respon
 
 static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend *lf, const char *url){
 	GtkWidget *dialog;
+	gchar *message;
 
 	if (linphone_gtk_get_ui_config_int("subscribe_deny_all",0)){
 		linphone_core_reject_subscriber(linphone_gtk_get_core(),lf);
 		return;
 	}
 
-	gchar *message=g_strdup_printf(_("%s would like to add you to his contact list.\nWould you allow him to see your presence status or add him to your contact list ?\nIf you answer no, this person will be temporarily blacklisted."),url);
+	message=g_strdup_printf(_("%s would like to add you to his contact list.\nWould you allow him to see your presence status or add him to your contact list ?\nIf you answer no, this person will be temporarily blacklisted."),url);
 	dialog = gtk_message_dialog_new (
 				GTK_WINDOW(linphone_gtk_get_main_window()),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1293,12 +1203,21 @@ static bool_t notify_actions_supported() {
 	return accepts_actions;
 }
 
-static NotifyNotification* build_notification(const char *title, const char *body){
-	 return notify_notification_new(title,body,linphone_gtk_get_ui_config("icon",LINPHONE_ICON)
+static NotifyNotification* build_notification(const char *title, const char *body) {
+	NotifyNotification *n = notify_notification_new(title, body, NULL
 #ifdef HAVE_NOTIFY1
-        ,NULL
+		,NULL
 #endif
 	);
+#ifndef HAVE_NOTIFY1
+	{
+		const char *icon_path = linphone_gtk_get_ui_config("icon", LINPHONE_ICON);
+		GdkPixbuf *pbuf = create_pixbuf(icon_path);
+		/*with notify1, this function makes the notification crash the app with obscure dbus glib critical errors*/
+		notify_notification_set_icon_from_pixbuf(n, pbuf);
+	}
+#endif
+	return n;
 }
 
 static void show_notification(NotifyNotification* n){
@@ -1376,6 +1295,9 @@ static void linphone_gtk_global_state_changed(LinphoneCore *lc, LinphoneGlobalSt
 		break;
 		case LinphoneGlobalOn:
 			linphone_gtk_init_ui();
+			if (selftest) {
+				gtk_timeout_add(300,(GtkFunction)gtk_main_quit,NULL);
+			}
 		break;
 		default:
 		break;
@@ -1506,7 +1428,7 @@ static void update_registration_status(LinphoneProxyConfig *cfg, LinphoneRegistr
 		}while(gtk_tree_model_iter_next(model,&iter));
 	}
 	if (!found) {
-		g_warning("Could not find proxy config in combo box of identities.");
+		/*ignored, this is a notification for a removed proxy config.*/
 		return;
 	}
 	switch (rs){
@@ -1608,13 +1530,13 @@ static GtkWidget *create_icon_menu(){
 }
 
 void linphone_gtk_save_main_window_position(GtkWindow* mw, GdkEvent *event, gpointer data){
-       gtk_window_get_position(GTK_WINDOW(mw), &main_window_x, &main_window_y); 
+       gtk_window_get_position(GTK_WINDOW(mw), &main_window_x, &main_window_y);
 }
 
 static void handle_icon_click() {
 	GtkWidget *mw=linphone_gtk_get_main_window();
 	if (!gtk_window_is_active((GtkWindow*)mw)) {
-		if(!gtk_widget_is_drawable(mw)){ 
+		if(!gtk_widget_is_drawable(mw)){
 			//we only move if window was hidden. If it was simply behind the window stack, ie, drawable, we keep it as it was
 			gtk_window_move (GTK_WINDOW(mw), main_window_x, main_window_y);
 		}
@@ -1883,10 +1805,11 @@ void linphone_gtk_manage_login(void){
 gboolean linphone_gtk_close(GtkWidget *mw){
 	/*shutdown calls if any*/
 	LinphoneCore *lc=linphone_gtk_get_core();
+	GtkWidget *camera_preview=linphone_gtk_get_camera_preview_window();
 	if (linphone_core_in_call(lc)){
 		linphone_core_terminate_all_calls(lc);
 	}
-	linphone_core_enable_video_preview(lc,FALSE);
+	if (camera_preview) gtk_widget_destroy(camera_preview);
 #ifdef __APPLE__ /*until with have a better option*/
 	gtk_window_iconify(GTK_WINDOW(mw));
 #else
@@ -1897,13 +1820,6 @@ gboolean linphone_gtk_close(GtkWidget *mw){
 
 #ifdef HAVE_GTK_OSX
 static gboolean on_window_state_event(GtkWidget *w, GdkEventWindowState *event){
-	bool_t video_enabled=linphone_gtk_video_enabled();
-	if ((event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) ||(event->new_window_state & GDK_WINDOW_STATE_WITHDRAWN) ){
-		linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
-	}else{
-		linphone_core_enable_video_preview(linphone_gtk_get_core(),
-		linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT) && video_enabled);
-	}
 	return FALSE;
 }
 #endif
@@ -1967,10 +1883,11 @@ void linphone_gtk_keypad_key_released(GtkWidget *w, GdkEvent *event, gpointer us
 void linphone_gtk_create_keypad(GtkWidget *button){
 	GtkWidget *mw=linphone_gtk_get_main_window();
 	GtkWidget *k=(GtkWidget *)g_object_get_data(G_OBJECT(mw),"keypad");
+	GtkWidget *keypad;
 	if(k!=NULL){
 		gtk_widget_destroy(k);
 	}
-	GtkWidget *keypad=linphone_gtk_create_window("keypad");
+	keypad=linphone_gtk_create_window("keypad");
 	linphone_gtk_connect_digits(keypad);
 	linphone_gtk_init_dtmf_table(keypad);
 	g_object_set_data(G_OBJECT(mw),"keypad",(gpointer)keypad);
@@ -2013,6 +1930,7 @@ static void linphone_gtk_init_main_window(){
 	g_signal_connect(G_OBJECT(main_window), "window-state-event",G_CALLBACK(on_window_state_event), NULL);
 #endif
 	linphone_gtk_check_menu_items();
+	linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
 }
 
 void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
@@ -2129,6 +2047,11 @@ static void linphone_gtk_init_ui(void){
 	}
 #endif
 
+	if(run_audio_assistant){
+		linphone_gtk_show_audio_assistant();
+		start_option=START_AUDIO_ASSISTANT;
+		iconified = TRUE;
+	}
 #ifndef HAVE_GTK_OSX
 	linphone_gtk_init_status_icon();
 #endif
@@ -2151,6 +2074,8 @@ int main(int argc, char *argv[]){
 	const char *app_name="Linphone";
 	LpConfig *factory;
 	const char *db_file;
+	GError *error=NULL;
+	const char *tmp;
 
 #if !GLIB_CHECK_VERSION(2, 31, 0)
 	g_thread_init(NULL);
@@ -2161,6 +2086,7 @@ int main(int argc, char *argv[]){
 
 	config_file=linphone_gtk_get_config_file(NULL);
 
+	workingdir= (tmp=g_getenv("LINPHONE_WORKDIR")) ? g_strdup(tmp) : NULL;
 
 #ifdef WIN32
 	/*workaround for windows: sometimes LANG is defined to an integer value, not understood by gtk */
@@ -2186,6 +2112,7 @@ int main(int argc, char *argv[]){
 		}
 #elif __APPLE__
 		setenv("LANG",lang,1);
+		setenv("LANGUAGE",lang,1);
 #else
 		setenv("LANGUAGE",lang,1);
 #endif
@@ -2206,11 +2133,12 @@ int main(int argc, char *argv[]){
 	gdk_threads_enter();
 
 	if (!gtk_init_with_args(&argc,&argv,_("A free SIP video-phone"),
-				linphone_options,NULL,NULL)){
+				linphone_options,NULL,&error)){
 		gdk_threads_leave();
+		g_critical("%s", error->message);
 		return -1;
 	}
-	if (config_file) free(config_file);
+	if (config_file) g_free(config_file);
 	if (custom_config_file && !g_path_is_absolute(custom_config_file)) {
 		gchar *res = g_get_current_dir();
 		res = g_strjoin(G_DIR_SEPARATOR_S, res, custom_config_file, NULL);
@@ -2219,6 +2147,9 @@ int main(int argc, char *argv[]){
 	}
 	config_file=linphone_gtk_get_config_file(custom_config_file);
 
+	if(run_audio_assistant) start_option=START_AUDIO_ASSISTANT;
+	if(addr_to_call != NULL) start_option=START_LINPHONE_WITH_CALL;
+
 	settings=gtk_settings_get_default();
 	g_type_class_unref (g_type_class_ref (GTK_TYPE_IMAGE_MENU_ITEM));
 	g_type_class_unref (g_type_class_ref (GTK_TYPE_BUTTON));
@@ -2230,7 +2161,7 @@ int main(int argc, char *argv[]){
 			g_error("Could not change directory to %s : %s",workingdir,strerror(errno));
 		}
 	}
-	
+
 #if defined(__APPLE__) && defined(ENABLE_NLS)
 	/*workaround for bundles. GTK is unable to find translations in the bundle (obscure bug again).
 	So we help it:*/
@@ -2242,6 +2173,9 @@ int main(int argc, char *argv[]){
 		}
 	}
 #endif
+	add_pixmap_directory("pixmaps");
+	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps/linphone");
+
 	/* Now, look for the factory configuration file, we do it this late
 		 since we want to have had time to change directory and to parse
 		 the options, in case we needed to access the working directory */
@@ -2256,9 +2190,6 @@ int main(int argc, char *argv[]){
 	pbuf=create_pixbuf(icon_path);
 	if (pbuf!=NULL) gtk_window_set_default_icon(pbuf);
 
-	add_pixmap_directory("pixmaps");
-	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps/linphone");
-
 #ifdef HAVE_GTK_OSX
 	GtkosxApplication *theMacApp = gtkosx_application_get();
 	g_signal_connect(G_OBJECT(theMacApp),"NSApplicationDidBecomeActive",(GCallback)linphone_gtk_show_main_window,NULL);
@@ -2268,7 +2199,7 @@ int main(int argc, char *argv[]){
 #endif
 
 core_start:
-	if (linphone_gtk_init_instance(app_name, addr_to_call) == FALSE){
+	if (linphone_gtk_init_instance(app_name, start_option, addr_to_call) == FALSE){
 		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
 		g_warning("This instance is going to exit now.");
 		gdk_threads_leave();
@@ -2281,23 +2212,24 @@ core_start:
 
 	linphone_gtk_create_log_window();
 	linphone_core_enable_logs_with_cb(linphone_gtk_log_handler);
-	
+
 	db_file=linphone_gtk_message_storage_get_db_file(NULL);
 
 	linphone_gtk_init_liblinphone(config_file, factory_config_file, db_file);
-	
+
 	/* do not lower timeouts under 30 ms because it exhibits a bug on gtk+/win32, with cpu running 20% all the time...*/
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)linphone_gtk_get_core());
-	
+
 	gtk_main();
 	linphone_gtk_quit();
-	
+
 	if (restart){
 		quit_done=FALSE;
 		restart=FALSE;
 		goto core_start;
 	}
+	if (config_file) g_free(config_file);
 #ifndef HAVE_GTK_OSX
 	/*workaround a bug on win32 that makes status icon still present in the systray even after program exit.*/
 	if (icon) gtk_status_icon_set_visible(icon,FALSE);
@@ -2306,3 +2238,8 @@ core_start:
 	return 0;
 }
 
+#ifdef _MSC_VER
+int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
+	return main(__argc, __argv);
+}
+#endif
