@@ -17,9 +17,9 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-/** 
+/**
  This header files defines the Signaling Abstraction Layer.
- The purpose of this layer is too allow experiment different call signaling 
+ The purpose of this layer is too allow experiment different call signaling
  protocols and implementations under linphone, for example SIP, JINGLE...
 **/
 
@@ -30,8 +30,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "config.h"
 #endif
 
-#include "mediastreamer2/mscommon.h"
-#include "ortp/ortp_srtp.h"
+#include "mediastreamer2/mediastream.h"
+#include "ortp/rtpsession.h"
 #include "belle-sip/belle-sip.h"
 
 #ifndef LINPHONE_PUBLIC
@@ -65,14 +65,18 @@ typedef enum {
 	SalTransportUDP, /*UDP*/
 	SalTransportTCP, /*TCP*/
 	SalTransportTLS, /*TLS*/
-	SalTransportDTLS /*DTLS*/
+	SalTransportDTLS, /*DTLS*/
 }SalTransport;
 
-#define SAL_MEDIA_DESCRIPTION_UNCHANGED		0x00
-#define SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED	0x01
-#define SAL_MEDIA_DESCRIPTION_CODEC_CHANGED	0x02
-#define SAL_MEDIA_DESCRIPTION_CRYPTO_CHANGED	0x04
-#define SAL_MEDIA_DESCRIPTION_CHANGED		(SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED | SAL_MEDIA_DESCRIPTION_CODEC_CHANGED | SAL_MEDIA_DESCRIPTION_CRYPTO_CHANGED)
+#define SAL_MEDIA_DESCRIPTION_UNCHANGED			0x00
+#define SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED		(1)
+#define SAL_MEDIA_DESCRIPTION_CODEC_CHANGED		(1<<1)
+#define SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED	(1<<2)
+#define SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED	(1<<3)
+#define SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED		(1<<4)
+
+#define SAL_MEDIA_DESCRIPTION_CHANGED		(SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED | SAL_MEDIA_DESCRIPTION_CODEC_CHANGED |\
+						SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED |SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED | SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED)
 
 const char* sal_transport_to_string(SalTransport transport);
 SalTransport sal_transport_parse(const char*);
@@ -88,6 +92,7 @@ const char *sal_address_get_username(const SalAddress *addr);
 const char *sal_address_get_domain(const SalAddress *addr);
 int sal_address_get_port(const SalAddress *addr);
 bool_t sal_address_is_secure(const SalAddress *addr);
+void sal_address_set_secure(SalAddress *addr, bool_t enabled);
 
 SalTransport sal_address_get_transport(const SalAddress* addr);
 const char* sal_address_get_transport_name(const SalAddress* addr);
@@ -105,7 +110,11 @@ void sal_address_set_transport(SalAddress* addr,SalTransport transport);
 void sal_address_set_transport_name(SalAddress* addr,const char* transport);
 void sal_address_set_params(SalAddress *addr, const char *params);
 void sal_address_set_uri_params(SalAddress *addr, const char *params);
-bool_t sal_address_is_ipv6(SalAddress *addr);
+bool_t sal_address_is_ipv6(const SalAddress *addr);
+bool_t sal_address_is_sip(const SalAddress *addr);
+void sal_address_set_password(SalAddress *addr, const char *passwd);
+const char *sal_address_get_password(const SalAddress *addr);
+void sal_address_set_header(SalAddress *addr, const char *header_name, const char *header_value);
 
 Sal * sal_init();
 void sal_uninit(Sal* sal);
@@ -121,9 +130,13 @@ typedef enum {
 const char* sal_stream_type_to_string(SalStreamType type);
 
 typedef enum{
-	SalProtoUnknown,
 	SalProtoRtpAvp,
-	SalProtoRtpSavp
+	SalProtoRtpSavp,
+	SalProtoRtpAvpf,
+	SalProtoRtpSavpf,
+	SalProtoUdpTlsRtpSavp,
+	SalProtoUdpTlsRtpSavpf,
+	SalProtoOther
 }SalMediaProto;
 const char* sal_media_proto_to_string(SalMediaProto type);
 
@@ -153,7 +166,7 @@ typedef struct SalIceCandidate {
 	int rport;
 } SalIceCandidate;
 
-#define SAL_MEDIA_DESCRIPTION_MAX_ICE_CANDIDATES 10
+#define SAL_MEDIA_DESCRIPTION_MAX_ICE_CANDIDATES 20
 
 typedef struct SalIceRemoteCandidate {
 	char addr[SAL_MEDIA_DESCRIPTION_MAX_ICE_ADDR_LEN];
@@ -165,57 +178,93 @@ typedef struct SalIceRemoteCandidate {
 #define SAL_MEDIA_DESCRIPTION_MAX_ICE_UFRAG_LEN 256
 #define SAL_MEDIA_DESCRIPTION_MAX_ICE_PWD_LEN 256
 
-#define SAL_SRTP_KEY_SIZE 41
+/*sufficient for 256bit keys encoded in base 64*/
+#define SAL_SRTP_KEY_SIZE 128
 
 typedef struct SalSrtpCryptoAlgo {
 	unsigned int tag;
-	enum ortp_srtp_crypto_suite_t algo;
-	/* 41= 40 max(key_length for all algo) + '\0' */
+	MSCryptoSuite algo;
 	char master_key[SAL_SRTP_KEY_SIZE];
 } SalSrtpCryptoAlgo;
 
 #define SAL_CRYPTO_ALGO_MAX 4
 
+typedef enum {
+	SalDtlsRoleInvalid,
+	SalDtlsRoleIsServer,
+	SalDtlsRoleIsClient,
+	SalDtlsRoleUnset
+} SalDtlsRole;
+
+typedef enum {
+	SalMulticastInactive=0,
+	SalMulticastSender,
+	SalMulticastReceiver,
+	SalMulticastSenderReceiver
+} SalMulticastRole;
+
+typedef enum {
+	SalOpSDPNormal = 0, /** No special handling for SDP */
+	SalOpSDPSimulateError, /** Will simulate an SDP parsing error */
+	SalOpSDPSimulateRemove /** Will simulate no SDP in the op */
+} SalOpSDPHandling;
+
 typedef struct SalStreamDescription{
+	char name[16]; /*unique name of stream, in order to ease offer/answer model algorithm*/
 	SalMediaProto proto;
 	SalStreamType type;
 	char typeother[32];
+	char proto_other[32];
 	char rtp_addr[64];
 	char rtcp_addr[64];
+	unsigned int rtp_ssrc;
+	char rtcp_cname[255];
 	int rtp_port;
 	int rtcp_port;
-	MSList *payloads; //<list of PayloadType
+	MSList *payloads; /*<list of PayloadType */
+	MSList *already_assigned_payloads; /*<list of PayloadType offered in the past, used for correct allocation of payload type numbers*/
 	int bandwidth;
 	int ptime;
 	SalStreamDir dir;
 	SalSrtpCryptoAlgo crypto[SAL_CRYPTO_ALGO_MAX];
 	unsigned int crypto_local_tag;
 	int max_rate;
+	OrtpRtcpXrConfiguration rtcp_xr;
 	SalIceCandidate ice_candidates[SAL_MEDIA_DESCRIPTION_MAX_ICE_CANDIDATES];
 	SalIceRemoteCandidate ice_remote_candidates[SAL_MEDIA_DESCRIPTION_MAX_ICE_REMOTE_CANDIDATES];
 	char ice_ufrag[SAL_MEDIA_DESCRIPTION_MAX_ICE_UFRAG_LEN];
 	char ice_pwd[SAL_MEDIA_DESCRIPTION_MAX_ICE_PWD_LEN];
 	bool_t ice_mismatch;
 	bool_t ice_completed;
+	bool_t pad[2];
+	char dtls_fingerprint[256];
+	SalDtlsRole dtls_role;
+	int ttl; /*for multicast -1 to disable*/
+	SalMulticastRole multicast_role;
 } SalStreamDescription;
 
-#define SAL_MEDIA_DESCRIPTION_MAX_STREAMS 4
+const char *sal_stream_description_get_type_as_string(const SalStreamDescription *desc);
+const char *sal_stream_description_get_proto_as_string(const SalStreamDescription *desc);
+
+#define SAL_MEDIA_DESCRIPTION_MAX_STREAMS 8
 
 typedef struct SalMediaDescription{
 	int refcount;
 	char name[64];
 	char addr[64];
 	char username[64];
-	int n_active_streams;
-	int n_total_streams;
+	int nb_streams;
 	int bandwidth;
 	unsigned int session_ver;
 	unsigned int session_id;
+	SalStreamDir dir;
 	SalStreamDescription streams[SAL_MEDIA_DESCRIPTION_MAX_STREAMS];
+	OrtpRtcpXrConfiguration rtcp_xr;
 	char ice_ufrag[SAL_MEDIA_DESCRIPTION_MAX_ICE_UFRAG_LEN];
 	char ice_pwd[SAL_MEDIA_DESCRIPTION_MAX_ICE_PWD_LEN];
 	bool_t ice_lite;
 	bool_t ice_completed;
+	bool_t pad[2];
 } SalMediaDescription;
 
 typedef struct SalMessage{
@@ -223,6 +272,7 @@ typedef struct SalMessage{
 	const char *text;
 	const char *url;
 	const char *message_id;
+	const char *content_type;
 	time_t time;
 }SalMessage;
 
@@ -241,7 +291,20 @@ int sal_media_description_equals(const SalMediaDescription *md1, const SalMediaD
 bool_t sal_media_description_has_dir(const SalMediaDescription *md, SalStreamDir dir);
 SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md,
     SalMediaProto proto, SalStreamType type);
+unsigned int sal_media_description_nb_active_streams_of_type(SalMediaDescription *md, SalStreamType type);
+SalStreamDescription * sal_media_description_get_active_stream_of_type(SalMediaDescription *md, SalStreamType type, unsigned int idx);
+SalStreamDescription * sal_media_description_find_secure_stream_of_type(SalMediaDescription *md, SalStreamType type);
+SalStreamDescription * sal_media_description_find_best_stream(SalMediaDescription *md, SalStreamType type);
 void sal_media_description_set_dir(SalMediaDescription *md, SalStreamDir stream_dir);
+bool_t sal_stream_description_active(const SalStreamDescription *sd);
+bool_t sal_stream_description_has_avpf(const SalStreamDescription *sd);
+bool_t sal_stream_description_has_srtp(const SalStreamDescription *sd);
+bool_t sal_stream_description_has_dtls(const SalStreamDescription *sd);
+bool_t sal_media_description_has_avpf(const SalMediaDescription *md);
+bool_t sal_media_description_has_srtp(const SalMediaDescription *md);
+bool_t sal_media_description_has_dtls(const SalMediaDescription *md);
+int sal_media_description_get_nb_active_streams(const SalMediaDescription *md);
+
 
 /*this structure must be at the first byte of the SalOp structure defined by implementors*/
 typedef struct SalOpBase{
@@ -262,25 +325,20 @@ typedef struct SalOpBase{
 	SalMediaDescription *remote_media;
 	void *user_pointer;
 	const char* call_id;
+	char* realm;
 	SalAddress* service_route; /*as defined by rfc3608, might be a list*/
 	SalCustomHeader *sent_custom_headers;
 	SalCustomHeader *recv_custom_headers;
 } SalOpBase;
 
 
-typedef enum SalError{
-	SalErrorNone,
-	SalErrorNoResponse,
-	SalErrorProtocol,
-	SalErrorFailure, /* see SalReason for more details */
-	SalErrorUnknown
-} SalError;
-
 typedef enum SalReason{
+	SalReasonNone, /*no error, please leave first so that it takes 0 value*/
 	SalReasonDeclined,
 	SalReasonBusy,
 	SalReasonRedirect,
 	SalReasonTemporarilyUnavailable,
+	SalReasonRequestTimeout,
 	SalReasonNotFound,
 	SalReasonDoNotDisturb,
 	SalReasonUnsupportedContent,
@@ -290,11 +348,26 @@ typedef enum SalReason{
 	SalReasonRequestPending,
 	SalReasonUnauthorized,
 	SalReasonNotAcceptable,
-	SalReasonNoMatch /*equivalent to 481 Transaction/Call leg does not exist*/
+	SalReasonNoMatch, /*equivalent to 481 Transaction/Call leg does not exist*/
+	SalReasonMovedPermanently,
+	SalReasonGone,
+	SalReasonAddressIncomplete,
+	SalReasonNotImplemented,
+	SalReasonBadGateway,
+	SalReasonServerTimeout,
+	SalReasonIOError
 }SalReason;
 
 const char* sal_reason_to_string(const SalReason reason);
 
+typedef struct SalErrorInfo{
+	SalReason reason;
+	char *status_string;
+	int protocol_code;
+	char *warnings;
+	char *full_string; /*concatenation of status_string + warnings*/
+}SalErrorInfo;
+
 typedef enum SalPresenceStatus{
 	SalPresenceOffline,
 	SalPresenceOnline,
@@ -380,23 +453,23 @@ typedef void (*SalOnCallReceived)(SalOp *op);
 typedef void (*SalOnCallRinging)(SalOp *op);
 typedef void (*SalOnCallAccepted)(SalOp *op);
 typedef void (*SalOnCallAck)(SalOp *op);
-typedef void (*SalOnCallUpdating)(SalOp *op);/*< Called when a reINVITE is received*/
+typedef void (*SalOnCallUpdating)(SalOp *op, bool_t is_update);/*< Called when a reINVITE/UPDATE is received*/
 typedef void (*SalOnCallTerminated)(SalOp *op, const char *from);
-typedef void (*SalOnCallFailure)(SalOp *op, SalError error, SalReason reason, const char *details, int code);
+typedef void (*SalOnCallFailure)(SalOp *op);
 typedef void (*SalOnCallReleased)(SalOp *salop);
 typedef void (*SalOnAuthRequestedLegacy)(SalOp *op, const char *realm, const char *username);
 typedef bool_t (*SalOnAuthRequested)(Sal *sal,SalAuthInfo* info);
 typedef void (*SalOnAuthFailure)(SalOp *op, SalAuthInfo* info);
 typedef void (*SalOnRegisterSuccess)(SalOp *op, bool_t registered);
-typedef void (*SalOnRegisterFailure)(SalOp *op, SalError error, SalReason reason, const char *details);
+typedef void (*SalOnRegisterFailure)(SalOp *op);
 typedef void (*SalOnVfuRequest)(SalOp *op);
 typedef void (*SalOnDtmfReceived)(SalOp *op, char dtmf);
 typedef void (*SalOnRefer)(Sal *sal, SalOp *op, const char *referto);
 typedef void (*SalOnTextReceived)(SalOp *op, const SalMessage *msg);
-typedef void (*SalOnTextDeliveryUpdate)(SalOp *op, SalTextDeliveryStatus, SalReason);
+typedef void (*SalOnTextDeliveryUpdate)(SalOp *op, SalTextDeliveryStatus);
 typedef void (*SalOnIsComposingReceived)(SalOp *op, const SalIsComposing *is_composing);
 typedef void (*SalOnNotifyRefer)(SalOp *op, SalReferStatus state);
-typedef void (*SalOnSubscribeResponse)(SalOp *op, SalSubscribeStatus status, SalError error, SalReason reason);
+typedef void (*SalOnSubscribeResponse)(SalOp *op, SalSubscribeStatus status);
 typedef void (*SalOnNotify)(SalOp *op, SalSubscribeStatus status, const char *event, const SalBody *body);
 typedef void (*SalOnSubscribeReceived)(SalOp *salop, const char *event, const SalBody *body);
 typedef void (*SalOnSubscribeClosed)(SalOp *salop);
@@ -407,7 +480,7 @@ typedef void (*SalOnSubscribePresenceReceived)(SalOp *salop, const char *from);
 typedef void (*SalOnSubscribePresenceClosed)(SalOp *salop, const char *from);
 typedef void (*SalOnPingReply)(SalOp *salop);
 typedef void (*SalOnInfoReceived)(SalOp *salop, const SalBody *body);
-typedef void (*SalOnPublishResponse)(SalOp *salop, SalError error, SalReason reason);
+typedef void (*SalOnPublishResponse)(SalOp *salop);
 typedef void (*SalOnExpire)(SalOp *salop);
 /*allows sal implementation to access auth info if available, return TRUE if found*/
 
@@ -473,25 +546,45 @@ void sal_certificates_chain_parse_file(SalAuthInfo* auth_info, const char* path,
  */
 void sal_signing_key_parse_file(SalAuthInfo* auth_info, const char* path, const char *passwd);
 
+/**
+ * Parse a directory for files containing certificate with the given subject CNAME
+ * @param[out]	certificate_pem				the address of a string to store the certificate in PEM format. To be freed by caller
+ * @param[out]	key_pem						the address of a string to store the key in PEM format. To be freed by caller
+ * @param[in]	path						directory to parse
+ * @param[in]	subject						subject CNAME
+ * @param[in]	format 						either PEM or DER
+ * @param[in]	generate_certificate		if true, if matching certificate and key can't be found, generate it and store it into the given dir, filename will be subject.pem
+ * @param[in]	generate_dtls_fingerprint	if true and we have a certificate, generate the dtls fingerprint as described in rfc4572
+ */
+void sal_certificates_chain_parse_directory(char **certificate_pem, char **key_pem, char **fingerprint, const char* path, const char *subject, SalCertificateRawFormat format, bool_t generate_certificate, bool_t generate_dtls_fingerprint);
+
 void sal_certificates_chain_delete(SalCertificatesChain *chain);
 void sal_signing_key_delete(SalSigningKey *key);
 
 
 
 void sal_set_callbacks(Sal *ctx, const SalCallbacks *cbs);
-int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int is_secure);
+int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int is_tunneled);
+int sal_get_listening_port(Sal *ctx, SalTransport tr);
 int sal_unlisten_ports(Sal *ctx);
 int sal_transport_available(Sal *ctx, SalTransport t);
 void sal_set_dscp(Sal *ctx, int dscp);
+void sal_set_supported_tags(Sal *ctx, const char* tags);
+void sal_add_supported_tag(Sal *ctx, const char* tag);
+void sal_remove_supported_tag(Sal *ctx, const char* tag);
+const char *sal_get_supported_tags(Sal *ctx);
 int sal_reset_transports(Sal *ctx);
 ortp_socket_t sal_get_socket(Sal *ctx);
 void sal_set_user_agent(Sal *ctx, const char *user_agent);
+const char* sal_get_user_agent(Sal *ctx);
 void sal_append_stack_string_to_user_agent(Sal *ctx);
 /*keepalive period in ms*/
 void sal_set_keepalive_period(Sal *ctx,unsigned int value);
 void sal_use_tcp_tls_keepalive(Sal *ctx, bool_t enabled);
-int sal_enable_tunnel(Sal *ctx, void *tunnelclient);
-void sal_disable_tunnel(Sal *ctx);
+int sal_set_tunnel(Sal *ctx, void *tunnelclient);
+/*Default value is true*/
+void sal_enable_sip_update_method(Sal *ctx,bool_t value);
+
 /**
  * returns keepalive period in ms
  * 0 desactiaved
@@ -508,7 +601,8 @@ void sal_verify_server_certificates(Sal *ctx, bool_t verify);
 void sal_verify_server_cn(Sal *ctx, bool_t verify);
 void sal_set_uuid(Sal*ctx, const char *uuid);
 int sal_create_uuid(Sal*ctx, char *uuid, size_t len);
-void sal_enable_test_features(Sal*ctx, bool_t enabled);
+LINPHONE_PUBLIC void sal_enable_test_features(Sal*ctx, bool_t enabled);
+void sal_use_no_initial_route(Sal *ctx, bool_t enabled);
 
 int sal_iterate(Sal *sal);
 MSList * sal_get_pending_auths(Sal *sal);
@@ -522,12 +616,17 @@ void sal_op_set_contact_address(SalOp *op, const SalAddress* address);
 void sal_op_set_route(SalOp *op, const char *route);
 void sal_op_set_route_address(SalOp *op, const SalAddress* address);
 void sal_op_add_route_address(SalOp *op, const SalAddress* address);
+void sal_op_set_realm(SalOp *op, const char *realm);
 void sal_op_set_from(SalOp *op, const char *from);
 void sal_op_set_from_address(SalOp *op, const SalAddress *from);
 void sal_op_set_to(SalOp *op, const char *to);
 void sal_op_set_to_address(SalOp *op, const SalAddress *to);
 SalOp *sal_op_ref(SalOp* h);
+void sal_op_stop_refreshing(SalOp *op);
 void sal_op_release(SalOp *h);
+/*same as release, but does not stop refresher if any*/
+void* sal_op_unref(SalOp* op);
+
 void sal_op_authenticate(SalOp *h, const SalAuthInfo *info);
 void sal_op_cancel_authentication(SalOp *h);
 void sal_op_set_user_pointer(SalOp *h, void *up);
@@ -549,6 +648,7 @@ const SalAddress *sal_op_get_network_origin_address(const SalOp *op);
 const char *sal_op_get_remote_ua(const SalOp *op);
 void *sal_op_get_user_pointer(const SalOp *op);
 const char* sal_op_get_call_id(const SalOp *op);
+char* sal_op_get_dialog_id(const SalOp *op);
 
 const SalAddress* sal_op_get_service_route(const SalOp *op);
 void sal_op_set_service_route(SalOp *op,const SalAddress* service_route);
@@ -558,6 +658,11 @@ bool_t sal_op_is_ipv6(SalOp *op);
 /*returns TRUE if there is no pending request that may block a future one */
 bool_t sal_op_is_idle(SalOp *op);
 
+const SalErrorInfo *sal_error_info_none(void);
+const SalErrorInfo *sal_op_get_error_info(const SalOp *op);
+void sal_error_info_reset(SalErrorInfo *ei);
+void sal_error_info_set(SalErrorInfo *ei, SalReason reason, int code, const char *status_string, const char *warning);
+
 /*Call API*/
 int sal_call_set_local_media_description(SalOp *h, SalMediaDescription *desc);
 int sal_call(SalOp *h, const char *from, const char *to);
@@ -565,7 +670,7 @@ int sal_call_notify_ringing(SalOp *h, bool_t early_media);
 /*accept an incoming call or, during a call accept a reINVITE*/
 int sal_call_accept(SalOp*h);
 int sal_call_decline(SalOp *h, SalReason reason, const char *redirection /*optional*/);
-int sal_call_update(SalOp *h, const char *subject);
+int sal_call_update(SalOp *h, const char *subject, bool_t no_user_consent);
 SalMediaDescription * sal_call_get_remote_media_description(SalOp *h);
 SalMediaDescription * sal_call_get_final_media_description(SalOp *h);
 int sal_call_refer(SalOp *h, const char *refer_to);
@@ -581,6 +686,21 @@ bool_t sal_call_autoanswer_asked(SalOp *op);
 void sal_call_send_vfu_request(SalOp *h);
 int sal_call_is_offerer(const SalOp *h);
 int sal_call_notify_refer_state(SalOp *h, SalOp *newcall);
+/* Call test API */
+
+
+/**
+ * @brief Invoking this on the SAL will modify every subsequent SalOp to have a special handling for SDP.
+ * @details This is especially useful while testing, to simulate some specific behaviors, like missing SDP or an error in parsing.
+ *
+ * @warning Don't forget to reset the handling method to SalOpSDPNormal afterwards.
+ *
+ * @param h the Sal instance
+ * @param handling_method Could be SalOpSDPNormal, SalOpSDPSimulateError, SalOpSDPSimulateRemoval (\ref SalOpSDPHandling)
+ */
+LINPHONE_PUBLIC void sal_default_set_sdp_handling(Sal* h, SalOpSDPHandling handling_method) ;
+/* Second version: for a specific call*/
+LINPHONE_PUBLIC void sal_call_set_sdp_handling(SalOp *h, SalOpSDPHandling handling) ;
 
 /*Registration*/
 int sal_register(SalOp *op, const char *proxy, const char *from, int expires);
@@ -591,6 +711,7 @@ int sal_unregister(SalOp *h);
 /*Messaging */
 int sal_text_send(SalOp *op, const char *from, const char *to, const char *text);
 int sal_message_send(SalOp *op, const char *from, const char *to, const char* content_type, const char *msg);
+int sal_message_reply(SalOp *op, SalReason reason);
 
 /*presence Subscribe/notify*/
 int sal_subscribe_presence(SalOp *op, const char *from, const char *to, int expires);
@@ -690,6 +811,8 @@ LINPHONE_PUBLIC void sal_enable_dns_srv(Sal *sal, bool_t enable);
 LINPHONE_PUBLIC bool_t sal_dns_srv_enabled(const Sal *sal);
 LINPHONE_PUBLIC void sal_set_dns_user_hosts_file(Sal *sal, const char *hosts_file);
 LINPHONE_PUBLIC const char *sal_get_dns_user_hosts_file(const Sal *sal);
+unsigned int sal_get_random(void);
+LINPHONE_PUBLIC char *sal_get_random_token(int size);
 unsigned char * sal_get_random_bytes(unsigned char *ret, size_t size);
 belle_sip_source_t * sal_create_timer(Sal *sal, belle_sip_source_func_t func, void *data, unsigned int timeout_value_ms, const char* timer_name);
 void sal_cancel_timer(Sal *sal, belle_sip_source_t *timer);
@@ -698,6 +821,12 @@ int sal_body_has_type(const SalBody *body, const char *type, const char *subtype
 /*this function parses a document with key=value pairs separated by new lines, and extracts the value for a given key*/
 int sal_lines_get_value(const char *data, const char *key, char *value, size_t value_size);
 
-belle_sip_stack_t *sal_get_belle_sip_stack(Sal *sal);
+LINPHONE_PUBLIC belle_sip_stack_t *sal_get_belle_sip_stack(Sal *sal);
+char* sal_op_get_public_uri(SalOp *sal);
+
+unsigned long sal_begin_background_task(const char *name, void (*max_time_reached)(void *), void *data);
+void sal_end_background_task(unsigned long id);
 
 #endif
+
+
