@@ -40,6 +40,7 @@ static int android_sdk_version = 5;
 static const char* AndroidApi9WrapperPath = "org/linphone/mediastream/video/capture/AndroidVideoApi9JniWrapper";
 static const char* AndroidApi8WrapperPath = "org/linphone/mediastream/video/capture/AndroidVideoApi8JniWrapper";
 static const char* AndroidApi5WrapperPath = "org/linphone/mediastream/video/capture/AndroidVideoApi5JniWrapper";
+static const char* VersionPath 			  = "org/linphone/mediastream/Version";
 
 #define UNDEFINED_ROTATION -1
 
@@ -75,7 +76,7 @@ struct AndroidReaderContext {
 
 	mblk_t *frame;
 	float fps;
-	MSVideoSize requestedSize, hwCapableSize;
+	MSVideoSize requestedSize, hwCapableSize, usedSize;
 	ms_mutex_t mutex;
 	int rotation, rotationSavedDuringVSize;
 	int useDownscaling;
@@ -111,7 +112,7 @@ static int video_capture_set_autofocus(MSFilter *f, void* data){
 
 static int video_capture_get_fps(MSFilter *f, void *arg){
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
-	*((float*)arg) = d->fps;
+	*((float*)arg) = ms_average_fps_get(&d->averageFps);
 	return 0;
 }
 
@@ -136,6 +137,7 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 	jobject resArray = env->CallStaticObjectMethod(d->helperClass, method, ((AndroidWebcamConfig*)d->webcam->data)->id, d->requestedSize.width, d->requestedSize.height);
 
 	if (!resArray) {
+		ms_mutex_unlock(&d->mutex);
 		ms_error("Failed to retrieve camera '%d' supported resolutions\n", ((AndroidWebcamConfig*)d->webcam->data)->id);
 		return -1;
 	}
@@ -145,7 +147,7 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
     //   - 1 : height
     //   - 2 : useDownscaling
 	jint res[3];
-   env->GetIntArrayRegion((jintArray)resArray, 0, 3, res);
+	env->GetIntArrayRegion((jintArray)resArray, 0, 3, res);
 	ms_message("Camera selected resolution is: %dx%d (requested: %dx%d) with downscaling?%d\n", res[0], res[1], d->requestedSize.width, d->requestedSize.height, res[2]);
 	d->hwCapableSize.width =  res[0];
 	d->hwCapableSize.height = res[1];
@@ -159,11 +161,16 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 	if ((hwSize * downscale * downscale) < rqSize) {
 		ms_message("Camera cannot produce requested resolution %dx%d, will supply smaller one: %dx%d\n",
 			d->requestedSize.width, d->requestedSize.height, (int) (res[0] * downscale), (int) (res[1]*downscale));
-		d->requestedSize.width = (int) (d->hwCapableSize.width * downscale);
-		d->requestedSize.height = (int) (d->hwCapableSize.height * downscale);
+		d->usedSize.width = (int) (d->hwCapableSize.width * downscale);
+		d->usedSize.height = (int) (d->hwCapableSize.height * downscale);
 	} else if ((hwSize * downscale * downscale) > rqSize) {
 		ms_message("Camera cannot produce requested resolution %dx%d, will capture a bigger one (%dx%d) and crop it to match encoder requested resolution\n",
 			d->requestedSize.width, d->requestedSize.height, (int)(res[0] * downscale), (int)(res[1] * downscale));
+		d->usedSize.width = d->requestedSize.width;
+		d->usedSize.height = d->requestedSize.height;
+	} else {
+		d->usedSize.width = d->requestedSize.width;
+		d->usedSize.height = d->requestedSize.height;
 	}
 	
 	// is phone held |_ to cam orientation ?
@@ -177,14 +184,14 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 			d->rotationSavedDuringVSize = d->rotation;
 		}
 		bool camIsLandscape = d->hwCapableSize.width > d->hwCapableSize.height;
-		bool reqIsLandscape = d->requestedSize.width > d->requestedSize.height;
+		bool useIsLandscape = d->usedSize.width > d->usedSize.height;
 
 		// if both are landscape or both portrait, swap
-		if (camIsLandscape == reqIsLandscape) {
-			int t = d->requestedSize.width;
-			d->requestedSize.width = d->requestedSize.height;
-			d->requestedSize.height = t;
-			ms_message("Swapped resolution width and height to : %dx%d\n", d->requestedSize.width, d->requestedSize.height);
+		if (camIsLandscape == useIsLandscape) {
+			int t = d->usedSize.width;
+			d->usedSize.width = d->usedSize.height;
+			d->usedSize.height = t;
+			ms_message("Swapped resolution width and height to : %dx%d\n", d->usedSize.width, d->usedSize.height);
 		}
 	} else {
 		d->rotationSavedDuringVSize = d->rotation;
@@ -196,7 +203,7 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 
 static int video_capture_get_vsize(MSFilter *f, void* data){
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
-	*(MSVideoSize*)data=d->requestedSize;
+	*(MSVideoSize*)data=d->usedSize;
 	return 0;
 }
 
@@ -208,6 +215,7 @@ static int video_capture_get_pix_fmt(MSFilter *f, void *data){
 // Java will give us a pointer to capture preview surface.
 static int video_set_native_preview_window(MSFilter *f, void *arg) {
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
+	
 	ms_mutex_lock(&d->mutex);
 
 	jobject w = *((jobject*)arg);
@@ -240,7 +248,6 @@ static int video_set_native_preview_window(MSFilter *f, void *arg) {
 						(jint)d->fps,
 						(d->rotation != UNDEFINED_ROTATION) ? d->rotation:0,
 						(jlong)d));
-
 		}
 		// if previewWindow AND camera are valid => set preview window
 		if (w && d->androidCamera)
@@ -248,10 +255,6 @@ static int video_set_native_preview_window(MSFilter *f, void *arg) {
 	} else {
 		ms_message("Preview capture window set but camera not created yet; remembering it for later use\n");
 	}
-	if (d->previewWindow) {
-		ms_message("Deleting previous preview window %p", d->previewWindow);
-		env->DeleteGlobalRef(d->previewWindow);
-	}
 	d->previewWindow = w;
 
 	ms_mutex_unlock(&d->mutex);
@@ -259,9 +262,9 @@ static int video_set_native_preview_window(MSFilter *f, void *arg) {
 }
 
 static int video_get_native_preview_window(MSFilter *f, void *arg) {
-    AndroidReaderContext* d = (AndroidReaderContext*) f->data;
-    arg = &d->previewWindow;
-    return 0;
+	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
+	arg = &d->previewWindow;
+	return 0;
 }
 
 static int video_set_device_rotation(MSFilter* f, void* arg) {
@@ -324,9 +327,10 @@ static void video_capture_process(MSFilter *f){
 static void video_capture_postprocess(MSFilter *f){
 	ms_message("Postprocessing of Android VIDEO capture filter");
 	AndroidReaderContext* d = getContext(f);
-	ms_mutex_lock(&d->mutex);
 	JNIEnv *env = ms_get_jni_env();
-
+	
+	ms_mutex_lock(&d->mutex);
+	
 	if (d->androidCamera) {
 		jmethodID method = env->GetStaticMethodID(d->helperClass,"stopRecording", "(Ljava/lang/Object;)V");
 
@@ -335,6 +339,10 @@ static void video_capture_postprocess(MSFilter *f){
 	}
 	d->androidCamera = 0;
 	d->previewWindow = 0;
+	if (d->frame){
+		freemsg(d->frame);
+		d->frame=NULL;
+	}
 	ms_mutex_unlock(&d->mutex);
 }
 
@@ -439,6 +447,9 @@ static void video_capture_detect(MSWebCamManager *obj){
 		ms_web_cam_manager_add_cam(obj,cam);
 		ms_message("camera created: id=%d frontFacing=%d orientation=%d [msid:%s]\n", c->id, c->frontFacing, c->orientation, idstring);
 	}
+	env->DeleteLocalRef(indexes);
+	env->DeleteLocalRef(frontFacing);
+	env->DeleteLocalRef(orientation);
 
 	env->DeleteGlobalRef(helperClass);
 	ms_message("Detection of Android VIDEO cards done");
@@ -450,19 +461,17 @@ static void video_capture_detect(MSWebCamManager *obj){
 #ifdef __cplusplus
 extern "C" {
 #endif
-	
-JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoApi5JniWrapper_setAndroidSdkVersion
-  (JNIEnv *env, jclass c, jint version) {
-	android_sdk_version = version;
-	ms_message("Android SDK version: %d\n", android_sdk_version);
-}
 
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoApi5JniWrapper_putImage(JNIEnv*  env,
 		jclass  thiz,jlong nativePtr,jbyteArray frame) {
 	AndroidReaderContext* d = (AndroidReaderContext*) nativePtr;
-	if (!d->androidCamera)
-		return;
+	
 	ms_mutex_lock(&d->mutex);
+	
+	if (!d->androidCamera){
+		ms_mutex_unlock(&d->mutex);
+		return;
+	}
 
 	if (!ms_video_capture_new_frame(&d->fpsControl,d->filter->ticker->time)) {
 		ms_mutex_unlock(&d->mutex);
@@ -484,7 +493,11 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoA
 	}
 
 	int y_cropping_offset=0, cbcr_cropping_offset=0;
-	//compute_cropping_offsets(d->hwCapableSize, d->requestedSize, &y_cropping_offset, &cbcr_cropping_offset);
+	MSVideoSize targetSize;
+	d->useDownscaling?targetSize.width=d->requestedSize.width*2:targetSize.width=d->requestedSize.width;
+	d->useDownscaling?targetSize.height=d->requestedSize.height*2:targetSize.height=d->requestedSize.height;
+
+	compute_cropping_offsets(d->hwCapableSize, targetSize, &y_cropping_offset, &cbcr_cropping_offset);
 
 	int width = d->hwCapableSize.width;
 	int height = d->hwCapableSize.height;
@@ -492,6 +505,7 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoA
 	uint8_t* y_src = (uint8_t*)(jinternal_buff + y_cropping_offset);
 	uint8_t* cbcr_src = (uint8_t*) (jinternal_buff + width * height + cbcr_cropping_offset);
 
+
 	/* Warning note: image_rotation_correction == 90 does not imply portrait mode !
 	   (incorrect function naming).
 	   It only implies one thing: image needs to rotated by that amount to be correctly
@@ -500,12 +514,12 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoA
  	mblk_t* yuv_block = copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(y_src
 														, cbcr_src
 														, image_rotation_correction
-														, d->requestedSize.width
-														, d->requestedSize.height
+														, d->usedSize.width
+														, d->usedSize.height
 														, d->hwCapableSize.width
-														, d->hwCapableSize.width,
-														false,
-														d->useDownscaling);
+														, d->hwCapableSize.width
+														, false
+														, d->useDownscaling);
 	if (yuv_block) {
 		if (d->frame)
 			freemsg(d->frame);
@@ -537,6 +551,7 @@ static int compute_image_rotation_correction(AndroidReaderContext* d, int rotati
 	return result % 360;
 }
 
+
 static void compute_cropping_offsets(MSVideoSize hwSize, MSVideoSize outputSize, int* yoff, int* cbcroff) {
 	// if hw <= out -> return
 	if (hwSize.width * hwSize.height <= outputSize.width * outputSize.height) {
@@ -552,25 +567,34 @@ static void compute_cropping_offsets(MSVideoSize hwSize, MSVideoSize outputSize,
 	*cbcroff = hwSize.width * halfDiffH * 0.5 + halfDiffW;
 }
 
+
 static jclass getHelperClassGlobalRef(JNIEnv *env) {
 	ms_message("getHelperClassGlobalRef (env: %p)", env);
-	
+	const char* className;
 	// FindClass only returns local references.
+	
+	// Find the current Android SDK version
+	jclass version = env->FindClass(VersionPath);
+	jmethodID method = env->GetStaticMethodID(version,"sdk", "()I");
+	android_sdk_version = env->CallStaticIntMethod(version, method);
+	ms_message("Android SDK version found is %i", android_sdk_version);
+	env->DeleteLocalRef(version);
+
 	if (android_sdk_version >= 9) {
-		jclass c = env->FindClass(AndroidApi9WrapperPath);
-		if (c == 0)
-			ms_error("Could not load class '%s' (%d)", AndroidApi9WrapperPath, android_sdk_version);
-		return reinterpret_cast<jclass>(env->NewGlobalRef(c));
+		className = AndroidApi9WrapperPath;
 	} else if (android_sdk_version >= 8) {
-		jclass c = env->FindClass(AndroidApi8WrapperPath);
-		if (c == 0)
-			ms_error("Could not load class '%s' (%d)", AndroidApi8WrapperPath, android_sdk_version);
-		return reinterpret_cast<jclass>(env->NewGlobalRef(c));
+		className = AndroidApi8WrapperPath;
 	} else {
-		jclass c = env->FindClass(AndroidApi5WrapperPath);
-		if (c == 0)
-			ms_error("Could not load class '%s' (%d)", AndroidApi5WrapperPath, android_sdk_version);
-		return reinterpret_cast<jclass>(env->NewGlobalRef(c));
+		className = AndroidApi5WrapperPath;
+	}
+	jclass c = env->FindClass(className);
+	if (c == 0) {
+		ms_error("Could not load class '%s' (%d)", className, android_sdk_version);
+		return NULL;
+	} else {
+		jclass globalRef = reinterpret_cast<jclass>(env->NewGlobalRef(c));
+		env->DeleteLocalRef(c);
+		return globalRef;
 	}
 }
 
