@@ -84,8 +84,7 @@ typedef struct V4l2State{
 	int frame_max;
 	float fps;
 	unsigned int start_time;
-	unsigned int last_frame_time;
-	float mean_inter_frame;
+	MSAverageFPS avgfps;
 	int th_frame_count;
 	int queued;
 	bool_t configured;
@@ -110,17 +109,16 @@ static int msv4l2_close(V4l2State *s){
 	return 0;
 }
 
-static bool_t v4lv2_try_format( V4l2State *s, struct v4l2_format *fmt, int fmtid){
-	
+static bool_t v4lv2_try_format( int fd, struct v4l2_format *fmt, int fmtid){
 	fmt->type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	fmt->fmt.pix.pixelformat = fmtid;
 	fmt->fmt.pix.field = V4L2_FIELD_ANY;
 
-        if (v4l2_ioctl (s->fd, VIDIOC_TRY_FMT, fmt)<0){
+	if (v4l2_ioctl (fd, VIDIOC_TRY_FMT, fmt)<0){
 		ms_message("VIDIOC_TRY_FMT: %s",strerror(errno));
 		return FALSE;
 	}
-	if (v4l2_ioctl (s->fd, VIDIOC_S_FMT, fmt)<0){
+	if (v4l2_ioctl (fd, VIDIOC_S_FMT, fmt)<0){
 		ms_message("VIDIOC_S_FMT: %s",strerror(errno));
 		return FALSE;
 	}
@@ -144,10 +142,177 @@ static int get_picture_buffer_size(MSPixFmt pix_fmt, int w, int h){
 	return 0;
 }
 
+static int query_max_fps_for_format_resolution(int fd, int pixelformat, MSVideoSize vsize) {
+	int fps = -1;
+	struct v4l2_frmivalenum frmival;
+	frmival.index = 0;
+	frmival.pixel_format = pixelformat;
+	frmival.width = vsize.width;
+	frmival.height = vsize.height;
+
+	while (v4l2_ioctl(fd, VIDIOC_ENUM_FRAMEINTERVALS, &frmival) >= 0) {
+		if (frmival.type == V4L2_FRMIVAL_TYPE_DISCRETE) {
+			fps = MAX(fps, (int) (frmival.discrete.denominator / frmival.discrete.numerator));
+		} else if (frmival.type == V4L2_FRMIVAL_TYPE_STEPWISE) {
+			return (int) (frmival.stepwise.max.denominator / frmival.stepwise.max.numerator);
+		} else if (frmival.type == V4L2_FRMIVAL_TYPE_CONTINUOUS) {
+			return (int) (frmival.stepwise.min.denominator / frmival.stepwise.min.numerator);
+		}
+		frmival.index++;
+	}
+	return fps;
+}
+
+typedef struct _V4L2FormatDescription {
+	/* format */
+	int pixel_format;
+	/* max fps */
+	int max_fps;
+	/* native or emulated */
+	bool_t native;
+	/* compressed or not */
+	bool_t compressed;
+	/*format is supported*/
+	bool_t supported;
+	
+} V4L2FormatDescription;
+
+static MSPixFmt v4l2_format_to_ms(int v4l2format) {
+	switch (v4l2format) {
+		case V4L2_PIX_FMT_YUV420:
+			return MS_YUV420P;
+		case V4L2_PIX_FMT_YUYV:
+			return MS_YUYV;
+		case V4L2_PIX_FMT_MJPEG:
+			return MS_MJPEG;
+		case V4L2_PIX_FMT_RGB24:
+			return MS_RGB24;
+		default:
+			ms_error("Unknown v4l2 format 0x%08x", v4l2format);
+			return MS_PIX_FMT_UNKNOWN;
+	}
+}
+
+static const V4L2FormatDescription* query_format_description_for_size(int fd, MSVideoSize vsize) {
+	/* hardcode supported format in preferred order*/
+	static V4L2FormatDescription formats[4];
+	memset(formats,0,sizeof(formats));
+	formats[0].pixel_format = V4L2_PIX_FMT_YUV420;
+	formats[0].max_fps = -1;
+	formats[1].pixel_format = V4L2_PIX_FMT_YUYV;
+	formats[1].max_fps = -1;
+	formats[2].pixel_format = V4L2_PIX_FMT_MJPEG;
+	formats[2].max_fps = -1;
+	formats[3].pixel_format = V4L2_PIX_FMT_RGB24;
+	formats[3].max_fps = -1;
+
+	{
+		struct v4l2_fmtdesc fmt;
+		memset(&fmt, 0, sizeof(fmt));
+		fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+
+		while (v4l2_ioctl(fd, VIDIOC_ENUM_FMT, &fmt) >= 0) {
+			int i;
+			for (i=0; i<4; i++) {
+				if (fmt.pixelformat == formats[i].pixel_format) {
+					formats[i].max_fps = query_max_fps_for_format_resolution(fd, fmt.pixelformat, vsize);
+					formats[i].native = !(fmt.flags & V4L2_FMT_FLAG_EMULATED);
+					formats[i].compressed = fmt.flags & V4L2_FMT_FLAG_COMPRESSED;
+					formats[i].supported = TRUE;
+					ms_message("format %s : max_fps=%i, native=%i, compressed=%i",
+						   ms_pix_fmt_to_string(v4l2_format_to_ms(fmt.pixelformat)),
+						   formats[i].max_fps,
+						   formats[i].native,
+						   formats[i].compressed);
+					break;
+				}
+			}
+			fmt.index++;
+		}
+	}
+	return formats;
+}
+
+static MSPixFmt pick_best_format(int fd, const V4L2FormatDescription* format_desc, MSVideoSize vsize) {
+	/* rules for picking a format are:
+	    - only max_fps >= 15 images/sec are considered
+	    - native > compressed > emulated
+	*/
+	enum { PREFER_NATIVE = 0, PREFER_COMPRESSED, NO_PREFERENCE} i;
+	int j;
+	for (i=PREFER_NATIVE; i<=NO_PREFERENCE; i++) {
+		for (j=0; j<4; j++) {
+			int candidate = -1;
+			if (!format_desc[j].supported) continue;
+			switch (i) {
+				case PREFER_NATIVE:
+					if (format_desc[j].max_fps >= 15 && format_desc[j].native)
+						candidate = j;
+					break;
+				case PREFER_COMPRESSED:
+					/*usually compressed format allow the biggest picture size*/
+					if (format_desc[j].compressed)
+						candidate = j;
+					break;
+				case NO_PREFERENCE:
+				default:
+					candidate = j;
+					break;
+			}
+
+			if (candidate != -1) {
+				struct v4l2_format fmt;
+				fmt.fmt.pix.width       = vsize.width;
+				fmt.fmt.pix.height      = vsize.height;
+				ms_message("Candidate: %i",candidate);
+
+				if (v4lv2_try_format(fd, &fmt, format_desc[j].pixel_format)) {
+					MSPixFmt selected=v4l2_format_to_ms(format_desc[j].pixel_format);
+					ms_message("V4L2: selected format is %s", ms_pix_fmt_to_string(selected));
+					return selected;
+				}
+			}
+		}
+	}
+
+	ms_error("No compatible format found");
+	return MS_PIX_FMT_UNKNOWN;
+}
+
+static int set_camera_feature(V4l2State *s, unsigned int ctl_id, int value, const char *feature_name){
+	struct v4l2_ext_control ctl={0};
+	struct v4l2_ext_controls ctls={0};
+	struct v4l2_queryctrl queryctrl={0};
+	
+	queryctrl.id = ctl_id;
+	if (ioctl (s->fd, VIDIOC_QUERYCTRL, &queryctrl)!=0) {
+		ms_warning("%s not supported: %s",feature_name,strerror(errno));
+		return -1;
+	} else if (queryctrl.flags & V4L2_CTRL_FLAG_DISABLED) {
+		ms_warning("%s setting disabled.",feature_name);
+		return -1;
+	}else {
+		ctl.id=ctl_id;
+		ctl.value=value;
+		ctl.size=sizeof(int);
+		ctls.count=1;
+		ctls.controls=&ctl;
+		ctls.ctrl_class=V4L2_CTRL_CLASS_CAMERA;
+
+		if (v4l2_ioctl(s->fd,VIDIOC_S_EXT_CTRLS,&ctls)!=0){
+			ms_warning("Could not enable %s: %s", feature_name, strerror(errno));
+			return -1;
+		}
+	}
+	return 0;
+}
+
+
 static int msv4l2_configure(V4l2State *s){
 	struct v4l2_capability cap;
 	struct v4l2_format fmt;
 	MSVideoSize vsize;
+	const char *focus;
 
 	if (v4l2_ioctl (s->fd, VIDIOC_QUERYCAP, &cap)<0) {
 		ms_message("Not a v4lv2 driver.");
@@ -163,8 +328,10 @@ static int msv4l2_configure(V4l2State *s){
 		ms_error("%s does not support streaming i/o\n",s->dev);
 		return -1;
 	}
-	
+
+
 	ms_message("Driver is %s",cap.driver);
+
 	memset(&fmt,0,sizeof(fmt));
 
 	fmt.type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -172,35 +339,15 @@ static int msv4l2_configure(V4l2State *s){
 		ms_error("VIDIOC_G_FMT failed: %s",strerror(errno));
 	}
 	vsize=s->vsize;
+
 	do{
-		fmt.fmt.pix.width       = s->vsize.width; 
-		fmt.fmt.pix.height      = s->vsize.height;
-		ms_message("v4l2: trying %ix%i",s->vsize.width,s->vsize.height);
-		if (v4lv2_try_format(s,&fmt,V4L2_PIX_FMT_YUV420)){
-			s->pix_fmt=MS_YUV420P;
-			s->int_pix_fmt=V4L2_PIX_FMT_YUV420;
-			ms_message("v4lv2: YUV420P chosen");
-			break;
-		}else if (v4lv2_try_format(s,&fmt,V4L2_PIX_FMT_YUYV)){
-			s->pix_fmt=MS_YUYV;
-			s->int_pix_fmt=V4L2_PIX_FMT_YUYV;
-			ms_message("v4lv2: V4L2_PIX_FMT_YUYV chosen");
-			break;
-		}else if (v4lv2_try_format(s,&fmt,V4L2_PIX_FMT_RGB24)){
-			s->pix_fmt=MS_RGB24;
-			s->int_pix_fmt=V4L2_PIX_FMT_RGB24;
-			ms_message("v4lv2: RGB24 chosen");
-			break;
-		}else if (v4lv2_try_format(s,&fmt,V4L2_PIX_FMT_MJPEG)){
-			s->pix_fmt=MS_MJPEG;
-			s->int_pix_fmt=V4L2_PIX_FMT_MJPEG;
-			ms_message("v4lv2: MJPEG chosen");
-			break;
-		}else{
-			ms_error("Could not find supported pixel format for %ix%i", s->vsize.width, s->vsize.height);
-		}
-		s->vsize=ms_video_size_get_just_lower_than(s->vsize);
-	}while(s->vsize.width!=0);
+		const V4L2FormatDescription* formats_desc = query_format_description_for_size(s->fd, s->vsize);
+		s->pix_fmt = pick_best_format(s->fd, formats_desc, s->vsize);
+
+		if (s->pix_fmt == MS_PIX_FMT_UNKNOWN)
+			s->vsize=ms_video_size_get_just_lower_than(s->vsize);
+	} while(s->vsize.width!=0 && (s->pix_fmt == MS_PIX_FMT_UNKNOWN));
+
 	if (s->vsize.width==0){
 		ms_message("Could not find any combination of resolution/pixel-format that works !");
 		s->vsize=vsize;
@@ -213,11 +360,27 @@ static int msv4l2_configure(V4l2State *s){
 	if (v4l2_ioctl (s->fd, VIDIOC_G_FMT, &fmt)<0){
 		ms_error("VIDIOC_G_FMT failed: %s",strerror(errno));
 	}else{
-		ms_message("Size of webcam delivered pictures is %ix%i",fmt.fmt.pix.width,fmt.fmt.pix.height);
+		ms_message("Size of webcam delivered pictures is %ix%i. Format:0x%08x",fmt.fmt.pix.width,fmt.fmt.pix.height, s->pix_fmt);
 		s->vsize.width=fmt.fmt.pix.width;
 		s->vsize.height=fmt.fmt.pix.height;
 	}
 	s->picture_size=get_picture_buffer_size(s->pix_fmt,s->vsize.width,s->vsize.height);
+	
+	focus=getenv("MS2_CAM_FOCUS");
+	if (focus){
+		if (strcasecmp(focus,"auto")==0){
+#ifdef V4L2_CID_AUTO_FOCUS_RANGE
+			set_camera_feature(s,V4L2_CID_AUTO_FOCUS_RANGE,V4L2_AUTO_FOCUS_RANGE_AUTO ,"auto range");
+#endif
+			set_camera_feature(s,V4L2_CID_FOCUS_AUTO,1,"auto-focus");
+		}else if (strcasecmp(focus,"infinity")==0){
+#ifdef V4L2_CID_AUTO_FOCUS_RANGE
+			set_camera_feature(s,V4L2_CID_AUTO_FOCUS_RANGE,V4L2_AUTO_FOCUS_RANGE_INFINITY ,"infinity range");
+#endif
+			set_camera_feature(s,V4L2_CID_FOCUS_AUTO,1,"auto-focus");
+		}
+	}
+	
 	s->configured=TRUE;
 	return 0;
 }
@@ -226,39 +389,39 @@ static int msv4l2_do_mmap(V4l2State *s){
 	struct v4l2_requestbuffers req;
 	int i;
 	enum v4l2_buf_type type;
-	
+
 	memset(&req,0,sizeof(req));
-	
+
 	req.count               = 4;
 	req.type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	req.memory              = V4L2_MEMORY_MMAP;
-	
+
 	if (v4l2_ioctl (s->fd, VIDIOC_REQBUFS, &req)<0) {
 		ms_error("Error requesting info on mmap'd buffers: %s",strerror(errno));
 		return -1;
 	}
-	
+
 	for (i=0; i<req.count; ++i) {
 		struct v4l2_buffer buf;
 		mblk_t *msg;
 		void *start;
 		memset(&buf,0,sizeof(buf));
-	
+
 		buf.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
 		buf.memory=V4L2_MEMORY_MMAP;
 		buf.index=i;
-	
+
 		if (v4l2_ioctl (s->fd, VIDIOC_QUERYBUF, &buf)<0){
 			ms_error("Could not VIDIOC_QUERYBUF : %s",strerror(errno));
 			return -1;
 		}
-		
+
 		start=v4l2_mmap (NULL /* start anywhere */,
 			buf.length,
 			PROT_READ | PROT_WRITE /* required */,
 			MAP_SHARED /* recommended */,
 			s->fd, buf.m.offset);
-	
+
 		if (start==NULL){
 			ms_error("Could not v4l2_mmap: %s",strerror(errno));
 		}
@@ -294,11 +457,11 @@ static mblk_t *v4l2_dequeue_ready_buffer(V4l2State *s, int poll_timeout_ms){
 	struct v4l2_buffer buf;
 	mblk_t *ret=NULL;
 	struct pollfd fds;
-	
+
 	memset(&buf,0,sizeof(buf));
 	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	buf.memory = V4L2_MEMORY_MMAP;
-	
+
 	memset(&fds,0,sizeof(fds));
 	fds.events=POLLIN;
 	fds.fd=s->fd;
@@ -340,21 +503,23 @@ static mblk_t *v4l2_dequeue_ready_buffer(V4l2State *s, int poll_timeout_ms){
 static mblk_t * v4lv2_grab_image(V4l2State *s, int poll_timeout_ms){
 	struct v4l2_buffer buf;
 	unsigned int k;
-	memset(&buf,0,sizeof(buf));
+	bool_t no_slot_available = TRUE;
 	mblk_t *ret=NULL;
+	
+	memset(&buf,0,sizeof(buf));
 
 	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	buf.memory = V4L2_MEMORY_MMAP;
-	
+
 	/*queue buffers whose ref count is 1, because they are not
-	still used anywhere in the filter chain */
+	 used anywhere in the filter chain */
 	for(k=0;k<s->frame_max;++k){
 		if (s->frames[k]->b_datap->db_ref==1){
+			no_slot_available = FALSE;
 			buf.index=k;
 			if (-1==v4l2_ioctl (s->fd, VIDIOC_QBUF, &buf))
 				ms_warning("VIDIOC_QBUF %i failed: %s",k,  strerror(errno));
 			else {
-				ms_debug("v4l2: queue buf %i",k);
 				/*increment ref count of queued buffer*/
 				inc_ref(s->frames[k]);
 				s->queued++;
@@ -364,6 +529,8 @@ static mblk_t * v4lv2_grab_image(V4l2State *s, int poll_timeout_ms){
 
 	if (s->queued){
 		ret=v4l2_dequeue_ready_buffer(s,poll_timeout_ms);
+	}else if (no_slot_available){
+		ms_usleep(100000);
 	}
 	return ret;
 }
@@ -405,14 +572,14 @@ static void msv4l2_uninit(MSFilter *f){
 	V4l2State *s=(V4l2State*)f->data;
 	ms_free(s->dev);
 	flushq(&s->rq,0);
-	ms_mutex_destroy(&s->mutex);	
+	ms_mutex_destroy(&s->mutex);
 	ms_free(s);
 }
 
 static void *msv4l2_thread(void *ptr){
 	V4l2State *s=(V4l2State*)ptr;
 	int try=0;
-	
+
 	ms_message("msv4l2_thread starting");
 	if (s->fd==-1){
 		if( msv4l2_open(s)!=0){
@@ -420,16 +587,18 @@ static void *msv4l2_thread(void *ptr){
 			goto close;
 		}
 	}
-	
+
 	if (!s->configured && msv4l2_configure(s)!=0){
-		ms_warning("msv4l2 could not be configured");		
+		ms_warning("msv4l2 could not be configured");
 		goto close;
 	}
+	
 	if (msv4l2_do_mmap(s)!=0)
 	{
 		ms_warning("msv4l2 do mmap");
 		goto close;
 	}
+	
 	ms_message("V4L2 video capture started.");
 	while(s->thread_run)
 	{
@@ -448,6 +617,7 @@ static void *msv4l2_thread(void *ptr){
 	/*dequeue pending buffers so that we can properly unref them (avoids memleak )*/
 	while(s->queued && try<10){
 		v4l2_dequeue_ready_buffer(s,50);
+		try++;
 	}
 	if (try==10) ms_warning("msv4l2: buffers not dequeued at exit !");
 	msv4l2_do_munmap(s);
@@ -463,7 +633,7 @@ static void msv4l2_preprocess(MSFilter *f){
 	s->thread_run=TRUE;
 	ms_thread_create(&s->thread,NULL,msv4l2_thread,s);
 	s->th_frame_count=-1;
-	s->mean_inter_frame=0;
+	ms_average_fps_init(&s->avgfps,"V4L2 capture: fps=%f");
 }
 
 static void msv4l2_process(MSFilter *f){
@@ -472,14 +642,14 @@ static void msv4l2_process(MSFilter *f){
 	int cur_frame;
 	uint32_t curtime=f->ticker->time;
 	float elapsed;
-	
+
 	if (s->th_frame_count==-1){
 		s->start_time=curtime;
 		s->th_frame_count=0;
 	}
 	elapsed=((float)(curtime-s->start_time))/1000.0;
 	cur_frame=elapsed*s->fps;
-	
+
 	if (cur_frame>=s->th_frame_count){
 		mblk_t *om=NULL;
 		ms_mutex_lock(&s->mutex);
@@ -497,20 +667,9 @@ static void msv4l2_process(MSFilter *f){
 			mblk_set_timestamp_info(om,timestamp);
 			mblk_set_marker_info(om,TRUE);
 			ms_queue_put(f->outputs[0],om);
-			if (s->last_frame_time!=-1){
-				float frame_interval=(float)(curtime-s->last_frame_time)/1000.0;
-				if (s->mean_inter_frame==0){
-					s->mean_inter_frame=frame_interval;
-				}else{
-					s->mean_inter_frame=(0.8*s->mean_inter_frame)+(0.2*frame_interval);
-				}
-			}
-			s->last_frame_time=curtime;
+			ms_average_fps_update(&s->avgfps,f->ticker->time);
 		}
 		s->th_frame_count++;
-		if (s->th_frame_count%50==0 && s->mean_inter_frame!=0){
-			ms_message("Captured mean fps=%f, expected=%f",1/s->mean_inter_frame, s->fps);
-		}
 	}
 }
 
@@ -574,11 +733,20 @@ static int msv4l2_set_devfile(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int msv4l2_get_fps(MSFilter *f, void *arg){
+	V4l2State *s=(V4l2State*)f->data;
+	if (f->ticker){
+		*(float*)arg=ms_average_fps_get(&s->avgfps);
+	}else *(float*)arg=s->fps;
+	return 0;
+}
+
 static MSFilterMethod msv4l2_methods[]={
 	{	MS_FILTER_SET_FPS	,	msv4l2_set_fps	},
 	{	MS_FILTER_SET_VIDEO_SIZE,	msv4l2_set_vsize	},
 	{	MS_FILTER_GET_VIDEO_SIZE,	msv4l2_get_vsize	},
 	{	MS_FILTER_GET_PIX_FMT	,	msv4l2_get_pixfmt	},
+	{	MS_FILTER_GET_FPS	,	msv4l2_get_fps	},
 	{	0			,	NULL		}
 };
 
@@ -622,6 +790,7 @@ static void msv4l2_detect(MSWebCamManager *obj){
 	struct v4l2_capability cap;
 	char devname[32];
 	int i;
+
 	for(i=0;i<10;++i){
 		int fd;
 		snprintf(devname,sizeof(devname),"/dev/video%i",i);
