@@ -26,6 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msticker.h"
+#include "mediastreamer2/videostarter.h"
 
 #ifdef _WIN32
 #include <ws2tcpip.h>
@@ -37,8 +38,70 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define RATE_CONTROL_MARGIN 15000 /*bits/second*/
 
+#define MS_VIDEOENC_CONF(required_bitrate, bitrate_limit, resolution, fps, cpu, qminvalue) \
+	{ required_bitrate, bitrate_limit, { MS_VIDEO_SIZE_ ## resolution ## _W, MS_VIDEO_SIZE_ ## resolution ## _H }, fps, cpu, (void *)&qmin ## qminvalue }
+
 static bool_t avcodec_initialized=FALSE;
 
+static const int qmin2 = 2;
+static const int qmin3 = 3;
+#if HAVE_AVCODEC_SNOW
+static const int qmin4 = 4;
+#endif
+static const int qmin5 = 5;
+
+static const MSVideoConfiguration h263_conf_list[] = {
+	MS_VIDEOENC_CONF( 800000, 1024000,  4CIF, 25, 2, 2),
+	MS_VIDEOENC_CONF( 512000,  800000,  CIF, 25, 1, 2),
+	MS_VIDEOENC_CONF( 256000,  512000,  CIF, 17, 1, 3),
+	MS_VIDEOENC_CONF( 128000,  256000, QCIF, 10, 1, 3),
+	MS_VIDEOENC_CONF(      0,  128000, QCIF,  5, 1, 5)
+};
+
+static const MSVideoConfiguration h263p_conf_list[] = {
+	MS_VIDEOENC_CONF(800000, 1024000, 4CIF, 25, 2, 2),
+	MS_VIDEOENC_CONF(512000,  800000,  CIF, 25, 1, 2),
+	MS_VIDEOENC_CONF(256000,  512000,  CIF, 17, 1, 3),
+	MS_VIDEOENC_CONF(128000,  256000, QCIF, 10, 1, 3),
+	MS_VIDEOENC_CONF(     0,  128000, QCIF,  5, 1, 5)
+};
+
+static const MSVideoConfiguration mjpeg_conf_list[] = {
+	MS_VIDEOENC_CONF(1024000, 1536000, SVGA, 25, 1, 2),
+	MS_VIDEOENC_CONF( 800000, 1024000,  VGA, 25, 1, 2),
+	MS_VIDEOENC_CONF( 512000,  800000,  CIF, 25, 1, 2),
+	MS_VIDEOENC_CONF( 256000,  512000,  CIF, 17, 1, 3),
+	MS_VIDEOENC_CONF( 170000,  256000, QVGA, 15, 1, 3),
+	MS_VIDEOENC_CONF( 128000,  170000, QCIF, 10, 1, 3),
+	MS_VIDEOENC_CONF(      0,  128000, QCIF,  5, 1, 5)
+};
+
+static const MSVideoConfiguration mpeg4_conf_list[] = {
+    	MS_VIDEOENC_CONF(2048000, 2560000, SXGA_MINUS, 25, 4, 2),
+	MS_VIDEOENC_CONF(2048000, 2560000, 720P, 25, 4, 2),
+    	MS_VIDEOENC_CONF(1536000, 2048000,  XGA, 25, 4, 2),
+	MS_VIDEOENC_CONF(1024000, 1536000, SVGA, 25, 2, 2),
+	MS_VIDEOENC_CONF( 800000, 1024000,  VGA, 25, 1, 2),
+	MS_VIDEOENC_CONF( 512000,  800000,  CIF, 25, 1, 2),
+	MS_VIDEOENC_CONF( 256000,  512000,  CIF, 17, 1, 3),
+	MS_VIDEOENC_CONF( 170000,  256000, QVGA, 15, 1, 3),
+	MS_VIDEOENC_CONF( 128000,  170000, QCIF, 10, 1, 3),
+	MS_VIDEOENC_CONF(      0,  128000, QCIF,  5, 1, 5)
+};
+
+#if HAVE_AVCODEC_SNOW
+static const MSVideoConfiguration snow_conf_list[] = {
+	MS_VIDEOENC_CONF(1024000, 1536000, SVGA, 25, 2, 2),
+	MS_VIDEOENC_CONF( 800000, 1024000,  VGA, 25, 1, 2),
+	MS_VIDEOENC_CONF( 512000,  800000,  CIF, 25, 1, 2),
+	MS_VIDEOENC_CONF( 256000,  512000,  CIF, 17, 1, 3),
+	MS_VIDEOENC_CONF( 170000,  256000, QVGA, 15, 1, 3),
+	MS_VIDEOENC_CONF( 128000,  170000, QCIF, 10, 1, 3),
+	MS_VIDEOENC_CONF(  64000,  128000, QCIF,  7, 1, 4),
+	MS_VIDEOENC_CONF(      0,   64000, QCIF,  5, 1, 5)
+};
+#endif
+
 #ifndef FF_I_TYPE
 #define FF_I_TYPE AV_PICTURE_TYPE_I
 #endif
@@ -47,19 +110,13 @@ static bool_t avcodec_initialized=FALSE;
 
 void ms_ffmpeg_log_callback(void* ptr, int level, const char* fmt, va_list vl)
 {
-	static char message[8192];
-
-    vsnprintf(message, sizeof message, fmt, vl);
-	ms_message(message);
+	ortp_logv(ORTP_MESSAGE,fmt,vl);
 }
 
 #endif
 
 void ms_ffmpeg_check_init(){
 	if(!avcodec_initialized){
-#ifdef FF_API_AVCODEC_INIT
-		avcodec_init();
-#endif
 		avcodec_register_all();
 		avcodec_initialized=TRUE;
 #ifdef ENABLE_LOG_FFMPEG
@@ -69,82 +126,22 @@ void ms_ffmpeg_check_init(){
 	}
 }
 
-/* the goal of this small object is to tell when to send I frames at startup:
-at 2 and 4 seconds*/
-typedef struct VideoStarter{
-	uint64_t next_time;
-	int i_frame_count;
-}VideoStarter;
-
-static void video_starter_init(VideoStarter *vs){
-	vs->next_time=0;
-	vs->i_frame_count=0;
-}
-
-static void video_starter_first_frame(VideoStarter *vs, uint64_t curtime){
-	vs->next_time=curtime+2000;
-}
-
-static bool_t video_starter_need_i_frame(VideoStarter *vs, uint64_t curtime){
-	if (vs->next_time==0) return FALSE;
-	if (curtime>=vs->next_time){
-		vs->i_frame_count++;
-		if (vs->i_frame_count==1){
-			vs->next_time+=2000;
-		}else{
-			vs->next_time=0;
-		}
-		return TRUE;
-	}
-	return FALSE;
-}
-
 typedef struct EncState{
 	AVCodecContext av_context;
 	AVCodec *av_codec;
+	AVFrame* pict;
 	enum CodecID codec;
 	mblk_t *comp_buf;
-	MSVideoSize vsize;
 	int mtu;	/* network maximum transmission unit in bytes */
 	int profile;
-	float fps;
-	int maxbr;
 	int qmin;
 	uint32_t framenum;
-	VideoStarter starter;
+	MSVideoStarter starter;
 	bool_t req_vfu;
+	const MSVideoConfiguration *vconf_list;
+	MSVideoConfiguration vconf;
 }EncState;
 
-static int enc_set_fps(MSFilter *f, void *arg){
-	EncState *s=(EncState*)f->data;
-	s->fps=*(float*)arg;
-	return 0;
-}
-
-static int enc_get_fps(MSFilter *f, void *arg){
-	EncState *s=(EncState*)f->data;
-	*(float*)arg=s->fps;
-	return 0;
-}
-
-static int enc_set_vsize(MSFilter *f,void *arg){
-	EncState *s=(EncState*)f->data;
-	s->vsize=*(MSVideoSize*)arg;
-	return 0;
-}
-
-static int enc_get_vsize(MSFilter *f,void *arg){
-	EncState *s=(EncState*)f->data;
-	*(MSVideoSize*)arg=s->vsize;
-	return 0;
-}
-
-static int enc_set_mtu(MSFilter *f,void *arg){
-	EncState *s=(EncState*)f->data;
-	s->mtu=*(int*)arg;
-	return 0;
-}
-
 static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize){
 	char *tmp=ms_strdup(fmtp);
 	char *semicolon;
@@ -193,7 +190,7 @@ static int enc_add_fmtp(MSFilter *f,void *arg){
 	char val[10];
 	if (fmtp_get_value(fmtp,"profile",val,sizeof(val))){
 		s->profile=atoi(val);
-	}else parse_video_fmtp(fmtp,&s->fps,&s->vsize);
+	}else parse_video_fmtp(fmtp,&s->vconf.fps,&s->vconf.vsize);
 	return 0;
 }
 
@@ -203,23 +200,40 @@ static int enc_req_vfu(MSFilter *f, void *unused){
 	return 0;
 }
 
+static const MSVideoConfiguration * get_vconf_list(EncState *s) {
+	switch (s->codec) {
+		case CODEC_ID_H263:
+			return  &h263_conf_list[0];
+		case CODEC_ID_H263P:
+			return &h263p_conf_list[0];
+		case CODEC_ID_MJPEG:
+			return &mjpeg_conf_list[0];
+		case CODEC_ID_MPEG4:
+		default:
+			return &mpeg4_conf_list[0];
+#if HAVE_AVCODEC_SNOW
+		case CODEC_ID_SNOW:
+			return &snow_conf_list[0];
+#endif
+	}
+}
+
 static void enc_init(MSFilter *f, enum CodecID codec)
 {
-	EncState *s=(EncState *)ms_new(EncState,1);
+	EncState *s=ms_new0(EncState,1);
 	f->data=s;
 	ms_ffmpeg_check_init();
 	s->profile=0;/*always default to profile 0*/
 	s->comp_buf=NULL;
-	s->fps=15;
 	s->mtu=ms_get_payload_max_size()-2;/*-2 for the H263 payload header*/
-	s->maxbr=500000;
 	s->codec=codec;
-	s->vsize.width=MS_VIDEO_SIZE_CIF_W;
-	s->vsize.height=MS_VIDEO_SIZE_CIF_H;
 	s->qmin=2;
 	s->req_vfu=FALSE;
 	s->framenum=0;
 	s->av_context.codec=NULL;
+	s->vconf_list = get_vconf_list(s);
+	s->vconf = ms_video_find_best_configuration_for_bitrate(s->vconf_list, 500000, ms_get_cpu_count());
+	s->pict = av_frame_alloc();
 }
 
 static void enc_h263_init(MSFilter *f){
@@ -229,11 +243,11 @@ static void enc_h263_init(MSFilter *f){
 static void enc_mpeg4_init(MSFilter *f){
 	enc_init(f,CODEC_ID_MPEG4);
 }
-
+#if HAVE_AVCODEC_SNOW
 static void enc_snow_init(MSFilter *f){
 	enc_init(f,CODEC_ID_SNOW);
 }
-
+#endif
 static void enc_mjpeg_init(MSFilter *f){
 	enc_init(f,CODEC_ID_MJPEG);
 }
@@ -242,15 +256,15 @@ static void prepare(EncState *s){
 	AVCodecContext *c=&s->av_context;
 	const int max_br_vbv=128000;
 
-	avcodec_get_context_defaults(c);
+	avcodec_get_context_defaults3(c, NULL);
 	if (s->codec==CODEC_ID_MJPEG)
 	{
 		ms_message("Codec bitrate set to %i",c->bit_rate);
-		c->width = s->vsize.width;  
-		c->height = s->vsize.height;
+		c->width = s->vconf.vsize.width;
+		c->height = s->vconf.vsize.height;
 		c->time_base.num = 1;
-		c->time_base.den = (int)s->fps;
-		c->gop_size=(int)s->fps*5; /*emit I frame every 5 seconds*/
+		c->time_base.den = (int)s->vconf.fps;
+		c->gop_size=(int)s->vconf.fps*5; /*emit I frame every 5 seconds*/
 		c->pix_fmt=PIX_FMT_YUVJ420P;
 		s->comp_buf=allocb(c->bit_rate*2,0);
 		return;
@@ -258,18 +272,22 @@ static void prepare(EncState *s){
 
 	/* put codec parameters */
 	/* in order to take in account RTP protocol overhead and avoid possible
-	 bitrate peaks especially on low bandwidth, we make a correction on the 
+	 bitrate peaks especially on low bandwidth, we make a correction on the
 	 codec's target bitrate.
 	*/
-	c->bit_rate=(float)s->maxbr*0.92;
+	c->bit_rate=(float)s->vconf.required_bitrate*0.92;
 	if (c->bit_rate>RATE_CONTROL_MARGIN){
 		 c->bit_rate -= RATE_CONTROL_MARGIN;
 	}
-	c->bit_rate_tolerance=s->fps>1?(float)c->bit_rate/(s->fps-1):c->bit_rate;
+	c->bit_rate_tolerance=s->vconf.fps>1?(float)c->bit_rate/(s->vconf.fps-1):c->bit_rate;
 
 	/* ffmpeg vbv rate control consumes too much cpu above a certain target bitrate.
 	We don't use it above max_br_vbv */
-	if (s->codec!=CODEC_ID_SNOW && s->maxbr<max_br_vbv){
+	if (
+#if HAVE_AVCODEC_SNOW
+		s->codec!=CODEC_ID_SNOW &&
+#endif
+		s->vconf.required_bitrate<max_br_vbv){
 		/*snow does not like 1st pass rate control*/
 		c->rc_max_rate=c->bit_rate;
 		c->rc_min_rate=0;
@@ -280,17 +298,18 @@ static void prepare(EncState *s){
 	}
 
 	ms_message("Codec bitrate set to %i",c->bit_rate);
-	c->width = s->vsize.width;  
-	c->height = s->vsize.height;
+	c->width = s->vconf.vsize.width;
+	c->height = s->vconf.vsize.height;
 	c->time_base.num = 1;
-	c->time_base.den = (int)s->fps;
-	c->gop_size=(int)s->fps*10; /*emit I frame every 10 seconds*/
+	c->time_base.den = (int)s->vconf.fps;
+	c->gop_size=(int)s->vconf.fps*10; /*emit I frame every 10 seconds*/
 	c->pix_fmt=PIX_FMT_YUV420P;
 	s->comp_buf=allocb(c->bit_rate*2,0);
+#if HAVE_AVCODEC_SNOW
 	if (s->codec==CODEC_ID_SNOW){
 		c->strict_std_compliance=-2;
 	}
-	
+#endif
 	ms_message("Codec size set to w=%i/h=%i",c->width, c->height);
 
 }
@@ -324,6 +343,7 @@ static void prepare_mpeg4(EncState *s){
 
 static void enc_uninit(MSFilter  *f){
 	EncState *s=(EncState*)f->data;
+	if (s->pict) av_frame_free(&s->pict);
 	ms_free(s);
 }
 
@@ -335,9 +355,12 @@ static void enc_preprocess(MSFilter *f){
 		prepare_h263(s);
 	else if (s->codec==CODEC_ID_MPEG4)
 		prepare_mpeg4(s);
+#if HAVE_AVCODEC_SNOW
 	else if (s->codec==CODEC_ID_SNOW){
 		/**/
-	}else if (s->codec==CODEC_ID_MJPEG){
+	}
+#endif
+	else if (s->codec==CODEC_ID_MJPEG){
 		/**/
 	}else {
 		ms_error("Unsupported codec id %i",s->codec);
@@ -348,12 +371,12 @@ static void enc_preprocess(MSFilter *f){
 		ms_error("could not find encoder for codec id %i",s->codec);
 		return;
 	}
-	error=avcodec_open(&s->av_context, s->av_codec);
+	error=avcodec_open2(&s->av_context, s->av_codec, NULL);
 	if (error!=0) {
 		ms_error("avcodec_open() failed: %i",error);
 		return;
 	}
-	video_starter_init(&s->starter);
+	ms_video_starter_init(&s->starter);
 	ms_debug("image format is %i.",s->av_context.pix_fmt);
 	ms_message("qmin=%i qmax=%i",s->av_context.qmin,s->av_context.qmax);
 	s->framenum=0;
@@ -428,7 +451,7 @@ static int get_gbsc_bytealigned(uint8_t *begin, uint8_t *end){
 
 static void rfc2190_generate_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp){
 	mblk_t *packet=NULL;
-	
+
 	while (frame->b_rptr<frame->b_wptr){
 		packet=dupb(frame);
 		/*frame->b_rptr=packet->b_wptr=packet->b_rptr+get_gbsc(packet->b_rptr, MIN(packet->b_rptr+s->mtu,frame->b_wptr));*/
@@ -462,15 +485,15 @@ static void mpeg4_fragment_and_send(MSFilter *f,EncState *s,mblk_t *frame, uint3
 static void rfc4629_generate_follow_on_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp, uint8_t *psc, uint8_t *end, bool_t last_packet){
 	mblk_t *packet;
 	int len=end-psc;
-	
-	packet=dupb(frame);	
+
+	packet=dupb(frame);
 	packet->b_rptr=psc;
 	packet->b_wptr=end;
 	/*ms_message("generating packet of size %i",end-psc);*/
 	rfc2429_set_P(psc,1);
 	mblk_set_timestamp_info(packet,timestamp);
 
-	
+
 	if (len>s->mtu){
 		/*need to slit the packet using "follow-on" packets */
 		/*compute the number of packets need (rounded up)*/
@@ -582,17 +605,17 @@ static void mjpeg_fragment_and_send(MSFilter *f,EncState *s,mblk_t *frame, uint3
 	struct jpeghdr_qtable qtblhdr;
 	int bytes_left = msgdsize(frame);
 	int data_len;
-	
+
 	mblk_t *packet;
-	
+
 	/* Initialize JPEG header
 	 */
 	//jpghdr.tspec = typespec;
 	jpghdr.off = 0;
 	jpghdr.type = type | ((dri != 0) ? RTP_JPEG_RESTART : 0);
 	jpghdr.q = q;
-	jpghdr.width = s->vsize.width / 8;
-	jpghdr.height = s->vsize.height / 8;
+	jpghdr.width = s->vconf.vsize.width / 8;
+	jpghdr.height = s->vconf.vsize.height / 8;
 
 	/* Initialize DRI header
 	 */
@@ -643,9 +666,9 @@ static void mjpeg_fragment_and_send(MSFilter *f,EncState *s,mblk_t *frame, uint3
 			mblk_set_marker_info(packet,TRUE);
 		}
 
-		memcpy(packet->b_wptr, frame->b_rptr + jpghdr.off, data_len);	
+		memcpy(packet->b_wptr, frame->b_rptr + jpghdr.off, data_len);
 		packet->b_wptr=packet->b_wptr + data_len;
-				
+
 		mblk_set_timestamp_info(packet,timestamp);
 		ms_queue_put(f->outputs[0],packet);
 
@@ -677,7 +700,7 @@ static int find_marker(uint8_t **pbuf_ptr, uint8_t *buf_end){
 static mblk_t *skip_jpeg_headers(mblk_t *full_frame, mblk_t **lqt, mblk_t **cqt){
 	int err;
 	uint8_t *pbuf_ptr=full_frame->b_rptr;
-	uint8_t *buf_end=full_frame->b_wptr;	
+	uint8_t *buf_end=full_frame->b_wptr;
 
 	ms_message("image size: %li)", (long)(buf_end-pbuf_ptr));
 
@@ -723,8 +746,12 @@ static void split_and_send(MSFilter *f, EncState *s, mblk_t *frame){
 	uint8_t *lastpsc;
 	uint8_t *psc;
 	uint32_t timestamp=f->ticker->time*90LL;
-	
-	if (s->codec==CODEC_ID_MPEG4 || s->codec==CODEC_ID_SNOW)
+
+	if (s->codec==CODEC_ID_MPEG4
+#if HAVE_AVCODEC_SNOW
+	|| s->codec==CODEC_ID_SNOW
+#endif
+	)
 	{
 		mpeg4_fragment_and_send(f,s,frame,timestamp);
 		return;
@@ -765,50 +792,55 @@ static void split_and_send(MSFilter *f, EncState *s, mblk_t *frame){
 
 static void process_frame(MSFilter *f, mblk_t *inm){
 	EncState *s=(EncState*)f->data;
-	AVFrame pict;
+
 	AVCodecContext *c=&s->av_context;
-	int error;
+	int error,got_packet;
 	mblk_t *comp_buf=s->comp_buf;
 	int comp_buf_sz=comp_buf->b_datap->db_lim-comp_buf->b_datap->db_base;
 	YuvBuf yuv;
+	struct AVPacket packet;
+	memset(&packet, 0, sizeof(packet));
 
 	ms_yuv_buf_init_from_mblk(&yuv, inm);
 	/* convert image if necessary */
-	avcodec_get_frame_defaults(&pict);
-	avpicture_fill((AVPicture*)&pict,yuv.planes[0],c->pix_fmt,c->width,c->height);
-	
+	av_frame_unref(s->pict);
+	avpicture_fill((AVPicture*)s->pict,yuv.planes[0],c->pix_fmt,c->width,c->height);
+
 	/* timestamp used by ffmpeg, unset here */
-	pict.pts=AV_NOPTS_VALUE;
-		
-	if (video_starter_need_i_frame (&s->starter,f->ticker->time)){
+	s->pict->pts=AV_NOPTS_VALUE;
+
+	if (ms_video_starter_need_i_frame (&s->starter, f->ticker->time)){
 		/*sends an I frame at 2 seconds and 4 seconds after the beginning of the call*/
 		s->req_vfu=TRUE;
 	}
 	if (s->req_vfu){
-		pict.pict_type=FF_I_TYPE;
+		s->pict->pict_type=FF_I_TYPE;
 		s->req_vfu=FALSE;
 	}
 	comp_buf->b_rptr=comp_buf->b_wptr=comp_buf->b_datap->db_base;
+#if HAVE_AVCODEC_SNOW
 	if (s->codec==CODEC_ID_SNOW){
 		//prepend picture size
-		uint32_t header=((s->vsize.width&0xffff)<<16) | (s->vsize.height&0xffff);
+		uint32_t header=((s->vconf.vsize.width&0xffff)<<16) | (s->vconf.vsize.height&0xffff);
 		*(uint32_t*)comp_buf->b_wptr=htonl(header);
 		comp_buf->b_wptr+=4;
 		comp_buf_sz-=4;
 	}
+#endif
+	packet.data=comp_buf->b_wptr;
+	packet.size=comp_buf_sz;
+	error=avcodec_encode_video2(c, &packet, s->pict, &got_packet);
 
-	error=avcodec_encode_video(c, (uint8_t*)comp_buf->b_wptr,comp_buf_sz, &pict);
-
-	if (error<=0) ms_warning("ms_AVencoder_process: error %i.",error);
-	else{
+	if (error<0) ms_warning("ms_AVencoder_process: error %i.",error);
+	else if (got_packet){
 		s->framenum++;
 		if (s->framenum==1){
-			video_starter_first_frame (&s->starter,f->ticker->time);
+			ms_video_starter_first_frame(&s->starter, f->ticker->time);
 		}
 		if (c->coded_frame->pict_type==FF_I_TYPE){
 			ms_message("Emitting I-frame");
 		}
-		comp_buf->b_wptr+=error;
+		comp_buf->b_wptr+=packet.size;
 		split_and_send(f,s,comp_buf);
 	}
 	freemsg(inm);
@@ -829,88 +861,112 @@ static void enc_process(MSFilter *f){
 }
 
 
-static int enc_get_br(MSFilter *f, void *arg){
-	EncState *s=(EncState*)f->data;
-	*(int*)arg=s->maxbr;
-	return 0;
-}
+static int enc_set_configuration(MSFilter *f, void *data) {
+	EncState *s = (EncState *)f->data;
+	const MSVideoConfiguration *vconf = (const MSVideoConfiguration *)data;
+	if (vconf != &s->vconf) memcpy(&s->vconf, vconf, sizeof(MSVideoConfiguration));
 
-static int enc_set_br(MSFilter *f, void *arg){
-	EncState *s=(EncState*)f->data;
-	bool_t snow=s->codec==CODEC_ID_SNOW;
-	s->maxbr=*(int*)arg;
-	if (s->av_context.codec!=NULL){
-		/*when we are processing, apply new settings immediately*/
+	if (s->vconf.required_bitrate > s->vconf.bitrate_limit)
+		s->vconf.required_bitrate = s->vconf.bitrate_limit;
+	if (s->av_context.codec != NULL) {
+		/* When we are processing, apply new settings immediately */
 		ms_filter_lock(f);
 		enc_postprocess(f);
 		enc_preprocess(f);
 		ms_filter_unlock(f);
 		return 0;
 	}
-	if (s->maxbr>=1024000 && s->codec!=CODEC_ID_H263P){
-		if (s->codec!=CODEC_ID_H263P){
-			s->vsize.width = MS_VIDEO_SIZE_SVGA_W;
-			s->vsize.height = MS_VIDEO_SIZE_SVGA_H;
-		}else{
-			s->vsize.width = MS_VIDEO_SIZE_4CIF_W;
-			s->vsize.height = MS_VIDEO_SIZE_4CIF_H;
-		}
-		s->fps=25;
-	}else if (s->maxbr>=800000 ){
-		if (s->codec!=CODEC_ID_H263P){
-			s->vsize.width = MS_VIDEO_SIZE_VGA_W;
-			s->vsize.height = MS_VIDEO_SIZE_VGA_H;
-		}else{
-			s->vsize.width = MS_VIDEO_SIZE_4CIF_W;
-			s->vsize.height = MS_VIDEO_SIZE_4CIF_H;
-		}
-		s->fps=25;
-	}else if (s->maxbr>=512000){
-		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
-		s->fps=25;
-	}else if (s->maxbr>=256000){
-		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
-		s->fps=17;
-		s->qmin=3;
-	}else if (s->maxbr>=170000 && s->codec!=CODEC_ID_H263P && s->codec!=CODEC_ID_H263){
-		s->vsize.width=MS_VIDEO_SIZE_QVGA_W;
-		s->vsize.height=MS_VIDEO_SIZE_QVGA_H;
-		s->fps=15;
-		s->qmin=3;
-	}else if (s->maxbr>=128000){
-		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=10;
-		s->qmin=3;
-	}else if (s->maxbr>=64000){
-		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=snow ? 7 : 5;
-		s->qmin=snow ? 4 : 5;
-	}else{
-		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=5;
-		s->qmin=5;
+
+	if (vconf->extra != NULL) {
+		s->qmin = *((int *)vconf->extra);
+	}
+	ms_message("Video configuration set: bitrate=%dbits/s, fps=%f, vsize=%dx%d for encoder [%p]", s->vconf.required_bitrate
+																								, s->vconf.fps
+																								, s->vconf.vsize.width
+																								, s->vconf.vsize.height
+																								, f);
+	return 0;
+}
+
+static int enc_set_fps(MSFilter *f, void *arg){
+	EncState *s=(EncState*)f->data;
+	s->vconf.fps=*(float*)arg;
+	enc_set_configuration(f, &s->vconf);
+	return 0;
+}
+
+static int enc_get_fps(MSFilter *f, void *arg){
+	EncState *s=(EncState*)f->data;
+	*(float*)arg=s->vconf.fps;
+	return 0;
+}
+
+static int enc_set_vsize(MSFilter *f, void *arg) {
+	MSVideoConfiguration best_vconf;
+	MSVideoSize *vs = (MSVideoSize *)arg;
+	EncState *s=(EncState*)f->data;
+	best_vconf = ms_video_find_best_configuration_for_size(s->vconf_list, *vs, ms_get_cpu_count());
+	s->vconf.vsize = *vs;
+	s->vconf.fps = best_vconf.fps;
+	s->vconf.bitrate_limit = best_vconf.bitrate_limit;
+	enc_set_configuration(f, &s->vconf);
+	return 0;
+}
+
+static int enc_get_vsize(MSFilter *f,void *arg){
+	EncState *s=(EncState*)f->data;
+	*(MSVideoSize*)arg=s->vconf.vsize;
+	return 0;
+}
+
+static int enc_set_mtu(MSFilter *f,void *arg){
+	EncState *s=(EncState*)f->data;
+	s->mtu=*(int*)arg;
+	return 0;
+}
+
+static int enc_get_br(MSFilter *f, void *arg){
+	EncState *s=(EncState*)f->data;
+	*(int*)arg=s->vconf.required_bitrate;
+	return 0;
+}
+
+static int enc_set_br(MSFilter *f, void *arg) {
+	EncState *s = (EncState *)f->data;
+	int br = *(int *)arg;
+	if (s->av_context.codec != NULL) {
+		/* Encoding is already ongoing, do not change video size, only bitrate. */
+		s->vconf.required_bitrate = br;
+		enc_set_configuration(f, &s->vconf);
+	} else {
+		MSVideoConfiguration best_vconf = ms_video_find_best_configuration_for_bitrate(s->vconf_list, br, ms_get_cpu_count());
+		enc_set_configuration(f, &best_vconf);
 	}
 	return 0;
 }
 
+static int enc_get_configuration_list(MSFilter *f, void *data) {
+	EncState *s = (EncState *)f->data;
+	const MSVideoConfiguration **vconf_list = (const MSVideoConfiguration **)data;
+	*vconf_list = s->vconf_list;
+	return 0;
+}
 
-static MSFilterMethod methods[]={
-	{	MS_FILTER_SET_FPS	,	enc_set_fps	},
-	{	MS_FILTER_GET_FPS	,	enc_get_fps	},
-	{	MS_FILTER_SET_VIDEO_SIZE ,	enc_set_vsize },
-	{	MS_FILTER_GET_VIDEO_SIZE ,	enc_get_vsize },
-	{	MS_FILTER_ADD_FMTP	,	enc_add_fmtp },
-	{	MS_FILTER_SET_BITRATE	,	enc_set_br	},
-	{	MS_FILTER_GET_BITRATE	,	enc_get_br	},
-	{	MS_FILTER_SET_MTU	,	enc_set_mtu	},
-	{	MS_FILTER_REQ_VFU	,	enc_req_vfu	},
-	{	MS_VIDEO_ENCODER_REQ_VFU,	enc_req_vfu	},
-	{	0			,	NULL	}
+
+static MSFilterMethod methods[] = {
+	{ MS_FILTER_SET_FPS,                       enc_set_fps                },
+	{ MS_FILTER_GET_FPS,                       enc_get_fps                },
+	{ MS_FILTER_SET_VIDEO_SIZE,                enc_set_vsize              },
+	{ MS_FILTER_GET_VIDEO_SIZE,                enc_get_vsize              },
+	{ MS_FILTER_ADD_FMTP,                      enc_add_fmtp               },
+	{ MS_FILTER_SET_BITRATE,                   enc_set_br                 },
+	{ MS_FILTER_GET_BITRATE,                   enc_get_br                 },
+	{ MS_FILTER_SET_MTU,                       enc_set_mtu                },
+	{ MS_FILTER_REQ_VFU,                       enc_req_vfu                },
+	{ MS_VIDEO_ENCODER_REQ_VFU,                enc_req_vfu                },
+	{ MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, enc_get_configuration_list },
+	{ MS_VIDEO_ENCODER_SET_CONFIGURATION,      enc_set_configuration      },
+	{ 0,                                       NULL                       }
 };
 
 #ifdef _MSC_VER
@@ -962,7 +1018,7 @@ MSFilterDesc ms_mpeg4_enc_desc={
 	enc_uninit,
 	methods
 };
-
+#if HAVE_AVCODEC_SNOW
 MSFilterDesc ms_snow_enc_desc={
 	MS_SNOW_ENC_ID,
 	"MSSnowEnc",
@@ -978,7 +1034,7 @@ MSFilterDesc ms_snow_enc_desc={
 	enc_uninit,
 	methods
 };
-
+#endif
 MSFilterDesc ms_mjpeg_enc_desc={
 	MS_JPEG_ENC_ID,
 	"MSJpegEnc",
@@ -1044,7 +1100,7 @@ MSFilterDesc ms_mpeg4_enc_desc={
 	.uninit=enc_uninit,
 	.methods=methods
 };
-
+#if HAVE_AVCODEC_SNOW
 MSFilterDesc ms_snow_enc_desc={
 	.id=MS_SNOW_ENC_ID,
 	.name="MSSnowEnc",
@@ -1064,7 +1120,7 @@ MSFilterDesc ms_snow_enc_desc={
 	.uninit=enc_uninit,
 	.methods=methods
 };
-
+#endif
 MSFilterDesc ms_mjpeg_enc_desc={
 	.id=MS_JPEG_ENC_ID,
 	.name="MSMJpegEnc",
@@ -1083,19 +1139,21 @@ MSFilterDesc ms_mjpeg_enc_desc={
 
 #endif
 
-void __register_ffmpeg_encoders_if_possible(void){
+void __register_ffmpeg_encoders_if_possible(MSFactory *obj){
 	ms_ffmpeg_check_init();
-	if (avcodec_find_encoder(CODEC_ID_MPEG4))
-		ms_filter_register(&ms_mpeg4_enc_desc);
-	if (avcodec_find_encoder(CODEC_ID_H263)){
-		ms_filter_register(&ms_h263_enc_desc);
-		ms_filter_register(&ms_h263_old_enc_desc);
+	if (avcodec_find_encoder(CODEC_ID_MPEG4) && HAVE_NON_FREE_CODECS)
+		ms_factory_register_filter(obj,&ms_mpeg4_enc_desc);
+	if (avcodec_find_encoder(CODEC_ID_H263) && HAVE_NON_FREE_CODECS){
+		ms_factory_register_filter(obj,&ms_h263_enc_desc);
+		ms_factory_register_filter(obj,&ms_h263_old_enc_desc);
 	}
+#if HAVE_AVCODEC_SNOW
 	if (avcodec_find_encoder(CODEC_ID_SNOW))
-		ms_filter_register(&ms_snow_enc_desc);
+		ms_factory_register_filter(obj,&ms_snow_enc_desc);
+#endif
 	if (avcodec_find_encoder(CODEC_ID_MJPEG))
 	{
-		ms_filter_register(&ms_mjpeg_enc_desc);
+		ms_factory_register_filter(obj,&ms_mjpeg_enc_desc);
 	}
 }
 
