@@ -17,7 +17,6 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#include "mediastreamer-config.h"
 
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/dtmfgen.h"
@@ -25,19 +24,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilerec.h"
 #include "mediastreamer2/msrtp.h"
 #include "mediastreamer2/mstonedetector.h"
-#include "private.h"
 #include "mediastreamer2_tester.h"
 #include "mediastreamer2_tester_private.h"
+#include "private.h"
 
 #include <stdio.h>
 #include "CUnit/Basic.h"
 
 
-#ifdef _MSC_VER
-#define unlink _unlink
-#endif
-
-
 static int basic_audio_tester_init(void) {
 	ms_init();
 	ms_filter_enable_statistics(TRUE);
@@ -58,11 +52,13 @@ static void tone_detected_cb(void *data, MSFilter *f, unsigned int event_id, MST
 static void dtmfgen_tonedet(void) {
 	MSConnectionHelper h;
 	unsigned int filter_mask = FILTER_MASK_VOIDSOURCE | FILTER_MASK_DTMFGEN | FILTER_MASK_TONEDET | FILTER_MASK_VOIDSINK;
+	bool_t send_silence = TRUE;
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
 	ms_tester_create_filters(filter_mask);
-	ms_filter_set_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL);
+	ms_filter_add_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL,TRUE);
+	ms_filter_call_method(ms_tester_voidsource, MS_VOID_SOURCE_SEND_SILENCE, &send_silence);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_voidsource, -1, 0);
 	ms_connection_helper_link(&h, ms_tester_dtmfgen, 0, 0);
@@ -91,6 +87,7 @@ static void dtmfgen_enc_dec_tonedet(char *mime, int sample_rate, int nchannels)
 
 	unsigned int filter_mask = FILTER_MASK_VOIDSOURCE | FILTER_MASK_DTMFGEN | FILTER_MASK_ENCODER
 		| FILTER_MASK_DECODER | FILTER_MASK_TONEDET | FILTER_MASK_VOIDSINK;
+	bool_t send_silence = TRUE;
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
@@ -100,6 +97,7 @@ static void dtmfgen_enc_dec_tonedet(char *mime, int sample_rate, int nchannels)
 	/* set sample rate and channel number to all filters (might need to check the return value to insert a resampler if needed?) */
 	ms_filter_call_method(ms_tester_voidsource, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
 	ms_filter_call_method(ms_tester_voidsource, MS_FILTER_SET_NCHANNELS, &nchannels);
+	ms_filter_call_method(ms_tester_voidsource, MS_VOID_SOURCE_SEND_SILENCE, &send_silence);
 	ms_filter_call_method(ms_tester_dtmfgen, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
 	ms_filter_call_method(ms_tester_dtmfgen, MS_FILTER_SET_NCHANNELS, &nchannels);
 	ms_filter_call_method(ms_tester_encoder, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
@@ -112,7 +110,7 @@ static void dtmfgen_enc_dec_tonedet(char *mime, int sample_rate, int nchannels)
 	ms_filter_call_method(ms_tester_voidsink, MS_FILTER_SET_NCHANNELS, &nchannels);
 
 
-	ms_filter_set_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL);
+	ms_filter_add_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL,TRUE);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_voidsource, -1, 0);
 	ms_connection_helper_link(&h, ms_tester_dtmfgen, 0, 0);
@@ -159,18 +157,20 @@ static void dtmfgen_enc_rtp_dec_tonedet(void) {
 	RtpSession *rtps;
 	unsigned int filter_mask = FILTER_MASK_VOIDSOURCE | FILTER_MASK_DTMFGEN | FILTER_MASK_ENCODER
 		| FILTER_MASK_RTPSEND | FILTER_MASK_RTPRECV | FILTER_MASK_DECODER | FILTER_MASK_TONEDET | FILTER_MASK_VOIDSINK;
+	bool_t send_silence = TRUE;
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
 	ms_tester_codec_mime = "pcmu";
 	ms_tester_create_filters(filter_mask);
-	ms_filter_set_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL);
-	rtps = create_duplex_rtpsession(50060, 0, FALSE);
+	ms_filter_add_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL,TRUE);
+	rtps = create_duplex_rtpsession("0.0.0.0", 50060, 0);
 	rtp_session_set_remote_addr_full(rtps, "127.0.0.1", 50060, "127.0.0.1", 50061);
 	rtp_session_set_payload_type(rtps, 8);
 	rtp_session_enable_rtcp(rtps,FALSE);
 	ms_filter_call_method(ms_tester_rtprecv, MS_RTP_RECV_SET_SESSION, rtps);
 	ms_filter_call_method(ms_tester_rtpsend, MS_RTP_SEND_SET_SESSION, rtps);
+	ms_filter_call_method(ms_tester_voidsource, MS_VOID_SOURCE_SEND_SILENCE, &send_silence);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_voidsource, -1, 0);
 	ms_connection_helper_link(&h, ms_tester_dtmfgen, 0, 0);
@@ -203,22 +203,25 @@ static void dtmfgen_enc_rtp_dec_tonedet(void) {
 	rtp_session_destroy(rtps);
 }
 
-#define DTMFGEN_FILE_NAME WRITE_FILE_PATH "dtmfgen_file.raw"
+#define DTMFGEN_FILE_NAME  "dtmfgen_file.raw"
 
 static void dtmfgen_filerec_fileplay_tonedet(void) {
 	MSConnectionHelper h;
 	unsigned int filter_mask = FILTER_MASK_VOIDSOURCE | FILTER_MASK_DTMFGEN | FILTER_MASK_FILEREC
 		| FILTER_MASK_FILEPLAY | FILTER_MASK_TONEDET | FILTER_MASK_VOIDSINK;
+	bool_t send_silence = TRUE;
+	char* recorded_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_writable_dir(), DTMFGEN_FILE_NAME);
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
 	ms_tester_create_filters(filter_mask);
-	ms_filter_set_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL);
+	ms_filter_add_notify_callback(ms_tester_tonedet, (MSFilterNotifyFunc)tone_detected_cb, NULL,TRUE);
 
 	// Generate tones and save them to a file
 	ms_filter_call_method_noarg(ms_tester_filerec, MS_FILE_REC_CLOSE);
-	ms_filter_call_method(ms_tester_filerec, MS_FILE_REC_OPEN, DTMFGEN_FILE_NAME);
+	ms_filter_call_method(ms_tester_filerec, MS_FILE_REC_OPEN, recorded_file);
 	ms_filter_call_method_noarg(ms_tester_filerec, MS_FILE_REC_START);
+	ms_filter_call_method(ms_tester_voidsource, MS_VOID_SOURCE_SEND_SILENCE, &send_silence);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_voidsource, -1, 0);
 	ms_connection_helper_link(&h, ms_tester_dtmfgen, 0, 0);
@@ -234,7 +237,7 @@ static void dtmfgen_filerec_fileplay_tonedet(void) {
 
 	// Read the previous file and detect the tones
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
-	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, DTMFGEN_FILE_NAME);
+	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, recorded_file);
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_START);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_fileplay, -1, 0);
@@ -251,7 +254,8 @@ static void dtmfgen_filerec_fileplay_tonedet(void) {
 	ms_filter_log_statistics();
 	ms_tester_destroy_filters(filter_mask);
 	ms_tester_destroy_ticker();
-	unlink(DTMFGEN_FILE_NAME);
+	unlink(recorded_file);
+    ms_free(recorded_file);
 }
 
 
