@@ -23,7 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilerec.h"
 #include "mediastreamer2/msrtp.h"
 #include "mediastreamer2/mstonedetector.h"
-#include "private.h"
 #include "mediastreamer2_tester.h"
 #include "mediastreamer2_tester_private.h"
 
@@ -63,12 +62,14 @@ static void dtmfgen_soundwrite(void) {
 	unsigned int filter_mask = FILTER_MASK_VOIDSOURCE | FILTER_MASK_DTMFGEN | FILTER_MASK_SOUNDWRITE;
 	int sample_rate = 8000;
 	int nchannels = 1;
+	bool_t send_silence = TRUE;
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
 	ms_tester_create_filters(filter_mask);
 	ms_filter_call_method(ms_tester_voidsource, MS_FILTER_SET_BITRATE, &sample_rate);
 	ms_filter_call_method(ms_tester_voidsource, MS_FILTER_SET_NCHANNELS, &nchannels);
+	ms_filter_call_method(ms_tester_voidsource, MS_VOID_SOURCE_SEND_SILENCE, &send_silence);
 	ms_filter_call_method(ms_tester_dtmfgen, MS_FILTER_SET_BITRATE, &sample_rate);
 	ms_filter_call_method(ms_tester_dtmfgen, MS_FILTER_SET_NCHANNELS, &nchannels);
 	if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_BITRATE, &sample_rate) != 0) {
@@ -108,14 +109,14 @@ static void dtmfgen_soundwrite(void) {
 	ms_tester_destroy_ticker();
 }
 
-#define CHIMES_48000_STEREO_FILE_NAME		SOUND_FILE_PATH "chimes_48000_stereo.wav"
-#define BIRD_44100_STEREO_FILE_NAME			SOUND_FILE_PATH "bird_44100_stereo.wav"
-#define PUNCH_16000_STEREO_FILE_NAME		SOUND_FILE_PATH "punch_16000_stereo.wav"
-#define PIANO_8000_STEREO_FILE_NAME			SOUND_FILE_PATH "piano_8000_stereo.wav"
-#define NYLON_48000_MONO_FILE_NAME			SOUND_FILE_PATH "nylon_48000_mono.wav"
-#define OWL_44100_MONO_FILE_NAME			SOUND_FILE_PATH "owl_44100_mono.wav"
-#define LASERROCKET_16000_MONO_FILE_NAME	SOUND_FILE_PATH "laserrocket_16000_mono.wav"
-#define ARPEGGIO_8000_MONO_FILE_NAME		SOUND_FILE_PATH "arpeggio_8000_mono.wav"
+#define CHIMES_48000_STEREO_FILE_NAME		"sounds/chimes_48000_stereo.wav"
+#define BIRD_44100_STEREO_FILE_NAME			"sounds/bird_44100_stereo.wav"
+#define PUNCH_16000_STEREO_FILE_NAME		"sounds/punch_16000_stereo.wav"
+#define PIANO_8000_STEREO_FILE_NAME			"sounds/piano_8000_stereo.wav"
+#define NYLON_48000_MONO_FILE_NAME			"sounds/nylon_48000_mono.wav"
+#define OWL_44100_MONO_FILE_NAME			"sounds/owl_44100_mono.wav"
+#define LASERROCKET_16000_MONO_FILE_NAME	"sounds/laserrocket_16000_mono.wav"
+#define ARPEGGIO_8000_MONO_FILE_NAME		"sounds/arpeggio_8000_mono.wav"
 
 static void fileplay_eof(void *user_data, MSFilter *f, unsigned int event, void *event_data) {
 	if (event == MS_FILE_PLAYER_EOF) {
@@ -132,11 +133,12 @@ static void fileplay_soundwrite(const char *filename) {
 	int sample_rate = 8000;
 	int nchannels = 1;
 	int done = FALSE;
+    int elapsed = 0;
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
 	ms_tester_create_filters(filter_mask);
-	ms_filter_set_notify_callback(ms_tester_fileplay, fileplay_eof, &done);
+	ms_filter_add_notify_callback(ms_tester_fileplay, fileplay_eof, &done,TRUE);
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
 	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, (void *)filename);
 	ms_filter_call_method(ms_tester_fileplay, MS_FILTER_GET_SAMPLE_RATE, &sample_rate);
@@ -164,7 +166,9 @@ static void fileplay_soundwrite(const char *filename) {
 	ms_connection_helper_link(&h, ms_tester_soundwrite, 0, -1);
 	ms_ticker_attach(ms_tester_ticker, ms_tester_fileplay);
 
-	while (done != TRUE) {
+
+	while (done != TRUE && elapsed < 6000) {
+        elapsed++;
 		ms_usleep(10000);
 	}
 
@@ -184,36 +188,42 @@ static void fileplay_soundwrite(const char *filename) {
 	ms_tester_destroy_ticker();
 }
 
+static void fileplay_soundwrite_from_file(const char *filepath){
+    char* file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_file_root(), filepath);
+    fileplay_soundwrite(file);
+    ms_free(file);
+}
+
 static void fileplay_soundwrite_48000_stereo(void) {
-	fileplay_soundwrite(CHIMES_48000_STEREO_FILE_NAME);
+	fileplay_soundwrite_from_file(CHIMES_48000_STEREO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_44100_stereo(void) {
-	fileplay_soundwrite(BIRD_44100_STEREO_FILE_NAME);
+	fileplay_soundwrite_from_file(BIRD_44100_STEREO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_16000_stereo(void) {
-	fileplay_soundwrite(PUNCH_16000_STEREO_FILE_NAME);
+	fileplay_soundwrite_from_file(PUNCH_16000_STEREO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_8000_stereo(void) {
-	fileplay_soundwrite(PIANO_8000_STEREO_FILE_NAME);
+	fileplay_soundwrite_from_file(PIANO_8000_STEREO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_48000_mono(void) {
-	fileplay_soundwrite(NYLON_48000_MONO_FILE_NAME);
+	fileplay_soundwrite_from_file(NYLON_48000_MONO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_44100_mono(void) {
-	fileplay_soundwrite(OWL_44100_MONO_FILE_NAME);
+	fileplay_soundwrite_from_file(OWL_44100_MONO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_16000_mono(void) {
-	fileplay_soundwrite(LASERROCKET_16000_MONO_FILE_NAME);
+	fileplay_soundwrite_from_file(LASERROCKET_16000_MONO_FILE_NAME);
 }
 
 static void fileplay_soundwrite_8000_mono(void) {
-	fileplay_soundwrite(ARPEGGIO_8000_MONO_FILE_NAME);
+	fileplay_soundwrite_from_file(ARPEGGIO_8000_MONO_FILE_NAME);
 }
 
 static void soundread_soundwrite(void) {
@@ -228,9 +238,9 @@ static void soundread_soundwrite(void) {
 	ms_tester_create_filters(filter_mask);
 	ms_filter_call_method(ms_tester_soundread, MS_FILTER_GET_SAMPLE_RATE, &sample_rate);
 	ms_filter_call_method(ms_tester_soundread, MS_FILTER_GET_NCHANNELS, &nchannels);
-	if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_BITRATE, &sample_rate) != 0) {
+	if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_SAMPLE_RATE, &sample_rate) != 0) {
 		int soundwrite_sample_rate = 48000;
-		ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_GET_BITRATE, &soundwrite_sample_rate);
+		ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_GET_SAMPLE_RATE, &soundwrite_sample_rate);
 		if (sample_rate != soundwrite_sample_rate) need_resampler = TRUE;
 	}
 	if (ms_filter_call_method(ms_tester_soundwrite, MS_FILTER_SET_NCHANNELS, &nchannels) != 0) {
@@ -347,7 +357,7 @@ static void soundread_speexenc_speexdec_soundwrite(void) {
 	ms_tester_destroy_ticker();
 }
 
-#define SOUNDREAD_FILE_NAME WRITE_FILE_PATH "soundread_file.raw"
+#define SOUNDREAD_FILE_NAME "soundread_file.raw"
 
 static void soundread_filerec_fileplay_soundwrite(void) {
 	MSConnectionHelper h;
@@ -356,6 +366,7 @@ static void soundread_filerec_fileplay_soundwrite(void) {
 	int playback_sample_rate = 8000;
 	int capture_nchannels = 1;
 	int playback_nchannels = 1;
+    char *writable_filename = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_writable_dir(), SOUNDREAD_FILE_NAME);
 
 	ms_filter_reset_statistics();
 	ms_tester_create_ticker();
@@ -367,7 +378,7 @@ static void soundread_filerec_fileplay_soundwrite(void) {
 	ms_filter_call_method(ms_tester_filerec, MS_FILTER_SET_SAMPLE_RATE, &capture_sample_rate);
 	ms_filter_call_method(ms_tester_filerec, MS_FILTER_SET_NCHANNELS, &capture_nchannels);
 	ms_filter_call_method_noarg(ms_tester_filerec, MS_FILE_REC_CLOSE);
-	ms_filter_call_method(ms_tester_filerec, MS_FILE_REC_OPEN, SOUNDREAD_FILE_NAME);
+	ms_filter_call_method(ms_tester_filerec, MS_FILE_REC_OPEN, writable_filename);
 	ms_filter_call_method_noarg(ms_tester_filerec, MS_FILE_REC_START);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_soundread, -1, 0);
@@ -389,7 +400,7 @@ static void soundread_filerec_fileplay_soundwrite(void) {
 		ms_tester_create_filter(&ms_tester_resampler, MS_RESAMPLE_ID);
 	}
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_CLOSE);
-	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, SOUNDREAD_FILE_NAME);
+	ms_filter_call_method(ms_tester_fileplay, MS_FILE_PLAYER_OPEN, writable_filename);
 	ms_filter_call_method_noarg(ms_tester_fileplay, MS_FILE_PLAYER_START);
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h, ms_tester_fileplay, -1, 0);
@@ -417,7 +428,8 @@ static void soundread_filerec_fileplay_soundwrite(void) {
 	ms_tester_destroy_filters(filter_mask);
 	ms_tester_destroy_ticker();
 
-	unlink(SOUNDREAD_FILE_NAME);
+	unlink(writable_filename);
+    ms_free(writable_filename);
 }
 
 
