@@ -22,20 +22,10 @@
 #include "private.h"
 #include "liblinphone_tester.h"
 
-static void auth_info_requested2(LinphoneCore *lc, const char *realm, const char *username, const char *domain) {
-	stats* counters;
 
-	ms_message("Auth info requested  for user id [%s] at realm [%s]\n"
-					,username
-					,realm);
-	counters = get_stats(lc);
-	counters->number_of_auth_info_requested++;
-
-}
 
 static void auth_info_requested(LinphoneCore *lc, const char *realm, const char *username, const char *domain) {
 	LinphoneAuthInfo *info;
-	auth_info_requested2(lc,realm,username,domain);
 	info=linphone_auth_info_new(test_username,NULL,test_password,NULL,realm,domain); /*create authentication structure from identity*/
 	linphone_core_add_auth_info(lc,info); /*add authentication info to LinphoneCore*/
 }
@@ -46,7 +36,9 @@ static LinphoneCoreManager* create_lcm_with_auth(unsigned int with_auth) {
 	LinphoneCoreManager* mgr=linphone_core_manager_new(NULL);
 	
 	if (with_auth) {
-		mgr->lc->vtable.auth_info_requested=auth_info_requested;
+		LinphoneCoreVTable* vtable = linphone_core_v_table_new();
+		vtable->auth_info_requested=auth_info_requested;
+		linphone_core_add_listener(mgr->lc,vtable);
 	}
 	
 	/*to allow testing with 127.0.0.1*/
@@ -119,7 +111,8 @@ static void register_with_refresh_base_3(LinphoneCore* lc
 	linphone_core_add_proxy_config(lc,proxy_cfg);
 	linphone_core_set_default_proxy(lc,proxy_cfg);
 
-	while (counters->number_of_LinphoneRegistrationOk<1+(refresh!=0) && retry++ <310) {
+	while (counters->number_of_LinphoneRegistrationOk<1+(refresh!=0)
+			&& retry++ <(110 /*only wait 11 s if final state is progress*/+(expected_final_state==LinphoneRegistrationProgress?0:200))) {
 		linphone_core_iterate(lc);
 		if (counters->number_of_auth_info_requested>0 && linphone_proxy_config_get_state(proxy_cfg) == LinphoneRegistrationFailed && late_auth_info) {
 			if (!linphone_core_get_auth_info_list(lc)) {
@@ -197,6 +190,71 @@ static void simple_register(){
 	linphone_core_manager_destroy(lcm);
 }
 
+static void register_with_custom_headers(void){
+	LinphoneCoreManager *marie=linphone_core_manager_new("marie_rc");
+	LinphoneProxyConfig *cfg=linphone_core_get_default_proxy_config(marie->lc);
+	int initial_register_ok=marie->stat.number_of_LinphoneRegistrationOk;
+	const char *value;
+	
+	linphone_core_set_network_reachable(marie->lc, FALSE);
+	linphone_proxy_config_set_custom_header(cfg, "ah-bah-ouais", "...mais bon.");
+	/*unfortunately it is difficult to programmatically check that sent custom headers are actually sent.
+	 * A server development would be required here.*/
+	
+	linphone_core_set_network_reachable(marie->lc, TRUE);
+	wait_for(marie->lc, NULL, &marie->stat.number_of_LinphoneRegistrationOk,initial_register_ok+1);
+	value=linphone_proxy_config_get_custom_header(cfg, "Server");
+	CU_ASSERT_PTR_NOT_NULL(value);
+	if (value) CU_ASSERT_TRUE(strstr(value, "Flexisip")!=NULL);
+	linphone_core_manager_destroy(marie);
+}
+
+static void simple_unregister(){
+	LinphoneCoreManager* lcm = create_lcm();
+	stats* counters = &lcm->stat;
+	LinphoneProxyConfig* proxy_config;
+	register_with_refresh_base(lcm->lc,FALSE,NULL,NULL);
+
+	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+
+	linphone_proxy_config_edit(proxy_config);
+	reset_counters(counters); /*clear stats*/
+
+	/*nothing is supposed to arrive until done*/
+	CU_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
+	linphone_proxy_config_enable_register(proxy_config,FALSE);
+	linphone_proxy_config_done(proxy_config);
+	CU_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1));
+	linphone_core_manager_destroy(lcm);
+}
+
+static void change_expires(){
+	LinphoneCoreManager* lcm = create_lcm();
+	stats* counters = &lcm->stat;
+	LinphoneProxyConfig* proxy_config;
+	register_with_refresh_base(lcm->lc,FALSE,NULL,NULL);
+
+	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+
+	linphone_proxy_config_edit(proxy_config);
+	reset_counters(counters); /*clear stats*/
+
+	/*nothing is supposed to arrive until done*/
+	CU_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
+
+	linphone_proxy_config_set_expires(proxy_config,3);
+
+	linphone_proxy_config_done(proxy_config);
+	CU_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,1));
+	/*wait 2s without receive refresh*/
+	CU_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,2,2000));
+	/* now, it should be ok*/
+	CU_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,2));
+
+
+	linphone_core_manager_destroy(lcm);
+}
+
 /*take care of min expires configuration from server*/
 static void simple_register_with_refresh() {
 	LinphoneCoreManager* lcm = create_lcm();
@@ -275,6 +333,7 @@ static void ha1_authenticated_register(){
 
 static void authenticated_register_with_no_initial_credentials(){
 	LinphoneCoreManager *mgr;
+	LinphoneCoreVTable* vtable = linphone_core_v_table_new();
 	stats* counters;
 	char route[256];
 	
@@ -282,7 +341,8 @@ static void authenticated_register_with_no_initial_credentials(){
 	
 	mgr = linphone_core_manager_new(NULL);
 	
-	mgr->lc->vtable.auth_info_requested=auth_info_requested;
+	vtable->auth_info_requested=auth_info_requested;
+	linphone_core_add_listener(mgr->lc,vtable);
 
 	counters= get_stats(mgr->lc);
 	counters->number_of_auth_info_requested=0;
@@ -332,8 +392,7 @@ static void authenticated_register_with_wrong_late_credentials(){
 	linphone_core_manager_destroy(mgr);
 }
 
-static void authenticated_register_with_wrong_credentials_with_params(const char* user_agent) {
-	LinphoneCoreManager *mgr;
+static void authenticated_register_with_wrong_credentials_with_params_base(const char* user_agent,LinphoneCoreManager *mgr) {
 	stats* counters;
 	LCSipTransports transport = {5070,5070,0,5071};
 	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,"wrong passwd",NULL,auth_domain,NULL); /*create authentication structure from identity*/
@@ -341,10 +400,6 @@ static void authenticated_register_with_wrong_credentials_with_params(const char
 	
 	sprintf(route,"sip:%s",test_route);
 	
-	mgr=linphone_core_manager_new(NULL);
-
-	mgr->lc->vtable.auth_info_requested=auth_info_requested2;
-
 	sal_set_refresher_retry_after(mgr->lc->sal,500);
 	if (user_agent) {
 		linphone_core_set_user_agent(mgr->lc,user_agent,NULL);
@@ -356,11 +411,49 @@ static void authenticated_register_with_wrong_credentials_with_params(const char
 	/*wait for retry*/
 	CU_ASSERT_TRUE(wait_for(mgr->lc,mgr->lc,&counters->number_of_auth_info_requested,4));
 	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,1);
+	
+	/*check the detailed error info */
+	if (!user_agent || strcmp(user_agent,"tester-no-403")!=0){
+		LinphoneProxyConfig *cfg=NULL;
+		linphone_core_get_default_proxy(mgr->lc,&cfg);
+		CU_ASSERT_PTR_NOT_NULL(cfg);
+		if (cfg){
+			const LinphoneErrorInfo *ei=linphone_proxy_config_get_error_info(cfg);
+			const char *phrase=linphone_error_info_get_phrase(ei);
+			CU_ASSERT_PTR_NOT_NULL(phrase);
+			if (phrase) CU_ASSERT_TRUE(strcmp(phrase,"Forbidden")==0);
+			CU_ASSERT_EQUAL(linphone_error_info_get_protocol_code(ei),403);
+			CU_ASSERT_PTR_NULL(linphone_error_info_get_details(ei));
+		}
+		
+	}
+	}
+static void authenticated_register_with_wrong_credentials_with_params(const char* user_agent) {
+	LinphoneCoreManager *mgr = linphone_core_manager_new(NULL);
+	authenticated_register_with_wrong_credentials_with_params_base(user_agent,mgr);
 	linphone_core_manager_destroy(mgr);
 }
 static void authenticated_register_with_wrong_credentials() {
 	authenticated_register_with_wrong_credentials_with_params(NULL);
 }
+static void authenticated_register_with_wrong_credentials_2() {
+	LinphoneCoreManager *mgr = linphone_core_manager_new(NULL);
+	stats* counters = get_stats(mgr->lc);
+	int current_in_progress;
+	LinphoneProxyConfig* proxy;
+
+	authenticated_register_with_wrong_credentials_with_params_base(NULL,mgr);
+
+	linphone_core_get_default_proxy(mgr->lc,&proxy);
+	/*Make sure registration attempts are stopped*/
+	linphone_proxy_config_edit(proxy);
+	linphone_proxy_config_enable_register(proxy,FALSE);
+	linphone_proxy_config_done(proxy);
+	current_in_progress=counters->number_of_LinphoneRegistrationProgress;
+	CU_ASSERT_FALSE(wait_for(mgr->lc,mgr->lc,&counters->number_of_LinphoneRegistrationProgress,current_in_progress+1));
+
+	linphone_core_manager_destroy(mgr);
+}
 static void authenticated_register_with_wrong_credentials_without_403() {
 	authenticated_register_with_wrong_credentials_with_params("tester-no-403");
 }
@@ -385,9 +478,12 @@ static void network_state_change(){
 	counters = get_stats(lc);
 	register_ok=counters->number_of_LinphoneRegistrationOk;
 	linphone_core_set_network_reachable(lc,FALSE);
+	CU_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_NetworkReachableFalse,1));
 	CU_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_LinphoneRegistrationNone,register_ok));
 	linphone_core_set_network_reachable(lc,TRUE);
+	CU_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_NetworkReachableTrue,1));
 	wait_for(lc,lc,&counters->number_of_LinphoneRegistrationOk,2*register_ok);
+
 	linphone_core_manager_destroy(mgr);
 }
 static int get_number_of_udp_proxy(const LinphoneCore* lc) {
@@ -432,6 +528,101 @@ static void transport_change(){
 	linphone_core_manager_destroy(mgr);
 }
 
+static void proxy_transport_change(){
+	LinphoneCoreManager* lcm = create_lcm();
+	stats* counters = &lcm->stat;
+	LinphoneProxyConfig* proxy_config;
+	LinphoneAddress* addr;
+	char* addr_as_string;
+	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
+	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
+
+	register_with_refresh_base(lcm->lc,FALSE,auth_domain,NULL);
+
+	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	reset_counters(counters); /*clear stats*/
+	linphone_proxy_config_edit(proxy_config);
+
+	CU_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
+	addr = linphone_address_new(linphone_proxy_config_get_addr(proxy_config));
+
+	if (LinphoneTransportTcp == linphone_address_get_transport(addr)) {
+		linphone_address_set_transport(addr,LinphoneTransportUdp);
+	} else {
+		linphone_address_set_transport(addr,LinphoneTransportTcp);
+	}
+	linphone_proxy_config_set_server_addr(proxy_config,addr_as_string=linphone_address_as_string(addr));
+
+	linphone_proxy_config_done(proxy_config);
+
+	CU_ASSERT(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,1));
+	/*as we change p[roxy server destination, we should'nt be notified about the clear*/
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationCleared,0);
+	ms_free(addr_as_string);
+	linphone_address_destroy(addr);
+	linphone_core_manager_destroy(lcm);
+
+}
+static void proxy_transport_change_with_wrong_port() {
+	LinphoneCoreManager* lcm = create_lcm();
+	stats* counters = &lcm->stat;
+	LinphoneProxyConfig* proxy_config;
+	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
+	char route[256];
+	LCSipTransports transport= {LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM};
+	sprintf(route,"sip:%s",test_route);
+
+	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
+
+	register_with_refresh_base_3(lcm->lc, FALSE, auth_domain, "sip2.linphone.org:5987", 0,transport,LinphoneRegistrationProgress);
+
+	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	linphone_proxy_config_edit(proxy_config);
+
+	CU_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
+	linphone_proxy_config_set_server_addr(proxy_config,route);
+	linphone_proxy_config_done(proxy_config);
+
+	CU_ASSERT(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,1));
+	/*as we change proxy server destination, we should'nt be notified about the clear*/
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationCleared,0);
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationOk,1);
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress,1);
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0);
+
+	linphone_core_manager_destroy(lcm);
+
+}
+
+static void proxy_transport_change_with_wrong_port_givin_up() {
+	LinphoneCoreManager* lcm = create_lcm();
+	stats* counters = &lcm->stat;
+	LinphoneProxyConfig* proxy_config;
+	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
+	char route[256];
+	LCSipTransports transport= {LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM};
+	sprintf(route,"sip:%s",test_route);
+
+	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
+
+	register_with_refresh_base_3(lcm->lc, FALSE, auth_domain, "sip2.linphone.org:5987", 0,transport,LinphoneRegistrationProgress);
+
+	linphone_core_get_default_proxy(lcm->lc,&proxy_config);
+	linphone_proxy_config_edit(proxy_config);
+
+	CU_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
+	linphone_proxy_config_enableregister(proxy_config,FALSE);
+	linphone_proxy_config_done(proxy_config);
+
+	CU_ASSERT(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1));
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationOk,0);
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress,1);
+	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0);
+
+	linphone_core_manager_destroy(lcm);
+
+}
+
 static void io_recv_error(){
 	LinphoneCoreManager *mgr;
 	LinphoneCore* lc;
@@ -504,7 +695,7 @@ static void io_recv_error_late_recovery(){
 	CU_ASSERT_TRUE(wait_for(lc,NULL,&counters->number_of_LinphoneRegistrationProgress,(register_ok-number_of_udp_proxy)+register_ok /*because 1 udp*/));
 	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0)
 
-	CU_ASSERT_TRUE(wait_for_list(lcs=ms_list_append(NULL,lc),&counters->number_of_LinphoneRegistrationFailed,(register_ok-number_of_udp_proxy),sal_get_refresher_retry_after(lc->sal)+1000));
+	CU_ASSERT_TRUE(wait_for_list(lcs=ms_list_append(NULL,lc),&counters->number_of_LinphoneRegistrationFailed,(register_ok-number_of_udp_proxy),sal_get_refresher_retry_after(lc->sal)+3000));
 
 	sal_set_recv_error(lc->sal, 1); /*reset*/
 	sal_set_send_error(lc->sal, 0);
@@ -532,16 +723,17 @@ static void io_recv_error_without_active_register(){
 	for (proxys=ms_list_copy(linphone_core_get_proxy_config_list(lc));proxys!=NULL;proxys=proxys->next) {
 		LinphoneProxyConfig* proxy_cfg=(LinphoneProxyConfig*)proxys->data;
 		linphone_proxy_config_edit(proxy_cfg);
-		linphone_proxy_config_enable_register(proxy_cfg,FALSE);
+		linphone_proxy_config_enableregister(proxy_cfg,FALSE);
 		linphone_proxy_config_done(proxy_cfg);
 	}
 	ms_list_free(proxys);
+	/*wait for unregistrations*/
 	CU_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_LinphoneRegistrationCleared,register_ok /*because 1 udp*/));
 
 	sal_set_recv_error(lc->sal, 0);
 
 	/*nothing should happen because no active registration*/
-	CU_ASSERT_FALSE(wait_for(lc,lc,&counters->number_of_LinphoneRegistrationProgress,2*(register_ok-number_of_udp_proxy) /*because 1 udp*/));
+	CU_ASSERT_FALSE(wait_for_until(lc,lc,&counters->number_of_LinphoneRegistrationProgress,2*(register_ok-number_of_udp_proxy) /*because 1 udp*/,3000));
 
 	CU_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0)
 
@@ -608,7 +800,7 @@ static void tls_alt_name_register(){
 	linphone_core_refresh_registers(mgr->lc);
 	CU_ASSERT_TRUE(wait_for(lc,lc,&mgr->stat.number_of_LinphoneRegistrationOk,1));
 	CU_ASSERT_EQUAL(mgr->stat.number_of_LinphoneRegistrationFailed,0);
-	linphone_core_destroy(mgr->lc);
+	linphone_core_manager_destroy(mgr);
 }
 
 static void tls_wildcard_register(){
@@ -626,9 +818,22 @@ static void tls_wildcard_register(){
 	linphone_core_destroy(mgr->lc);
 }
 
+static void redirect(){
+	char route[256];
+	LinphoneCoreManager* lcm;
+	LCSipTransports transport = {-1,0,0,0};
+	sprintf(route,"sip:%s:5064",test_route);
+	lcm = create_lcm();
+	linphone_core_set_user_agent(lcm->lc,"redirect",NULL);
+	register_with_refresh_base_2(lcm->lc,FALSE,test_domain,route,FALSE,transport);
+	linphone_core_manager_destroy(lcm);
+}
+
 test_t register_tests[] = {
 	{ "Simple register", simple_register },
+	{ "Simple register unregister", simple_unregister },
 	{ "TCP register", simple_tcp_register },
+	{ "Register with custom headers", register_with_custom_headers },
 	{ "TCP register compatibility mode", simple_tcp_register_compatibility_mode },
 	{ "TLS register", simple_tls_register },
 	{ "TLS register with alt. name certificate", tls_alt_name_register },
@@ -639,6 +844,7 @@ test_t register_tests[] = {
 	{ "Ha1 authenticated register", ha1_authenticated_register },
 	{ "Digest auth without initial credentials", authenticated_register_with_no_initial_credentials },
 	{ "Digest auth with wrong credentials", authenticated_register_with_wrong_credentials },
+	{ "Digest auth with wrong credentials, check if registration attempts are stopped", authenticated_register_with_wrong_credentials_2 },
 	{ "Digest auth with wrong credentials without 403", authenticated_register_with_wrong_credentials_without_403},
 	{ "Authenticated register with wrong late credentials", authenticated_register_with_wrong_late_credentials},
 	{ "Authenticated register with late credentials", authenticated_register_with_late_credentials },
@@ -646,12 +852,17 @@ test_t register_tests[] = {
 	{ "Authenticated register with refresh", simple_auth_register_with_refresh },
 	{ "Register with refresh and send error", register_with_refresh_with_send_error },
 	{ "Multi account", multiple_proxy },
-	{ "Transport change", transport_change },
+	{ "Transport changes", transport_change },
+	{ "Proxy transport changes", proxy_transport_change},
+	{ "Proxy transport changes with wrong address at first", proxy_transport_change_with_wrong_port},
+	{ "Proxy transport changes with wrong address, giving up",proxy_transport_change_with_wrong_port_givin_up},
+	{ "Change expires", change_expires},
 	{ "Network state change", network_state_change },
 	{ "Io recv error", io_recv_error },
 	{ "Io recv error with recovery", io_recv_error_retry_immediatly},
 	{ "Io recv error with late recovery", io_recv_error_late_recovery},
-	{ "Io recv error without active registration", io_recv_error_without_active_register}
+	{ "Io recv error without active registration", io_recv_error_without_active_register},
+	{ "Simple redirect", redirect}
 };
 
 test_suite_t register_test_suite = {
